@@ -1,14 +1,11 @@
 #
 # RGBMap.py -- color mapping
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
 import math
-import numpy
+import numpy as np
 
 from ginga.misc import Callback
 from ginga import ColorDist
@@ -56,7 +53,9 @@ class RGBPlanes(object):
         if order == self.order:
             return self.rgbarr
         l = [ self.get_slice(c) for c in order ]
-        return numpy.dstack(l)
+        return np.concatenate([ arr[..., np.newaxis]
+                                for arr in l ], axis=-1)
+
 
     def get_size(self):
         """Returns (height, width) tuple of slice size."""
@@ -85,6 +84,8 @@ class RGBMapper(Callback.Callbacks):
         self.carr = None
         self.sarr = None
         self.scale_pct = 1.0
+        self.maxv = 255
+        self.nptype = np.uint8
 
         # For scaling algorithms
         hashsize = 65536
@@ -117,15 +118,15 @@ class RGBMapper(Callback.Callbacks):
 
     def calc_cmap(self):
         clst = self.cmap.clst
-        arr = numpy.array(clst).transpose() * 255.0
-        self.carr = numpy.round(arr).astype('uint8')
+        arr = np.array(clst).transpose() * float(self.maxv)
+        self.carr = np.round(arr).astype(self.nptype)
 
     def invert_cmap(self, callback=True):
-        self.carr = numpy.fliplr(self.carr)
+        self.carr = np.fliplr(self.carr)
         self.recalc(callback=callback)
 
     def rotate_cmap(self, num, callback=True):
-        self.carr = numpy.roll(self.carr, num, axis=1)
+        self.carr = np.roll(self.carr, num, axis=1)
         self.recalc(callback=callback)
 
     def restore_cmap(self, callback=True):
@@ -135,19 +136,19 @@ class RGBMapper(Callback.Callbacks):
 
     def get_rgb(self, index):
         """
-        Return a tuple of (R, G, B) values in the 0-255 range associated
+        Return a tuple of (R, G, B) values in the 0-maxv range associated
         mapped by the value of `index`.
         """
         return tuple(self.arr[index])
 
     def get_rgbval(self, index):
         """
-        Return a tuple of (R, G, B) values in the 0-255 range associated
+        Return a tuple of (R, G, B) values in the 0-maxv range associated
         mapped by the value of `index`.
         """
-        assert (index >= 0) and (index < 256), \
-               RGBMapError("Index must be in range 0-255 !")
-        index = self.sarr[index].clip(0, 255)
+        assert (index >= 0) and (index <= self.maxv), \
+               RGBMapError("Index must be in range 0-%d !" % (self.maxv))
+        index = self.sarr[index].clip(0, self.maxv)
         return (self.arr[0][index],
                 self.arr[1][index],
                 self.arr[2][index])
@@ -170,19 +171,21 @@ class RGBMapper(Callback.Callbacks):
         return self.imap
 
     def calc_imap(self):
-        arr = numpy.array(self.imap.ilst) * 255.0
-        self.iarr = numpy.round(arr).astype('uint')
+        arr = np.array(self.imap.ilst) * float(self.maxv)
+        self.iarr = np.round(arr).astype(np.uint)
 
     def reset_sarr(self, callback=True):
-        self.sarr = numpy.arange(256)
+        maxlen = self.maxv + 1
+        self.sarr = np.arange(maxlen)
         self.scale_pct = 1.0
         if callback:
             self.make_callback('changed')
 
     def set_sarr(self, sarr, callback=True):
-        assert len(sarr) == 256, \
-               RGBMapError("shift map length %d != 256" % (len(sarr)))
-        self.sarr = sarr.astype('uint')
+        maxlen = self.maxv + 1
+        assert len(sarr) == maxlen, \
+               RGBMapError("shift map length %d != %d" % (len(sarr), maxlen))
+        self.sarr = sarr.astype(np.uint)
         self.scale_pct = 1.0
 
         if callback:
@@ -192,7 +195,7 @@ class RGBMapper(Callback.Callbacks):
         return self.sarr
 
     def recalc(self, callback=True):
-        self.arr = numpy.copy(self.carr)
+        self.arr = np.copy(self.carr)
         # Apply intensity map to rearrange colors
         if self.iarr is not None:
             idx = self.iarr
@@ -205,7 +208,6 @@ class RGBMapper(Callback.Callbacks):
         if callback:
             self.make_callback('changed')
 
-
     def get_hash_size(self):
         return self.dist.get_hash_size()
 
@@ -245,40 +247,63 @@ class RGBMapper(Callback.Callbacks):
         cs = cs.upper()
         return [ order.index(c) for c in cs ]
 
-    def _get_rgbarray(self, idx, rgbobj, image_order):
-        # NOTE: data is assumed to be in the range 0-255 at this point
+    def _get_rgbarray(self, idx, rgbobj, image_order=''):
+        # NOTE: data is assumed to be in the range 0-maxv at this point
         # but clip as a precaution
         # See NOTE [A]: idx is always an array calculated in the caller and
         #    discarded afterwards
-        # idx = idx.clip(0, 255)
-        idx.clip(0, 255, out=idx)
+        # idx = idx.clip(0, self.maxv)
+        idx.clip(0, self.maxv, out=idx)
 
         # run it through the shift array and clip the result
         # See NOTE [A]
-        # idx = self.sarr[idx].clip(0, 255)
+        # idx = self.sarr[idx].clip(0, self.maxv)
         idx = self.sarr[idx]
-        idx.clip(0, 255, out=idx)
+        idx.clip(0, self.maxv, out=idx)
 
         ri, gi, bi = self.get_order_indexes(rgbobj.get_order(), 'RGB')
         out = rgbobj.rgbarr
 
-        if len(idx.shape) == 2:
+        # change [A]
+        if (image_order is None) or (len(image_order) < 3):
             out[..., ri] = self.arr[0][idx]
             out[..., gi] = self.arr[1][idx]
             out[..., bi] = self.arr[2][idx]
         else:
+            # <== indexes already contain RGB info.
             rj, gj, bj = self.get_order_indexes(image_order, 'RGB')
             out[..., ri] = self.arr[0][idx[..., rj]]
             out[..., gi] = self.arr[1][idx[..., gj]]
             out[..., bi] = self.arr[2][idx[..., bj]]
 
-    def get_rgbarray(self, idx, out=None, order='RGB', image_order='RGB'):
+    def get_rgbarray(self, idx, out=None, order='RGB', image_order=''):
+        """
+        Parameters
+        ----------
+        idx : index array
+
+        out : output array or None
+            The output array.  If `None` one of the correct size and depth
+            will be created.
+
+        order : str
+            The order of the color planes in the output array (e.g. "ARGB")
+
+        image_order : str or None
+            The order of channels if indexes already contain RGB info.
+        """
         # prepare output array
         shape = idx.shape
         depth = len(order)
-        res_shape = (shape[0], shape[1], depth)
+
+        if (image_order is not None) and (len(image_order) > 1):
+            # indexes contain RGB axis, so omit this
+            res_shape = shape[:-1] + (depth, )
+        else:
+            res_shape = shape + (depth, )
+
         if out is None:
-            out = numpy.empty(res_shape, dtype=numpy.uint8, order='C')
+            out = np.empty(res_shape, dtype=self.nptype, order='C')
         else:
             # TODO: assertion check on shape of out
             assert res_shape == out.shape, \
@@ -290,11 +315,11 @@ class RGBMapper(Callback.Callbacks):
         # set alpha channel
         if res.hasAlpha:
             aa = res.get_slice('A')
-            aa.fill(255)
+            aa.fill(self.maxv)
 
         idx = self.get_hasharray(idx)
 
-        self._get_rgbarray(idx, res, image_order)
+        self._get_rgbarray(idx, res, image_order=image_order)
 
         return res
 
@@ -304,7 +329,7 @@ class RGBMapper(Callback.Callbacks):
     def _shift(self, sarr, pct, rotate=False):
         n = len(sarr)
         num = int(n * pct)
-        arr = numpy.roll(sarr, num)
+        arr = np.roll(sarr, num)
         if not rotate:
             if num > 0:
                 arr[0:num] = sarr[0]
@@ -317,7 +342,7 @@ class RGBMapper(Callback.Callbacks):
         new_wd = int(round(scale * old_wd))
 
         # Is there a more efficient way to do this?
-        xi = numpy.mgrid[0:new_wd]
+        xi = np.mgrid[0:new_wd]
         iscale_x = float(old_wd) / float(new_wd)
 
         xi = (xi * iscale_x).astype('int')
@@ -327,8 +352,9 @@ class RGBMapper(Callback.Callbacks):
 
     def shift(self, pct, rotate=False, callback=True):
         work = self._shift(self.sarr, pct, rotate=rotate)
-        assert len(work) == 256, \
-               RGBMapError("shifted shift map is != 256")
+        maxlen = self.maxv + 1
+        assert len(work) == maxlen, \
+               RGBMapError("shifted shift map is != %d" % (maxlen))
         self.sarr = work
         if callback:
             self.make_callback('changed')
@@ -336,7 +362,8 @@ class RGBMapper(Callback.Callbacks):
     def scale_and_shift(self, scale_pct, shift_pct, callback=True):
         """Stretch and/or shrink the color map via altering the shift map.
         """
-        self.sarr = numpy.arange(256)
+        maxlen = self.maxv + 1
+        self.sarr = np.arange(maxlen)
 
         #print "amount=%.2f location=%.2f" % (scale_pct, shift_pct)
         # limit shrinkage to 5% of original size
@@ -345,26 +372,27 @@ class RGBMapper(Callback.Callbacks):
 
         work = self._stretch(self.sarr, scale)
         n = len(work)
-        if n < 256:
+        if n < maxlen:
             # pad on the lowest and highest values of the shift map
-            m = (256 - n) // 2 + 1
-            barr = numpy.array([0]*m)
-            tarr = numpy.array([255]*m)
-            work = numpy.concatenate([barr, work, tarr])
-            work = work[:256]
+            m = (maxlen - n) // 2 + 1
+            barr = np.array([0]*m)
+            tarr = np.array([self.maxv] * m)
+            work = np.concatenate([barr, work, tarr])
+            work = work[:maxlen]
 
         # we are mimicing ds9's stretch and shift algorithm here.
         # ds9 seems to cut the center out of the stretched array
         # BEFORE shifting
         n = len(work) // 2
-        work = work[n-128:n+128].astype('uint')
-        assert len(work) == 256, \
-               RGBMapError("scaled shift map is != 256")
+        halflen = maxlen // 2
+        work = work[n-halflen:n+halflen].astype(np.uint)
+        assert len(work) == maxlen, \
+               RGBMapError("scaled shift map is != %d" % (maxlen))
 
         # shift map according to the shift_pct
         work = self._shift(work, shift_pct)
-        assert len(work) == 256, \
-               RGBMapError("shifted shift map is != 256")
+        assert len(work) == maxlen, \
+               RGBMapError("shifted shift map is != %d" % (maxlen))
 
         self.sarr = work
         if callback:
@@ -378,9 +406,13 @@ class RGBMapper(Callback.Callbacks):
         self.scale_and_shift(self.scale_pct, 0.0, callback=callback)
 
     def copy_attributes(self, dst_rgbmap):
-        dst_rgbmap.set_cmap(self.cmap)
-        dst_rgbmap.set_imap(self.imap)
-        dst_rgbmap.set_hash_algorithm(str(self.dist))
+        dst_rgbmap.set_cmap(self.cmap, callback=False)
+        dst_rgbmap.set_imap(self.imap, callback=False)
+        dst_rgbmap.set_hash_algorithm(str(self.dist), callback=False)
+        # TODO: set hash size
+        dst_rgbmap.carr = np.copy(self.carr)
+        dst_rgbmap.sarr = np.copy(self.sarr)
+        dst_rgbmap.recalc()
 
     def reset_cmap(self):
         self.recalc()
@@ -388,8 +420,7 @@ class RGBMapper(Callback.Callbacks):
 
 class NonColorMapper(RGBMapper):
     """
-    A kind of color mapper for data that is already in RGB 24/32bpp
-    (0-255 per channel) form.
+    A kind of color mapper for data that is already in RGB form.
 
     This mapper allows changing of color distribution and contrast
     adjustment, but does no coloring.
@@ -397,20 +428,21 @@ class NonColorMapper(RGBMapper):
     def __init__(self, logger, dist=None):
         super(NonColorMapper, self).__init__(logger)
 
-        self.dist.set_hash_size(256)
+        maxlen = self.maxv + 1
+        self.dist.set_hash_size(maxlen)
         self.reset_sarr(callback=False)
 
-    def _get_rgbarray(self, idx, rgbobj, image_order):
-        # NOTE: data is assumed to be in the range 0-255 at this point
+    def _get_rgbarray(self, idx, rgbobj, image_order='RGB'):
+        # NOTE: data is assumed to be in the range 0-maxv at this point
         # but clip as a precaution
         # See NOTE [A]: idx is always an array calculated in the caller and
         #    discarded afterwards
-        idx.clip(0, 255, out=idx)
+        idx.clip(0, self.maxv, out=idx)
 
         # run it through the shift array and clip the result
         # See NOTE [A]
         idx = self.sarr[idx]
-        idx.clip(0, 255, out=idx)
+        idx.clip(0, self.maxv, out=idx)
 
         ri, gi, bi = self.get_order_indexes(rgbobj.get_order(), 'RGB')
         rj, gj, bj = self.get_order_indexes(image_order, 'RGB')
@@ -423,8 +455,7 @@ class NonColorMapper(RGBMapper):
 
 class PassThruRGBMapper(RGBMapper):
     """
-    A kind of color mapper for data that is already in RGB 24/32bpp
-    (0-255 per channel) form.
+    A kind of color mapper for data that is already in RGB form.
 
     This mapper bypasses color distribution, contrast adjustment and
     coloring.  It is thus the most efficient one to use for maximum
@@ -434,19 +465,20 @@ class PassThruRGBMapper(RGBMapper):
         super(PassThruRGBMapper, self).__init__(logger)
 
         # ignore passed in distribution
-        self.dist = ColorDist.LinearDist(256)
+        maxlen = self.maxv + 1
+        self.dist = ColorDist.LinearDist(maxlen)
 
     def get_hasharray(self, idx):
-        # data is already constrained to 0..255 and we want to
+        # data is already constrained to 0..maxv and we want to
         # bypass color redistribution
         return idx
 
-    def _get_rgbarray(self, idx, rgbobj, image_order):
-        # NOTE: data is assumed to be in the range 0-255 at this point
+    def _get_rgbarray(self, idx, rgbobj, image_order='RGB'):
+        # NOTE: data is assumed to be in the range 0-maxv at this point
         # but clip as a precaution
         # See NOTE [A]: idx is always an array calculated in the caller and
         #    discarded afterwards
-        idx.clip(0, 255, out=idx)
+        idx.clip(0, self.maxv, out=idx)
 
         # bypass the shift array and skip color mapping,
         # index is the final data
@@ -458,5 +490,4 @@ class PassThruRGBMapper(RGBMapper):
         out[..., gi] = idx[..., gj]
         out[..., bi] = idx[..., bj]
 
-
 #END
