@@ -4,12 +4,12 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import numpy
+import numpy as np
 
 from ginga.canvas.CanvasObject import (CanvasObjectBase, _bool, _color,
                                        Point, MovePoint, ScalePoint,
                                        register_canvas_types,
-                                       colors_plus_none)
+                                       colors_plus_none, coord_names)
 from ginga.misc.ParamSet import Param
 from ginga.misc import Bunch
 from ginga import trcalc
@@ -26,9 +26,9 @@ class Image(OnePointMixin, CanvasObjectBase):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of corner of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -70,7 +70,8 @@ class Image(OnePointMixin, CanvasObjectBase):
                  showcap=False, flipy=False, optimize=True,
                  **kwdargs):
         self.kind = 'image'
-        CanvasObjectBase.__init__(self, x=x, y=y, image=image, alpha=alpha,
+        points = np.asarray([(x, y)], dtype=np.float)
+        CanvasObjectBase.__init__(self, points=points, image=image, alpha=alpha,
                                   scale_x=scale_x, scale_y=scale_y,
                                   interpolation=interpolation,
                                   linewidth=linewidth, linestyle=linestyle,
@@ -141,35 +142,34 @@ class Image(OnePointMixin, CanvasObjectBase):
 
         cache = self.get_cache(viewer)
 
-        #print("redraw whence=%f" % (whence))
         dst_order = viewer.get_rgb_order()
         image_order = self.image.get_order()
 
         if (whence <= 0.0) or (cache.cutout is None) or (not self.optimize):
             # get extent of our data coverage in the window
-            ((x0, y0), (x1, y1), (x2, y2), (x3, y3)) = viewer.get_pan_rect()
-            xmin = int(min(x0, x1, x2, x3))
-            ymin = int(min(y0, y1, y2, y3))
-            xmax = int(numpy.ceil(max(x0, x1, x2, x3)))
-            ymax = int(numpy.ceil(max(y0, y1, y2, y3)))
+            pts = np.asarray(viewer.get_pan_rect()).T
+            xmin = int(np.min(pts[0]))
+            ymin = int(np.min(pts[1]))
+            xmax = int(np.ceil(np.max(pts[0])))
+            ymax = int(np.ceil(np.max(pts[1])))
 
             # destination location in data_coords
             #dst_x, dst_y = self.x, self.y + ht
-            dst_x, dst_y = self.crdmap.to_data(self.x, self.y)
+            dst_x, dst_y = self.crdmap.to_data((self.x, self.y))
 
             a1, b1, a2, b2 = 0, 0, self.image.width, self.image.height
 
             # calculate the cutout that we can make and scale to merge
             # onto the final image--by only cutting out what is necessary
             # this speeds scaling greatly at zoomed in sizes
-            dst_x, dst_y, a1, b1, a2, b2 = \
-                   trcalc.calc_image_merge_clip(xmin, ymin, xmax, ymax,
-                                                dst_x, dst_y, a1, b1, a2, b2)
+            ((dst_x, dst_y), (a1, b1), (a2, b2)) = \
+                 trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
+                                              (dst_x, dst_y),
+                                              (a1, b1), (a2, b2))
 
             # is image completely off the screen?
             if (a2 - a1 <= 0) or (b2 - b1 <= 0):
                 # no overlay needed
-                #print "no overlay needed"
                 return
 
             # cutout and scale the piece appropriately by the viewer scale
@@ -177,10 +177,10 @@ class Image(OnePointMixin, CanvasObjectBase):
             # scale additionally by our scale
             _scale_x, _scale_y = scale_x * self.scale_x, scale_y * self.scale_y
 
-            res = self.image.get_scaled_cutout(a1, b1, a2, b2,
-                                               _scale_x, _scale_y,
-                                               #flipy=self.flipy,
-                                               method=self.interpolation)
+            res = self.image.get_scaled_cutout2((a1, b1), (a2, b2),
+                                                (_scale_x, _scale_y),
+                                                #flipy=self.flipy,
+                                                method=self.interpolation)
 
             # don't ask for an alpha channel from overlaid image if it
             # doesn't have one
@@ -199,27 +199,26 @@ class Image(OnePointMixin, CanvasObjectBase):
             pan_x, pan_y = viewer.get_pan()
             pan_off = viewer.data_off
             pan_x, pan_y = pan_x + pan_off, pan_y + pan_off
-            #print "pan x,y=%f,%f" % (pan_x, pan_y)
             off_x, off_y = dst_x - pan_x, dst_y - pan_y
             # scale offset
             off_x *= scale_x
             off_y *= scale_y
-            #print "off_x,y=%f,%f" % (off_x, off_y)
 
             # dst position in the pre-transformed array should be calculated
             # from the center of the array plus offsets
             ht, wd, dp = dstarr.shape
-            cache.cvs_x = int(round(wd / 2.0  + off_x))
-            cache.cvs_y = int(round(ht / 2.0  + off_y))
+            cvs_x = int(np.round(wd / 2.0  + off_x))
+            cvs_y = int(np.round(ht / 2.0  + off_y))
+            cache.cvs_pos = (cvs_x, cvs_y)
 
         # composite the image into the destination array at the
         # calculated position
-        trcalc.overlay_image(dstarr, cache.cvs_x, cache.cvs_y, cache.cutout,
+        trcalc.overlay_image(dstarr, cache.cvs_pos, cache.cutout,
                              dst_order=dst_order, src_order=image_order,
                              alpha=self.alpha, flipy=False)
 
     def _reset_cache(self, cache):
-        cache.setvals(cutout=None, drawn=False, cvs_x=0, cvs_y=0)
+        cache.setvals(cutout=None, drawn=False, cvs_pos=(0, 0))
         return cache
 
     def reset_optimize(self):
@@ -241,20 +240,25 @@ class Image(OnePointMixin, CanvasObjectBase):
         return (width, height)
 
     def get_coords(self):
-        x1, y1 = self.crdmap.to_data(self.x, self.y)
+        x1, y1 = self.crdmap.to_data((self.x, self.y))
         wd, ht = self.get_scaled_wdht()
         x2, y2 = x1 + wd, y1 + ht
         return (x1, y1, x2, y2)
 
+    def get_llur(self):
+        return self.get_coords()
+
     def get_center_pt(self):
         wd, ht = self.get_scaled_wdht()
-        return (self.x + wd / 2.0, self.y + ht / 2.0)
+        x1, y1, x2, y2 = self.get_coords()
+        return ((x1 + x2) / 2.0, (y1 + y2) / 2.0)
 
     def get_points(self):
         x1, y1, x2, y2 = self.get_coords()
         return [(x1, y1), (x2, y1), (x2, y2), (x1, y2)]
 
-    def contains(self, data_x, data_y):
+    def contains_pt(self, pt):
+        data_x, data_y = pt[:2]
         x1, y1, x2, y2 = self.get_coords()
         if ((x1 <= data_x < x2) and (y1 <= data_y < y2)):
             return True
@@ -270,8 +274,7 @@ class Image(OnePointMixin, CanvasObjectBase):
 
     def set_edit_point(self, i, pt, detail):
         if i == 0:
-            x, y = pt
-            self.move_to(x, y)
+            self.move_to_pt(pt)
         elif i == 1:
             scale_x, scale_y = self.calc_dual_scale_from_pt(pt, detail)
             self.scale_x = detail.scale_x * scale_x
@@ -321,9 +324,9 @@ class NormImage(Image):
     @classmethod
     def get_params_metadata(cls):
         return [
-            ## Param(name='coord', type=str, default='data',
-            ##       valid=['data'],
-            ##       description="Set type of coordinates"),
+            Param(name='coord', type=str, default='data',
+                  valid=coord_names,
+                  description="Set type of coordinates"),
             Param(name='x', type=float, default=0.0, argpos=0,
                   description="X coordinate of corner of object"),
             Param(name='y', type=float, default=0.0, argpos=1,
@@ -368,13 +371,13 @@ class NormImage(Image):
                  linewidth=0, linestyle='solid', color='lightgreen', showcap=False,
                  optimize=True, rgbmap=None, autocuts=None, **kwdargs):
         self.kind = 'normimage'
-        super(NormImage, self).__init__(x=x, y=y, image=image, alpha=alpha,
-                                            scale_x=scale_x, scale_y=scale_y,
-                                            interpolation=interpolation,
-                                            linewidth=linewidth, linestyle=linestyle,
-                                            color=color,
-                                            showcap=showcap, optimize=optimize,
-                                            **kwdargs)
+        super(NormImage, self).__init__(x, y, image=image, alpha=alpha,
+                                        scale_x=scale_x, scale_y=scale_y,
+                                        interpolation=interpolation,
+                                        linewidth=linewidth, linestyle=linestyle,
+                                        color=color,
+                                        showcap=showcap, optimize=optimize,
+                                        **kwdargs)
         self.rgbmap = rgbmap
         self.autocuts = autocuts
 
@@ -383,33 +386,32 @@ class NormImage(Image):
         if self.image is None:
             return
 
-        #print("redraw whence=%f" % (whence))
         cache = self.get_cache(viewer)
 
         if (whence <= 0.0) or (cache.cutout is None) or (not self.optimize):
             # get extent of our data coverage in the window
-            ((x0, y0), (x1, y1), (x2, y2), (x3, y3)) = viewer.get_pan_rect()
-            xmin = int(min(x0, x1, x2, x3))
-            ymin = int(min(y0, y1, y2, y3))
-            xmax = int(numpy.ceil(max(x0, x1, x2, x3)))
-            ymax = int(numpy.ceil(max(y0, y1, y2, y3)))
+            pts = np.asarray(viewer.get_pan_rect()).T
+            xmin = int(np.min(pts[0]))
+            ymin = int(np.min(pts[1]))
+            xmax = int(np.ceil(np.max(pts[0])))
+            ymax = int(np.ceil(np.max(pts[1])))
 
             # destination location in data_coords
-            dst_x, dst_y = self.crdmap.to_data(self.x, self.y)
+            dst_x, dst_y = self.crdmap.to_data((self.x, self.y))
 
             a1, b1, a2, b2 = 0, 0, self.image.width, self.image.height
 
             # calculate the cutout that we can make and scale to merge
             # onto the final image--by only cutting out what is necessary
             # this speeds scaling greatly at zoomed in sizes
-            dst_x, dst_y, a1, b1, a2, b2 = \
-                   trcalc.calc_image_merge_clip(xmin, ymin, xmax, ymax,
-                                                dst_x, dst_y, a1, b1, a2, b2)
+            ((dst_x, dst_y), (a1, b1), (a2, b2)) = \
+                 trcalc.calc_image_merge_clip((xmin, ymin), (xmax, ymax),
+                                              (dst_x, dst_y),
+                                              (a1, b1), (a2, b2))
 
             # is image completely off the screen?
             if (a2 - a1 <= 0) or (b2 - b1 <= 0):
                 # no overlay needed
-                #print "no overlay needed"
                 return
 
             # cutout and scale the piece appropriately by viewer scale
@@ -417,27 +419,26 @@ class NormImage(Image):
             # scale additionally by our scale
             _scale_x, _scale_y = scale_x * self.scale_x, scale_y * self.scale_y
 
-            res = self.image.get_scaled_cutout(a1, b1, a2, b2,
-                                               _scale_x, _scale_y,
-                                               method=self.interpolation)
+            res = self.image.get_scaled_cutout2((a1, b1), (a2, b2),
+                                                (_scale_x, _scale_y),
+                                                method=self.interpolation)
             cache.cutout = res.data
 
             # calculate our offset from the pan position
             pan_x, pan_y = viewer.get_pan()
             pan_off = viewer.data_off
             pan_x, pan_y = pan_x + pan_off, pan_y + pan_off
-            #print "pan x,y=%f,%f" % (pan_x, pan_y)
             off_x, off_y = dst_x - pan_x, dst_y - pan_y
             # scale offset
             off_x *= scale_x
             off_y *= scale_y
-            #print "off_x,y=%f,%f" % (off_x, off_y)
 
             # dst position in the pre-transformed array should be calculated
             # from the center of the array plus offsets
             ht, wd, dp = dstarr.shape
-            cache.cvs_x = int(round(wd / 2.0  + off_x))
-            cache.cvs_y = int(round(ht / 2.0  + off_y))
+            cvs_x = int(np.round(wd / 2.0  + off_x))
+            cvs_y = int(np.round(ht / 2.0  + off_y))
+            cache.cvs_pos = (cvs_x, cvs_y)
 
         if self.rgbmap is not None:
             rgbmap = self.rgbmap
@@ -450,8 +451,8 @@ class NormImage(Image):
             newdata = self.apply_visuals(viewer, cache.cutout, 0, vmax)
 
             # result becomes an index array fed to the RGB mapper
-            if not numpy.issubdtype(newdata.dtype, numpy.dtype('uint')):
-                newdata = newdata.astype(numpy.uint)
+            if not np.issubdtype(newdata.dtype, np.dtype('uint')):
+                newdata = newdata.astype(np.uint)
             idx = newdata
 
             self.logger.debug("shape of index is %s" % (str(idx.shape)))
@@ -471,7 +472,7 @@ class NormImage(Image):
 
         # composite the image into the destination array at the
         # calculated position
-        trcalc.overlay_image(dstarr, cache.cvs_x, cache.cvs_y, cache.rgbarr,
+        trcalc.overlay_image(dstarr, cache.cvs_pos, cache.rgbarr,
                              dst_order=dst_order, src_order=get_order,
                              alpha=self.alpha, flipy=False)
 
@@ -489,7 +490,7 @@ class NormImage(Image):
 
     def _reset_cache(self, cache):
         cache.setvals(cutout=None, prergb=None, rgbarr=None,
-                      drawn=False, cvs_x=0, cvs_y=0)
+                      drawn=False, cvs_pos=(0, 0))
         return cache
 
     def set_image(self, image):
@@ -499,11 +500,9 @@ class NormImage(Image):
         self.make_callback('image-set', image)
 
     def scale_by(self, scale_x, scale_y):
-        #print("scaling image")
         self.scale_x *= scale_x
         self.scale_y *= scale_y
         self.reset_optimize()
-        #print("image scale_x=%f scale_y=%f" % (self.scale_x, self.scale_y))
 
 
 # register our types
