@@ -12,7 +12,7 @@ import logging
 from ginga import AstroImage, colors
 from ginga.canvas.CanvasObject import get_canvas_types
 from ginga.misc import log
-from ginga.web.pgw import Widgets, Viewers, PgMain
+from ginga.web.pgw import Widgets, Viewers
 
 
 class FitsViewer(object):
@@ -36,10 +36,10 @@ class FitsViewer(object):
         fi.set_zoom_algorithm('rate')
         fi.set_zoomrate(1.4)
         fi.show_pan_mark(True)
-        fi.set_callback('drag-drop', self.drop_file)
-        fi.set_callback('none-move', self.motion)
+        fi.set_callback('drag-drop', self.drop_file_cb)
+        fi.set_callback('cursor-changed', self.cursor_cb)
         fi.set_bg(0.2, 0.2, 0.2)
-        fi.ui_setActive(True)
+        fi.ui_set_active(True)
         self.fitsimage = fi
 
         bd = fi.get_bindings()
@@ -55,12 +55,12 @@ class FitsViewer(object):
         canvas.enable_draw(True)
         canvas.enable_edit(True)
         canvas.set_drawtype('rectangle', color='lightblue')
-        canvas.setSurface(fi)
+        canvas.set_surface(fi)
         self.canvas = canvas
         # add canvas to view
         private_canvas = fi.get_canvas()
         private_canvas.add(canvas)
-        canvas.ui_setActive(True)
+        canvas.ui_set_active(True)
         canvas.register_for_cursor_drawing(fi)
         self.drawtypes = canvas.get_drawtypes()
         self.drawtypes.sort()
@@ -80,6 +80,7 @@ class FitsViewer(object):
 
         hbox = Widgets.HBox()
         hbox.set_border_width(2)
+        hbox.set_spacing(4)
 
         wdrawtype = Widgets.ComboBox()
         for name in self.drawtypes:
@@ -124,6 +125,7 @@ class FitsViewer(object):
 
         mode = self.canvas.get_draw_mode()
         hbox = Widgets.HBox()
+        hbox.set_spacing(4)
         btn1 = Widgets.RadioButton("Draw")
         btn1.set_state(mode == 'draw')
         btn1.add_callback('activated', lambda w, val: self.set_mode_cb('draw', val))
@@ -192,18 +194,20 @@ class FitsViewer(object):
         if len(fileName) != 0:
             self.load_file(fileName)
 
-    def drop_file(self, fitsimage, paths):
-        fileName = paths[0]
-        self.load_file(fileName)
-
-    def motion(self, viewer, button, data_x, data_y):
+    def drop_file_cb(self, viewer, paths):
+        filename = paths[0]
+        self.load_file(filename)
 
+    def cursor_cb(self, viewer, button, data_x, data_y):
+        """This gets called when the data position relative to the cursor
+        changes.
+        """
         # Get the value under the data coordinates
         try:
-            #value = viewer.get_data(data_x, data_y)
             # We report the value across the pixel, even though the coords
             # change halfway across the pixel
-            value = viewer.get_data(int(data_x+0.5), int(data_y+0.5))
+            value = viewer.get_data(int(data_x + viewer.data_off),
+                                    int(data_y + viewer.data_off))
 
         except Exception:
             value = None
