@@ -32,12 +32,12 @@ class FitsViewer(QtGui.QMainWindow):
         fi.enable_autozoom('on')
         fi.set_zoom_algorithm('rate')
         fi.set_zoomrate(1.4)
-        fi.show_pan_mark(True)
+        #fi.show_pan_mark(True)
         #fi.enable_draw(False)
-        fi.set_callback('drag-drop', self.drop_file)
-        fi.set_callback('none-move', self.motion)
+        fi.add_callback('drag-drop', self.drop_file_cb)
+        fi.add_callback('cursor-changed', self.cursor_cb)
         fi.set_bg(0.2, 0.2, 0.2)
-        fi.ui_setActive(True)
+        fi.ui_set_active(True)
         self.fitsimage = fi
 
         bd = fi.get_bindings()
@@ -54,7 +54,7 @@ class FitsViewer(QtGui.QMainWindow):
         private_canvas = fi.get_canvas()
         private_canvas.register_for_cursor_drawing(fi)
         private_canvas.add(canvas)
-        canvas.ui_setActive(True)
+        canvas.ui_set_active(True)
         self.drawtypes = canvas.get_drawtypes()
         self.drawtypes.sort()
 
@@ -164,19 +164,20 @@ class FitsViewer(QtGui.QMainWindow):
         if len(fileName) != 0:
             self.load_file(fileName)
 
-    def drop_file(self, fitsimage, paths):
+    def drop_file_cb(self, fitsimage, paths):
         fileName = paths[0]
-        #print(fileName)
         self.load_file(fileName)
 
-    def motion(self, viewer, button, data_x, data_y):
-
+    def cursor_cb(self, viewer, button, data_x, data_y):
+        """This gets called when the data position relative to the cursor
+        changes.
+        """
         # Get the value under the data coordinates
         try:
-            #value = viewer.get_data(data_x, data_y)
             # We report the value across the pixel, even though the coords
             # change halfway across the pixel
-            value = viewer.get_data(int(data_x+0.5), int(data_y+0.5))
+            value = viewer.get_data(int(data_x + viewer.data_off),
+                                    int(data_y + viewer.data_off))
 
         except Exception:
             value = None
