@@ -8,7 +8,7 @@ import sys
 import os.path
 import math
 
-from ginga.misc import Bunch
+from ginga.misc import Bunch, Callback
 import ginga.toolkit
 
 import gi
@@ -267,6 +267,59 @@ class MultiDragDropTreeView(Gtk.TreeView):
         self.defer_select=False
 
 
+class MDISubWindow(Callback.Callbacks):
+
+    def __init__(self, widget, label):
+        super(MDISubWindow, self).__init__()
+
+        self.widget = widget
+
+        vbox = Gtk.VBox()
+        vbox.set_border_width(4)
+        hbox = Gtk.HBox()
+        close = Gtk.Button("x")
+        maxim = Gtk.Button("^")
+        minim = Gtk.Button("v")
+        hbox.pack_start(close, False, False, 0)
+        hbox.pack_start(minim, False, False, 0)
+        hbox.pack_start(maxim, False, False, 0)
+
+        evbox = Gtk.EventBox()
+        evbox.add(label)
+        evbox.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray90"))
+        self.label = evbox
+        self.evbox = evbox
+        hbox.pack_start(evbox, True, True, 2)
+
+        vbox.pack_start(hbox, False, False, 0)
+        vbox.pack_start(widget, True, True, 4)
+
+        # what size does the widget want to be?
+        rect = widget.get_allocation()
+        self.x, self.y, wd, ht = rect.x, rect.y, rect.width, rect.height
+        ## wd = widget.get_preferred_width()
+        ## ht = widget.get_preferred_height()
+        ## wd, ht = widget.get_size_request()
+        self.width, self.height = max(wd, 300), max(ht, 300)
+
+        frame = Gtk.EventBox()
+        frame.set_size_request(self.width, self.height)
+        frame.props.visible_window = True
+        frame.set_border_width(0)
+        frame.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray70"))
+        self.frame = frame
+
+        frame.add(vbox)
+        frame.show_all()
+
+        for name in ('close', 'maximize', 'minimize'):
+            self.enable_callback(name)
+
+        maxim.connect('clicked', lambda *args: self.make_callback('maximize'))
+        minim.connect('clicked', lambda *args: self.make_callback('minimize'))
+        close.connect('clicked', lambda *args: self.make_callback('close'))
+
+
 class MDIWidget(Gtk.Layout):
     """
     Multiple Document Interface type widget for Gtk.
@@ -274,7 +327,7 @@ class MDIWidget(Gtk.Layout):
     NOTE: *** This is a work in progress! ***
     """
     def __init__(self):
-        super(MDIWidget, self).__init__()
+        Gtk.Layout.__init__(self)
 
         self.children = []
         self.cur_index = -1
@@ -282,6 +335,7 @@ class MDIWidget(Gtk.Layout):
         self.kbdmouse_mask = 0
         self.cascade_offset = 50
         self.minimized_width = 50
+        self.delta_px = 50
 
         self.connect("motion_notify_event", self.motion_notify_event)
         self.connect("button_press_event", self.button_press_event)
@@ -303,57 +357,20 @@ class MDIWidget(Gtk.Layout):
         self.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray50"))
 
     def append_page(self, widget, label):
-        vbox = Gtk.VBox()
-        vbox.set_border_width(4)
-        hbox = Gtk.HBox()
-        close = Gtk.Button("x")
-        maxim = Gtk.Button("^")
-        minim = Gtk.Button("v")
-        hbox.pack_start(close, False, False, 0)
-        hbox.pack_start(minim, False, False, 0)
-        hbox.pack_start(maxim, False, False, 0)
 
-        evbox = Gtk.EventBox()
-        evbox.add(label)
-        evbox.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray90"))
-        hbox.pack_start(evbox, True, True, 2)
-
-        vbox.pack_start(hbox, False, False, 0)
-        vbox.pack_start(widget, True, True, 4)
-
-        # what size does the widget want to be?
-        rect = widget.get_allocation()
-        x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
-        ## wd = widget.get_preferred_width()
-        ## ht = widget.get_preferred_height()
-        ## wd, ht = widget.get_size_request()
-        wd, ht = max(wd, 300), max(ht, 300)
-
-        frame = Gtk.EventBox()
-        frame.set_size_request(wd, ht)
-        frame.props.visible_window = True
-        frame.set_border_width(0)
-        frame.modify_bg(Gtk.StateType.NORMAL, Gdk.color_parse("gray70"))
-
-        frame.add(vbox)
-        frame.show_all()
-
-        pos = self.get_widget_position(frame)
-        if pos is None:
-            x, y = 0, 0
-        else:
-            x, y = pos
-        wd, ht = self.get_widget_size(frame)
-        subwin = Bunch.Bunch(widget=widget, label=evbox, frame=frame,
-                             x=x, y=y, width=wd, height=ht)
+        subwin = MDISubWindow(widget, label)
         self.children.append(subwin)
 
-        evbox.connect("button_press_event", self.select_child_cb, subwin)
-        frame.connect("button_press_event", self.start_resize_cb, subwin)
-        maxim.connect('clicked', lambda *args: self.maximize_page(subwin))
-        minim.connect('clicked', lambda *args: self.minimize_page(subwin))
+        subwin.evbox.connect("button_press_event", self.select_child_cb, subwin)
+        subwin.frame.connect("button_press_event", self.start_resize_cb, subwin)
+        subwin.add_callback('maximize', lambda *args: self.maximize_page(subwin))
+        subwin.add_callback('minimize', lambda *args: self.minimize_page(subwin))
+
+        self.put(subwin.frame, self.cascade_offset, self.cascade_offset)
+
+        self.update_subwin_position(subwin)
+        self.update_subwin_size(subwin)
 
-        self.put(frame, self.cascade_offset, self.cascade_offset)
         return subwin
 
     def set_tab_reorderable(self, w, tf):
@@ -434,7 +451,7 @@ class MDIWidget(Gtk.Layout):
         self.put(frame, subwin.x, subwin.y)
 
     def select_child_cb(self, layout, event, subwin):
-        ex, ey = event.x_root, event.y_root
+        x_root, y_root = event.x_root, event.y_root
 
         x, y = self.get_widget_position(subwin.frame)
         subwin.x, subwin.y = x, y
@@ -445,16 +462,60 @@ class MDIWidget(Gtk.Layout):
             self.set_current_page(idx)
 
         self.selected_child = Bunch.Bunch(subwin=subwin, action='move',
-                                          x_origin=x, y_origin=y, dx=ex, dy=ey)
+                                          x_origin=x, y_origin=y,
+                                          x_root=x_root, y_root=y_root)
         return True
 
     def start_resize_cb(self, widget, event, subwin):
-        ex, ey = event.x_root, event.y_root
-        x, y = self.get_widget_position(subwin.frame)
-        subwin.x, subwin.y = x, y
+        self.update_subwin_size(subwin)
+
+        x_root, y_root = event.x_root, event.y_root
+        x, y = widget.translate_coordinates(self, event.x, event.y)
+
+        rect = subwin.frame.get_allocation()
+        x1, y1, wd, ht = rect.x, rect.y, rect.width, rect.height
+        x2, y2 = x1 + wd, y1 + ht
+        subwin.x, subwin.y = x1, y1
+        subwin.width, subwin.height = wd, ht
+
+        updates = set([])
+        if abs(x - x2) < self.delta_px:
+            # right side
+            if abs(y - y2) < self.delta_px:
+                # lower right corner
+                origin = 'lr'
+                updates = set(['w', 'h'])
+            elif abs(y - y1) < self.delta_px:
+                origin = 'ur'
+                updates = set(['w', 'h', 'y'])
+            else:
+                origin = 'r'
+                updates = set(['w'])
+        elif abs(x - x1) < self.delta_px:
+            # left side
+            if abs(y - y2) < self.delta_px:
+                # lower left corner
+                origin = 'll'
+                updates = set(['w', 'h', 'x'])
+            elif abs(y - y1) < self.delta_px:
+                origin = 'ul'
+                updates = set(['w', 'h', 'x', 'y'])
+            else:
+                origin = 'l'
+                updates = set(['w', 'x'])
+        elif abs(y - y2) < self.delta_px:
+            # bottom
+            origin = 'b'
+            updates = set(['h'])
+        else:
+            origin = 't'
+            updates = set(['h', 'y'])
 
         self.selected_child = Bunch.Bunch(subwin=subwin, action='resize',
-                                          x_origin=x, y_origin=y, dx=ex, dy=ey)
+                                          x_origin=x1, y_origin=y1,
+                                          wd=wd, ht=ht,
+                                          x_root=x_root, y_root=y_root,
+                                          origin=origin, updates=updates)
         return True
 
     def button_press_event(self, widget, event):
@@ -464,8 +525,46 @@ class MDIWidget(Gtk.Layout):
             button |= 0x1 << (event.button - 1)
         return True
 
+    def _resize(self, bnch, x_root, y_root):
+        subwin = bnch.subwin
+        updates = bnch.updates
+
+        dx, dy = x_root - bnch.x_root, y_root - bnch.y_root
+
+        wd = bnch.wd
+        if 'w' in updates:
+            wd = int(wd + dx)
+        ht = bnch.ht
+        if 'h' in updates:
+            ht = int(ht + dy)
+
+        if 'x' in updates or 'y' in updates:
+            x = bnch.x_origin
+            if 'x' in updates:
+                x = int(x + dx)
+                if x < bnch.x_origin:
+                    wd = bnch.wd + abs(dx)
+                else:
+                    wd = bnch.wd + -abs(dx)
+
+            y = bnch.y_origin
+            if 'y' in updates:
+                y = int(y + dy)
+                if y < bnch.y_origin:
+                    ht = bnch.ht + abs(dy)
+                else:
+                    ht = bnch.ht + -abs(dy)
+
+            # this works better if it is not self.move_page()
+            self.move(subwin.frame, x, y)
+
+        if 'w' in updates or 'h' in updates:
+            # this works better if it is not self.resize_page()
+            subwin.frame.set_size_request(wd, ht)
+
+
     def button_release_event(self, widget, event):
-        x = event.x_root; y = event.y_root
+        x_root, y_root = event.x_root, event.y_root
         button = self.kbdmouse_mask
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
@@ -474,21 +573,27 @@ class MDIWidget(Gtk.Layout):
             bnch = self.selected_child
             subwin = bnch.subwin
             if bnch.action == 'move':
-                x = int(subwin.x + (x - bnch.dx))
-                y = int(subwin.y + (y - bnch.dy))
+                x = int(subwin.x + (x_root - bnch.x_root))
+                y = int(subwin.y + (y_root - bnch.y_root))
                 self.move_page(subwin, x, y)
 
             elif bnch.action == 'resize':
-                wd = int(subwin.width + (x - bnch.dx))
-                ht = int(subwin.height + (y - bnch.dy))
-                self.resize_page(subwin, wd, ht)
+                self._resize(bnch, x_root, y_root)
+
+                self.update_subwin_position(subwin)
+                # NOTE: necessary for wrapped widget to remember position
+                self.move_page(subwin, subwin.x, subwin.y)
+
+                self.update_subwin_size(subwin)
+                # NOTE: necessary for wrapped widget to remember size
+                self.resize_page(subwin, subwin.width, subwin.height)
 
             self.selected_child = None
         return True
 
     def motion_notify_event(self, widget, event):
         button = self.kbdmouse_mask
-        x, y, state = event.x_root, event.y_root, event.state
+        x_root, y_root, state = event.x_root, event.y_root, event.state
 
         if state & Gdk.ModifierType.BUTTON1_MASK:
             button |= 0x1
@@ -501,16 +606,13 @@ class MDIWidget(Gtk.Layout):
             bnch = self.selected_child
             subwin = bnch.subwin
             if bnch.action == 'move':
-                x = int(subwin.x + (x - bnch.dx))
-                y = int(subwin.y + (y - bnch.dy))
+                x = int(subwin.x + (x_root - bnch.x_root))
+                y = int(subwin.y + (y_root - bnch.y_root))
                 # this works better if it is not self.move_page()
                 self.move(subwin.frame, x, y)
 
             elif bnch.action == 'resize':
-                wd = int(subwin.width + (x - bnch.dx))
-                ht = int(subwin.height + (y - bnch.dy))
-                # this works better if it is not self.resize_page()
-                subwin.frame.set_size_request(wd, ht)
+                self._resize(bnch, x_root, y_root)
 
         return True
 
@@ -583,6 +685,9 @@ class MDIWidget(Gtk.Layout):
         self.move_page(subwin, x, height-ht)
         #self.lower_widget(subwin)
 
+    def close_page(self, subwin):
+        pass
+
 
 class FileSelection(object):
 
@@ -640,39 +745,45 @@ class DirectorySelection(FileSelection):
         super(DirectorySelection, self).popup(title, callfn, initialdir)
 
 
-class Timer(object):
+class Timer(Callback.Callbacks):
     """Abstraction of a GUI-toolkit implemented timer."""
 
-    def __init__(self, ival_sec, expire_cb, data=None):
-        """Create a timer set to expire after `ival_sec` and which will
-        call the callable `expire_cb` when it expires.
+    def __init__(self, duration=0.0):
+        """Create a timer set to expire after `duration` sec.
         """
-        self.ival_sec = ival_sec
-        self.cb = expire_cb
-        self.data = data
+        super(Timer, self).__init__()
+
+        self.duration = duration
+        # For storing aritrary data with timers
+        self.data = Bunch.Bunch()
+
         self._timer = None
 
-    def start(self, ival_sec=None):
-        """Start the timer.  If `ival_sec` is not None, it should
+        for name in ('expired', 'canceled'):
+            self.enable_callback(name)
+
+    def start(self, duration=None):
+        """Start the timer.  If `duration` is not None, it should
         specify the time to expiration in seconds.
         """
-        if ival_sec is None:
-            ival_sec = self.ival_sec
+        if duration is None:
+            duration = self.duration
 
-        self.cancel()
+        self.set(duration)
+
+    def set(self, duration):
+
+        self.stop()
 
         # Gtk timer set in milliseconds
-        time_ms = int(ival_sec * 1000.0)
+        time_ms = int(duration * 1000.0)
         self._timer = GObject.timeout_add(time_ms, self._redirect_cb)
 
     def _redirect_cb(self):
         self._timer = None
-        self.cb(self)
+        self.make_callback('expired')
 
-    def cancel(self):
-        """Cancel this timer.  If the timer is not running, there
-        is no error.
-        """
+    def stop(self):
         try:
             if self._timer is not None:
                 GObject.source_remove(self._timer)
@@ -680,6 +791,15 @@ class Timer(object):
         except:
             pass
 
+    def cancel(self):
+        """Cancel this timer.  If the timer is not running, there
+        is no error.
+        """
+        self.stop()
+        self.make_callback('canceled')
+
+    clear = cancel
+
 
 def combo_box_new_text():
     liststore = Gtk.ListStore(GObject.TYPE_STRING)
@@ -694,21 +814,40 @@ def get_scroll_info(event):
     """
     Returns the (degrees, direction) of a scroll motion Gtk event.
     """
-    direction = None
-    if event.direction == Gdk.ScrollDirection.UP:
-        direction = 0.0
-    elif event.direction == Gdk.ScrollDirection.DOWN:
-        direction = 180.0
-    elif event.direction == Gdk.ScrollDirection.LEFT:
-        direction = 270.0
-    elif event.direction == Gdk.ScrollDirection.RIGHT:
-        direction = 90.0
+    valid, dx, dy = event.get_scroll_deltas()
+    if valid:
+        # we have a trackpad or some device that reports pixel deltas
+        delta = math.sqrt(dx ** 2 + dy ** 2)
+        if dy < 0:
+            delta = -delta
 
-    # TODO: does Gtk encode the amount of scroll?
-    # 15 deg is standard 1-click turn for a wheel mouse
-    degrees = 15.0
+        ang_rad = math.atan2(dy, dx)
+        direction = math.degrees(ang_rad) - 90.0
+        direction = math.fmod(direction + 360.0, 360.0)
 
-    return (degrees, direction)
+        # TODO: is this accurate?--NOT TESTED
+        num_degrees = delta / 8.0
+
+    else:
+        valid, direction = event.get_scroll_direction()
+        if valid:
+            if event.direction == Gdk.ScrollDirection.UP:
+                direction = 0.0
+            elif event.direction == Gdk.ScrollDirection.DOWN:
+                direction = 180.0
+            elif event.direction == Gdk.ScrollDirection.LEFT:
+                direction = 270.0
+            elif event.direction == Gdk.ScrollDirection.RIGHT:
+                direction = 90.0
+
+        else:
+            direction = None
+
+        # TODO: does Gtk encode the amount of scroll?
+        # 15 deg is standard 1-click turn for a wheel mouse
+        num_degrees = 15.0
+
+    return (num_degrees, direction)
 
 def get_icon(iconpath, size=None):
     if size is not None:
