@@ -4,7 +4,8 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import time
+
+# import time
 
 from ginga.gtk3w import GtkHelp
 import ginga.util.six as six
@@ -17,15 +18,39 @@ from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import GdkPixbuf
 
+has_webkit = False
+try:
+    # this is necessary to prevent a warning message on import
+    import gi
+    gi.require_version('WebKit', '3.0')
+
+    from gi.repository import WebKit  # noqa
+    has_webkit = True
+except ImportError:
+    pass
+
+__all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
+           'TextArea', 'Label', 'Button', 'ComboBox',
+           'SpinBox', 'Slider', 'ScrollBar', 'CheckBox', 'ToggleButton',
+           'RadioButton', 'Image', 'ProgressBar', 'StatusBar', 'TreeView',
+           'WebView', 'ContainerBase', 'Box', 'HBox', 'VBox', 'Frame',
+           'Expander', 'TabWidget', 'StackWidget', 'MDIWidget', 'ScrollArea',
+           'Splitter', 'GridBox', 'Toolbar', 'MenuAction',
+           'Menu', 'Menubar', 'TopLevelMixin', 'TopLevel', 'Application',
+           'Dialog', 'SaveDialog', 'DragPackage', 'WidgetMoveEvent',
+           'name_mangle', 'make_widget', 'hadjust', 'build_info', 'wrap',
+           'has_webkit']
+
+
 class WidgetError(Exception):
     """For errors thrown in this module."""
     pass
 
-
 # (see TabWidget)
 _widget_move_event = None
 _app = None
 
+
 # BASE
 
 class WidgetBase(Callback.Callbacks):
@@ -43,27 +68,32 @@ class WidgetBase(Callback.Callbacks):
     def set_tooltip(self, text):
         self.widget.set_tooltip_text(text)
 
+    def get_enabled(self):
+        self.widget.get_sensitive()
+
     def set_enabled(self, tf):
         self.widget.set_sensitive(tf)
 
     def get_size(self):
         try:
             rect = self.widget.get_allocation()
-            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+            # x, y = rect.x, rect.y
+            wd, ht = rect.width, rect.height
 
         except Exception as e:
             # window maybe isn't realized yet--try other ways
             min_req, nat_req = self.widget.get_preferred_size()
             wd, ht = nat_req.width, nat_req.height
-            #req = self.widget.get_size_request()
-            #wd, ht = req
+            # req = self.widget.get_size_request()
+            # wd, ht = req
 
-            #wd, ht = max(1, wd), max(1, ht)
+            # wd, ht = max(1, wd), max(1, ht)
         return wd, ht
 
     def get_pos(self):
-        rect = widget.get_allocation()
-        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        rect = self.widget.get_allocation()
+        x, y = rect.x, rect.y
+        # width, height = rect.width, rect.height
         return x, y
 
     def get_app(self):
@@ -73,7 +103,7 @@ class WidgetBase(Callback.Callbacks):
         self.widget.destroy()
 
     def show(self):
-        #self.widget.show()
+        # self.widget.show()
         self.widget.show_all()
 
     def hide(self):
@@ -85,7 +115,7 @@ class WidgetBase(Callback.Callbacks):
     def resize(self, width, height):
         self.widget.set_size_request(width, height)
         # hackish way to allow the widget to be resized down again later
-        GObject.idle_add(self.widget.set_size_request, -1, -1)
+        # GObject.idle_add(self.widget.set_size_request, -1, -1)
 
     def get_font(self, font_family, point_size):
         font = GtkHelp.get_font(font_family, point_size)
@@ -95,6 +125,7 @@ class WidgetBase(Callback.Callbacks):
         # this is for compatibility with Qt widgets
         pass
 
+
 # BASIC WIDGETS
 
 class TextEntry(WidgetBase):
@@ -153,6 +184,7 @@ class TextEntry(WidgetBase):
         self.widget.set_width_chars(numchars)
         pass
 
+
 class TextEntrySet(WidgetBase):
     def __init__(self, text='', editable=True):
         super(TextEntrySet, self).__init__()
@@ -189,13 +221,14 @@ class TextEntrySet(WidgetBase):
         self.widget.modify_font(font)
 
     def set_length(self, numchars):
-        #self.widget.set_width_chars(numchars)
+        # self.widget.set_width_chars(numchars)
         pass
 
     def set_enabled(self, tf):
         super(TextEntrySet, self).set_enabled(tf)
         self.entry.set_sensitive(tf)
 
+
 class TextArea(WidgetBase):
     def __init__(self, wrap=False, editable=False):
         super(TextArea, self).__init__()
@@ -229,12 +262,12 @@ class TextArea(WidgetBase):
 
         end = buf.get_end_iter()
         mark = buf.get_insert()
-        #self.tw.scroll_to_iter(end, 0.5)
+        # self.tw.scroll_to_iter(end, 0.5)
         # NOTE: this was causing a segfault if the text widget is
         # not mapped yet!  Seems to be fixed in recent versions of
         # gtk
         buf.move_mark(mark, end)
-        res = self.tw.scroll_to_mark(mark, 0.2, False, 0.0, 0.0)
+        res = self.tw.scroll_to_mark(mark, 0.2, False, 0.0, 0.0)  # noqa
 
     def get_text(self):
         buf = self.tw.get_buffer()
@@ -247,7 +280,7 @@ class TextArea(WidgetBase):
         if numlines > self.histlimit:
             rmcount = int(numlines - self.histlimit)
             start = buf.get_iter_at_line(0)
-            end   = buf.get_iter_at_line(rmcount)
+            end = buf.get_iter_at_line(rmcount)
             buf.delete(start, end)
 
     def clear(self):
@@ -276,6 +309,7 @@ class TextArea(WidgetBase):
         else:
             self.tw.set_wrap_mode(Gtk.WrapMode.NONE)
 
+
 class Label(WidgetBase):
     def __init__(self, text='', halign='left', style='normal', menu=None):
         super(Label, self).__init__()
@@ -285,7 +319,6 @@ class Label(WidgetBase):
         evbox.set_border_width(0)
         evbox.props.visible_window = False
         evbox.add(label)
-        evbox.connect("button_press_event", self._cb_redirect)
 
         if halign == 'left':
             label.set_justify(Gtk.Justification.LEFT)
@@ -296,6 +329,8 @@ class Label(WidgetBase):
 
         evbox.connect("button_press_event", self._cb_redirect)
         self.enable_callback('activated')
+        evbox.connect("button_release_event", self._cb_redirect2)
+        self.enable_callback('released')
 
         self.label = label
         self.menu = menu
@@ -317,8 +352,16 @@ class Label(WidgetBase):
 
         elif event.button == 3 and self.menu is not None:
             menu_w = self.menu.get_widget()
-            return menu_w.popup(None, None, None, None,
-                                event.button, event.time)
+            if menu_w.get_sensitive():
+                return menu_w.popup(None, None, None, None,
+                                    event.button, event.time)
+        return False
+
+    def _cb_redirect2(self, widget, event):
+        if event.button == 1:
+            self.make_callback('released')
+            return True
+
         return False
 
     def get_text(self):
@@ -336,6 +379,7 @@ class Label(WidgetBase):
         if fg is not None:
             self.label.modify_fg(Gtk.StateType.NORMAL, Gdk.color_parse(fg))
 
+
 class Button(WidgetBase):
     def __init__(self, text=''):
         super(Button, self).__init__()
@@ -484,10 +528,10 @@ class Slider(WidgetBase):
 
     def set_tracking(self, tf):
         if tf:
-            #self.widget.set_update_policy(Gtk.UPDATE_CONTINUOUS)
+            # self.widget.set_update_policy(Gtk.UPDATE_CONTINUOUS)
             pass
         else:
-            #self.widget.set_update_policy(Gtk.UPDATE_DISCONTINUOUS)
+            # self.widget.set_update_policy(Gtk.UPDATE_DISCONTINUOUS)
             pass
 
     def set_limits(self, minval, maxval, incr_value=1):
@@ -604,8 +648,9 @@ class Image(WidgetBase):
 
             elif event.button == 3 and self.menu is not None:
                 menu_w = self.menu.get_widget()
-                return menu_w.popup(None, None, None, None,
-                                    event.button, event.time)
+                if menu_w.get_sensitive():
+                    return menu_w.popup(None, None, None, None,
+                                        event.button, event.time)
 
     def _cb_redirect2(self, widget, event):
         if event.type == Gdk.EventType.BUTTON_RELEASE:
@@ -616,14 +661,15 @@ class Image(WidgetBase):
     def _set_image(self, native_image):
         self.image.set_from_pixbuf(native_image.get_pixbuf())
 
+
 class ProgressBar(WidgetBase):
     def __init__(self):
         super(ProgressBar, self).__init__()
 
         w = Gtk.ProgressBar()
         # GTK3
-        #w.set_orientation(Gtk.Orientation.HORIZONTAL)
-        #w.set_inverted(False)
+        # w.set_orientation(Gtk.Orientation.HORIZONTAL)
+        # w.set_inverted(False)
         self.widget = w
 
     def set_value(self, pct):
@@ -631,6 +677,7 @@ class ProgressBar(WidgetBase):
         self.widget.set_fraction(pct)
         self.widget.set_text("%.2f %%" % (pct * 100.0))
 
+
 class StatusBar(WidgetBase):
     def __init__(self):
         super(StatusBar, self).__init__()
@@ -685,7 +732,7 @@ class TreeView(WidgetBase):
         if self.dragable:
             tv = GtkHelp.MultiDragDropTreeView()
             # enable drag from this widget
-            toImage = [ ( "text/plain", 0, 0 ) ]
+            toImage = [("text/plain", 0, 0)]
             tv.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK,
                                         toImage, Gdk.DragAction.COPY)
             tv.connect("drag-data-get", self._start_drag)
@@ -717,8 +764,8 @@ class TreeView(WidgetBase):
         # create the column headers
         if not isinstance(columns[0], str):
             # columns specifies a mapping
-            headers = [ col[0] for col in columns ]
-            datakeys = [ col[1] for col in columns ]
+            headers = [col[0] for col in columns]
+            datakeys = [col[1] for col in columns]
         else:
             headers = datakeys = columns
 
@@ -749,7 +796,7 @@ class TreeView(WidgetBase):
                 tvc.set_clickable(True)
             if n == 0:
                 fn_data = self._mkcolfn0(kwd)
-                ## cell.set_property('xalign', 1.0)
+                # cell.set_property('xalign', 1.0)
             else:
                 fn_data = self._mkcolfnN(kwd)
             tvc.set_cell_data_func(cell, fn_data)
@@ -895,7 +942,7 @@ class TreeView(WidgetBase):
         treeselection.select_iter(item)
 
     def highlight_path(self, path, onoff, font_color='green'):
-        item = self._path_to_item(path)
+        item = self._path_to_item(path)  # noqa
         # TODO
 
     def _path_to_item(self, path):
@@ -942,13 +989,13 @@ class TreeView(WidgetBase):
             bnch2 = model.get_value(iter2, 0)
             if isinstance(bnch1, str):
                 if isinstance(bnch2, str):
-                    return cmp(bnch1.lower(), bnch2.lower())
+                    return bnch1.lower() == bnch2.lower()  # was using cmp
                 return 0
             val1, val2 = bnch1[idx], bnch2[idx]
             if isinstance(val1, str):
                 val1 = val1.lower()
                 val2 = val2.lower()
-            res = cmp(val1, val2)
+            res = val1 == val2  # was using nonexistent cmp
             return res
         return fn
 
@@ -981,12 +1028,40 @@ class TreeView(WidgetBase):
         return fn
 
     def _start_drag(self, treeview, context, selection,
-                         info, timestamp):
+                    info, timestamp):
         res_dict = self.get_selected()
         drag_pkg = DragPackage(self.tv, selection)
         self.make_callback('drag-start', drag_pkg, res_dict)
         drag_pkg.start_drag()
 
+
+class WebView(WidgetBase):
+    def __init__(self):
+        if not has_webkit:
+            raise NotImplementedError("Missing webkit")
+
+        super(WebView, self).__init__()
+        self.widget = WebKit.WebView()
+
+    def load_url(self, url):
+        self.widget.open(url)
+
+    def load_html_string(self, html_string):
+        self.widget.load_string(html_string, 'text/html', 'utf-8', 'file://')
+
+    def go_back(self):
+        self.widget.go_back()
+
+    def go_forward(self):
+        self.widget.go_forward()
+
+    def reload_page(self):
+        self.widget.reload()
+
+    def stop_loading(self):
+        self.widget.stop_loading()
+
+
 # CONTAINERS
 
 class ContainerBase(WidgetBase):
@@ -1004,7 +1079,7 @@ class ContainerBase(WidgetBase):
             childw.destroy()
 
     def remove(self, w, delete=False):
-        if not w in self.children:
+        if w not in self.children:
             raise KeyError("Widget is not a child of this container")
         self.children.remove(w)
 
@@ -1059,10 +1134,12 @@ class Box(ContainerBase):
         self.widget.pack_start(child_w, expand, True, 0)
         self.widget.show_all()
 
+
 class VBox(Box):
     def __init__(self):
         super(VBox, self).__init__(orientation='vertical')
 
+
 class HBox(Box):
     def __init__(self):
         super(HBox, self).__init__(orientation='horizontal')
@@ -1107,17 +1184,17 @@ class TabWidget(ContainerBase):
         self.detachable = detachable
 
         nb = GtkHelp.Notebook()
-        #nb = Gtk.Notebook()
+        # nb = Gtk.Notebook()
         nb.set_show_border(False)
         nb.set_scrollable(True)
         # Allows drag-and-drop between notebooks
-        #nb.set_group_id(group)  # in gtk3?
+        # nb.set_group_id(group)  # in gtk3?
         if self.detachable:
             nb.connect("create-window", self._tab_detach_cb)
         nb.connect("page-added", self._tab_insert_cb)
         nb.connect("page-removed", self._tab_remove_cb)
         nb.sconnect("switch-page", self._cb_redirect)
-        #nb.connect("switch-page", self._cb_redirect)
+        # nb.connect("switch-page", self._cb_redirect)
         self.widget = nb
         self.set_tab_position(tabpos)
 
@@ -1141,7 +1218,7 @@ class TabWidget(ContainerBase):
         # (native widget already has been removed by gtk)
         self.children.remove(child)
 
-        #nchild_w.unparent()
+        # nchild_w.unparent()
         self.make_callback('page-detach', child)
 
     def _tab_insert_cb(self, nbw, nchild_w, page_num):
@@ -1211,15 +1288,17 @@ class TabWidget(ContainerBase):
         else:
             evbox.modify_bg(Gtk.StateType.NORMAL, None)
 
+
 class StackWidget(TabWidget):
     def __init__(self):
         super(StackWidget, self).__init__()
 
         nb = self.widget
-        #nb.set_scrollable(False)
+        # nb.set_scrollable(False)
         nb.set_show_tabs(False)
         nb.set_show_border(False)
 
+
 class MDIWidget(ContainerBase):
 
     def __init__(self, tabpos='top', mode='tabs'):
@@ -1244,6 +1323,9 @@ class MDIWidget(ContainerBase):
         sw.set_vadjustment(self.mdi_w.get_vadjustment())
         sw.add(self.mdi_w)
 
+        for name in ('page-switch', 'page-close'):
+            self.enable_callback(name)
+
     def get_mode(self):
         return self.mode
 
@@ -1255,7 +1337,7 @@ class MDIWidget(ContainerBase):
         child_w = child.get_widget()
         label = Gtk.Label(title)
         subwin = self.mdi_w.append_page(child_w, label)
-        #self.mdi_w.show_all()
+        # self.mdi_w.show_all()
         # attach title to child
         child.extdata.tab_title = title
 
@@ -1271,6 +1353,8 @@ class MDIWidget(ContainerBase):
             x, y = pos
             self.mdi_w.move_page(subwin, x, y)
 
+        subwin.add_callback('close', self._window_close, child)
+
     def _remove(self, childw, delete=False):
         self.mdi_w.remove(childw)
         if delete:
@@ -1298,6 +1382,9 @@ class MDIWidget(ContainerBase):
         child.extdata.mdi_pos = (x, y)
         return True
 
+    def _window_close(self, subwin, child):
+        return self.make_callback('page-close', child)
+
     def get_index(self):
         return self.mdi_w.get_current_page()
 
@@ -1321,6 +1408,7 @@ class MDIWidget(ContainerBase):
     def use_tabs(self, tf):
         pass
 
+
 class ScrollArea(ContainerBase):
     def __init__(self):
         super(ScrollArea, self).__init__()
@@ -1335,7 +1423,8 @@ class ScrollArea(ContainerBase):
 
     def _resize_cb(self, widget, allocation):
         rect = widget.get_allocation()
-        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        # x, y = rect.x, rect.y
+        width, height = rect.width, rect.height
         self.make_callback('configure', width, height)
         return True
 
@@ -1502,15 +1591,23 @@ class Toolbar(ContainerBase):
         w = child.get_widget()
         tool_w.add(w)
         tool = ContainerBase()
+        tool.widget = tool_w
+        tool_w.show()
         tool.add_ref(child)
         self.add_ref(tool)
         self.widget.insert(tool_w, -1)
         return tool
 
-    def add_menu(self, text, menu=None):
+    def add_menu(self, text, menu=None, mtype='tool'):
         if menu is None:
             menu = Menu()
-        child = self.add_action(text)
+        if mtype == 'tool':
+            child = self.add_action(text)
+        else:
+            child = Label(text, style='clickable', menu=menu)
+            self.add_widget(child)
+            child.add_callback('released', lambda w: menu.hide())
+
         child.add_callback('activated', lambda w: menu.popup())
         return menu
 
@@ -1559,19 +1656,33 @@ class Menu(ContainerBase):
         super(Menu, self).__init__()
 
         self.widget = Gtk.Menu()
+        self.menus = Bunch.Bunch(caseless=True)
         self.widget.show()
 
     def add_widget(self, child):
         menuitem_w = child.get_widget()
         self.widget.append(menuitem_w)
         self.add_ref(child)
-        #self.widget.show_all()
+        # self.widget.show_all()
 
     def add_name(self, name, checkable=False):
         child = MenuAction(text=name, checkable=checkable)
         self.add_widget(child)
         return child
 
+    def add_menu(self, name):
+        item_w = Gtk.MenuItem(label=name)
+        child = Menu()
+        self.add_ref(child)
+        self.menus[name] = child
+        item_w.set_submenu(child.get_widget())
+        self.widget.append(item_w)
+        item_w.show()
+        return child
+
+    def get_menu(self, name):
+        return self.menus[name]
+
     def add_separator(self):
         sep = Gtk.SeparatorMenuItem()
         self.widget.append(sep)
@@ -1581,10 +1692,11 @@ class Menu(ContainerBase):
         menu = self.widget
         menu.show_all()
         if six.PY2:
-            now = long(0)
+            now = long(0)  # noqa
         else:
             now = int(0)
-        menu.popup(None, None, None, None, 0, now)
+        if menu.get_sensitive():
+            menu.popup(None, None, None, None, 0, now)
 
 
 class Menubar(ContainerBase):
@@ -1625,7 +1737,7 @@ class TopLevelMixin(object):
         self.widget.connect("window_state_event", self._window_event)
         self.widget.connect("configure-event", self._configure_event)
 
-        if not title is None:
+        if title is not None:
             self.widget.set_title(title)
 
         self.enable_callback('close')
@@ -1649,7 +1761,7 @@ class TopLevelMixin(object):
 
     def _window_event(self, widget, event):
         if ((event.changed_mask & Gdk.WindowState.FULLSCREEN) or
-            (event.changed_mask & Gdk.WindowState.MAXIMIZED)):
+                (event.changed_mask & Gdk.WindowState.MAXIMIZED)):
             self._fullscreen = True
         else:
             self._fullscreen = False
@@ -1661,23 +1773,24 @@ class TopLevelMixin(object):
         return False
 
     def close(self):
-        ## try:
-        ##     self.widget.destroy()
-        ## except Exception as e:
-        ##     pass
-        #self.widget = None
+        # try:
+        #     self.widget.destroy()
+        # except Exception as e:
+        #     pass
+        # self.widget = None
 
         self.make_callback('close')
 
     def get_size(self):
         try:
             rect = self.widget.get_allocation()
-            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+            # x, y = rect.x, rect.y
+            wd, ht = rect.width, rect.height
 
         except Exception as e:
             # window maybe isn't realized yet--try other ways
-            #req = self.widget.get_size_request()
-            #wd, ht = req
+            # req = self.widget.get_size_request()
+            # wd, ht = req
             min_req, nat_req = self.widget.get_preferred_size()
             wd, ht = nat_req.width, nat_req.height
             ed = self.extdata
@@ -1686,13 +1799,15 @@ class TopLevelMixin(object):
         return wd, ht
 
     def get_pos(self):
+        res = None
         window = self.widget.get_window()
         if window is not None:
-            x, y = window.get_origin()
-        else:
-            ed = self.extdata
-            x, y = ed.get('x', None), ed.get('y', None)
+            res = window.get_origin()
+            if isinstance(res, tuple) and len(res) == 2:
+                return res
 
+        ed = self.extdata
+        x, y = ed.get('x', None), ed.get('y', None)
         return x, y
 
     def raise_(self):
@@ -1798,14 +1913,14 @@ class Application(Callback.Callbacks):
         except:
             self.screen_wd = 1600
             self.screen_ht = 1200
-        ## self.logger.debug("screen dimensions %dx%d" % (
-        ##     self.screen_wd, self.screen_ht))
+        # self.logger.debug("screen dimensions %dx%d" % (
+        #     self.screen_wd, self.screen_ht))
 
         _app = self
 
         # supposedly needed for GObject < 3.10.2
         GObject.threads_init()
-        ## self._time_save = time.time()
+        # self._time_save = time.time()
 
         for name in ('shutdown', ):
             self.enable_callback(name)
@@ -1825,16 +1940,15 @@ class Application(Callback.Callbacks):
                 # lost--we want to know whether the process_event loop
                 # is running, so ping periodically if events are showing
                 # up
-                ## cur_time = time.time()
-                ## if cur_time - self._time_save > 10.0:
-                ##     self.logger.info("process_events ping!")
-                ##     self._time_save = cur_time
+                # cur_time = time.time()
+                # if cur_time - self._time_save > 10.0:
+                #     self.logger.info("process_events ping!")
+                #     self._time_save = cur_time
 
             except Exception as e:
                 self.logger.error("Exception in main_iteration() loop: %s" %
                                   (str(e)))
 
-
     def process_end(self):
         pass
 
@@ -1935,7 +2049,8 @@ class SaveDialog(object):
 
         if response == Gtk.ResponseType.OK:
             path = self.widget.get_filename()
-            if self.selectedfilter is not None and not path.endswith(self.selectedfilter):
+            if (self.selectedfilter is not None and
+                    not path.endswith(self.selectedfilter)):
                 path += self.selectedfilter
             self.widget.destroy()
             return path
@@ -1943,6 +2058,7 @@ class SaveDialog(object):
             self.widget.destroy()
             return None
 
+
 class DragPackage(object):
     def __init__(self, src_widget, selection):
         self.src_widget = src_widget
@@ -1955,6 +2071,7 @@ class DragPackage(object):
     def start_drag(self):
         pass
 
+
 class WidgetMoveEvent(object):
     def __init__(self, src_widget, child):
         self.src_widget = src_widget
@@ -1967,6 +2084,7 @@ class WidgetMoveEvent(object):
     def reject(self):
         self._result = False
 
+
 # MODULE FUNCTIONS
 
 def name_mangle(name, pfx=''):
@@ -1978,6 +2096,7 @@ def name_mangle(name, pfx=''):
             newname.append(c)
     return pfx + ''.join(newname)
 
+
 def make_widget(title, wtype):
     if wtype == 'label':
         w = Label(title)
@@ -1987,7 +2106,7 @@ def make_widget(title, wtype):
         w.label.set_alignment(0.05, 0.95)
     elif wtype == 'entry':
         w = TextEntry()
-        #w.get_widget().set_width_chars(12)
+        # w.get_widget().set_width_chars(12)
     elif wtype == 'entryset':
         w = TextEntrySet()
     elif wtype == 'combobox':
@@ -2084,36 +2203,4 @@ def wrap(native_widget):
     return wrapper
 
 
-def get_orientation(container):
-    if not hasattr(container, 'size'):
-        return 'vertical'
-    (wd, ht) = container.size
-    ## wd, ht = container.get_size()
-    #print('container size is %dx%d' % (wd, ht))
-    if wd < ht:
-        return 'vertical'
-    else:
-        return 'horizontal'
-
-def get_oriented_box(container, scrolled=True, fill=False):
-    orientation = get_orientation(container)
-
-    if orientation == 'vertical':
-        box1 = VBox()
-        box2 = VBox()
-    else:
-        box1 = HBox()
-        box2 = VBox()
-
-    box2.add_widget(box1, stretch=1)
-    if not fill:
-        box2.add_widget(Label(''), stretch=1)
-    if scrolled:
-        sw = ScrollArea()
-        sw.set_widget(box2)
-    else:
-        sw = box2
-
-    return box1, sw, orientation
-
-#END
+# END
