@@ -9,7 +9,7 @@ import sys
 import os.path
 import math
 
-from ginga.misc import Bunch
+from ginga.misc import Bunch, Callback
 import ginga.toolkit
 
 import gtk
@@ -215,7 +215,8 @@ class Notebook(WidgetMask, gtk.Notebook):
         gtk.Notebook.__init__(self, *args, **kwdargs)
 
     def set_group_id(self, id):
-        super(Notebook, self).set_group_name(str(id))
+        if hasattr(self, 'set_group_name'):
+            super(Notebook, self).set_group_name(str(id))
 
     def set_current_page(self, new_idx):
         old_idx = self.get_current_page()
@@ -263,6 +264,58 @@ class MultiDragDropTreeView(gtk.TreeView):
         self.defer_select=False
 
 
+class MDISubWindow(Callback.Callbacks):
+
+    def __init__(self, widget, label):
+        super(MDISubWindow, self).__init__()
+
+        self.widget = widget
+
+        vbox = gtk.VBox()
+        vbox.set_border_width(4)
+        hbox = gtk.HBox()
+        close = gtk.Button("x")
+        maxim = gtk.Button("^")
+        minim = gtk.Button("v")
+        hbox.pack_start(close, False, False, 0)
+        hbox.pack_start(minim, False, False, 0)
+        hbox.pack_start(maxim, False, False, 0)
+
+        evbox = gtk.EventBox()
+        evbox.add(label)
+        evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray90"))
+        self.label = evbox
+        self.evbox = evbox
+        hbox.pack_start(evbox, True, True, 2)
+
+        vbox.pack_start(hbox, False, False, 0)
+        vbox.pack_start(widget, True, True, 4)
+
+        # what size does the widget want to be?
+        self.x, self.y, wd, ht = widget.get_allocation()
+        ## wd = widget.get_preferred_width()
+        ## ht = widget.get_preferred_height()
+        ## wd, ht = widget.get_size_request()
+        self.width, self.height = max(wd, 300), max(ht, 300)
+
+        frame = gtk.EventBox()
+        frame.set_size_request(self.width, self.height)
+        frame.props.visible_window = True
+        frame.set_border_width(0)
+        frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray70"))
+        self.frame = frame
+
+        frame.add(vbox)
+        frame.show_all()
+
+        for name in ('close', 'maximize', 'minimize'):
+            self.enable_callback(name)
+
+        maxim.connect('clicked', lambda *args: self.make_callback('maximize'))
+        minim.connect('clicked', lambda *args: self.make_callback('minimize'))
+        close.connect('clicked', lambda *args: self.make_callback('close'))
+
+
 class MDIWidget(gtk.Layout):
     """
     Multiple Document Interface type widget for Gtk.
@@ -278,6 +331,7 @@ class MDIWidget(gtk.Layout):
         self.kbdmouse_mask = 0
         self.cascade_offset = 50
         self.minimized_width = 150
+        self.delta_px = 50
 
         self.connect("motion_notify_event", self.motion_notify_event)
         self.connect("button_press_event", self.button_press_event)
@@ -299,56 +353,20 @@ class MDIWidget(gtk.Layout):
         self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray50"))
 
     def append_page(self, widget, label):
-        vbox = gtk.VBox()
-        vbox.set_border_width(4)
-        hbox = gtk.HBox()
-        close = gtk.Button("x")
-        maxim = gtk.Button("^")
-        minim = gtk.Button("v")
-        hbox.pack_start(close, False, False, 0)
-        hbox.pack_start(minim, False, False, 0)
-        hbox.pack_start(maxim, False, False, 0)
 
-        evbox = gtk.EventBox()
-        evbox.add(label)
-        evbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray90"))
-        hbox.pack_start(evbox, True, True, 2)
-
-        vbox.pack_start(hbox, fill=False, expand=False, padding=0)
-        vbox.pack_start(widget, fill=True, expand=True, padding=4)
-
-        # what size does the widget want to be?
-        x, y, wd, ht = widget.get_allocation()
-        ## wd = widget.get_preferred_width()
-        ## ht = widget.get_preferred_height()
-        ## wd, ht = widget.get_size_request()
-        wd, ht = max(wd, 300), max(ht, 300)
-
-        frame = gtk.EventBox()
-        frame.set_size_request(wd, ht)
-        frame.props.visible_window = True
-        frame.set_border_width(0)
-        frame.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("gray70"))
-
-        frame.add(vbox)
-        frame.show_all()
-
-        pos = self.get_widget_position(frame)
-        if pos is None:
-            x, y = 0, 0
-        else:
-            x, y = pos
-        wd, ht = self.get_widget_size(frame)
-        subwin = Bunch.Bunch(widget=widget, label=evbox, frame=frame,
-                             x=x, y=y, width=wd, height=ht)
+        subwin = MDISubWindow(widget, label)
         self.children.append(subwin)
 
-        evbox.connect("button_press_event", self.select_child_cb, subwin)
-        frame.connect("button_press_event", self.start_resize_cb, subwin)
-        maxim.connect('clicked', lambda *args: self.maximize_page(subwin))
-        minim.connect('clicked', lambda *args: self.minimize_page(subwin))
+        subwin.evbox.connect("button_press_event", self.select_child_cb, subwin)
+        subwin.frame.connect("button_press_event", self.start_resize_cb, subwin)
+        subwin.add_callback('maximize', lambda *args: self.maximize_page(subwin))
+        subwin.add_callback('minimize', lambda *args: self.minimize_page(subwin))
+
+        self.put(subwin.frame, self.cascade_offset, self.cascade_offset)
+
+        self.update_subwin_position(subwin)
+        self.update_subwin_size(subwin)
 
-        self.put(frame, self.cascade_offset, self.cascade_offset)
         return subwin
 
     def set_tab_reorderable(self, w, tf):
@@ -427,7 +445,7 @@ class MDIWidget(gtk.Layout):
         self.put(frame, subwin.x, subwin.y)
 
     def select_child_cb(self, layout, event, subwin):
-        ex, ey = event.x_root, event.y_root
+        x_root, y_root = event.x_root, event.y_root
 
         x, y = self.get_widget_position(subwin.frame)
         subwin.x, subwin.y = x, y
@@ -438,27 +456,110 @@ class MDIWidget(gtk.Layout):
             self.set_current_page(idx)
 
         self.selected_child = Bunch.Bunch(subwin=subwin, action='move',
-                                          x_origin=x, y_origin=y, dx=ex, dy=ey)
+                                          x_origin=x, y_origin=y,
+                                          x_root=x_root, y_root=y_root)
 
         return True
 
     def start_resize_cb(self, widget, event, subwin):
-        ex, ey = event.x_root, event.y_root
-        x, y = self.get_widget_position(subwin.frame)
-        subwin.x, subwin.y = x, y
+        self.update_subwin_size(subwin)
+
+        x_root, y_root = event.x_root, event.y_root
+        x, y = widget.translate_coordinates(self, int(event.x), int(event.y))
+
+        rect = subwin.frame.get_allocation()
+        x1, y1, wd, ht = rect.x, rect.y, rect.width, rect.height
+        x2, y2 = x1 + wd, y1 + ht
+        subwin.x, subwin.y = x1, y1
+        subwin.width, subwin.height = wd, ht
+
+        updates = set([])
+        if abs(x - x2) < self.delta_px:
+            # right side
+            if abs(y - y2) < self.delta_px:
+                # lower right corner
+                origin = 'lr'
+                updates = set(['w', 'h'])
+            elif abs(y - y1) < self.delta_px:
+                origin = 'ur'
+                updates = set(['w', 'h', 'y'])
+            else:
+                origin = 'r'
+                updates = set(['w'])
+        elif abs(x - x1) < self.delta_px:
+            # left side
+            if abs(y - y2) < self.delta_px:
+                # lower left corner
+                origin = 'll'
+                updates = set(['w', 'h', 'x'])
+            elif abs(y - y1) < self.delta_px:
+                origin = 'ul'
+                updates = set(['w', 'h', 'x', 'y'])
+            else:
+                origin = 'l'
+                updates = set(['w', 'x'])
+        elif abs(y - y2) < self.delta_px:
+            # bottom
+            origin = 'b'
+            updates = set(['h'])
+        else:
+            origin = 't'
+            updates = set(['h', 'y'])
+
         self.selected_child = Bunch.Bunch(subwin=subwin, action='resize',
-                                          x_origin=x, y_origin=y, dx=ex, dy=ey)
+                                          x_origin=x1, y_origin=y1,
+                                          wd=wd, ht=ht,
+                                          x_root=x_root, y_root=y_root,
+                                          origin=origin, updates=updates)
         return True
 
     def button_press_event(self, widget, event):
-        x = event.x; y = event.y
+        x, y = event.x, event.y
         button = self.kbdmouse_mask
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
         return True
 
+    def _resize(self, bnch, x_root, y_root):
+        subwin = bnch.subwin
+        updates = bnch.updates
+
+        dx, dy = x_root - bnch.x_root, y_root - bnch.y_root
+
+        wd = bnch.wd
+        if 'w' in updates:
+            wd = int(wd + dx)
+        ht = bnch.ht
+        if 'h' in updates:
+            ht = int(ht + dy)
+
+        if 'x' in updates or 'y' in updates:
+            x = bnch.x_origin
+            if 'x' in updates:
+                x = int(x + dx)
+                if x < bnch.x_origin:
+                    wd = int(bnch.wd + abs(dx))
+                else:
+                    wd = int(bnch.wd + -abs(dx))
+
+            y = bnch.y_origin
+            if 'y' in updates:
+                y = int(y + dy)
+                if y < bnch.y_origin:
+                    ht = int(bnch.ht + abs(dy))
+                else:
+                    ht = int(bnch.ht + -abs(dy))
+
+            # this works better if it is not self.move_page()
+            self.move(subwin.frame, x, y)
+
+        if 'w' in updates or 'h' in updates:
+            # this works better if it is not self.resize_page()
+            subwin.frame.set_size_request(wd, ht)
+
+
     def button_release_event(self, widget, event):
-        x = event.x_root; y = event.y_root
+        x_root, y_root = event.x_root, event.y_root
         button = self.kbdmouse_mask
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
@@ -466,21 +567,27 @@ class MDIWidget(gtk.Layout):
             bnch = self.selected_child
             subwin = bnch.subwin
             if bnch.action == 'move':
-                x = int(subwin.x + (x - bnch.dx))
-                y = int(subwin.y + (y - bnch.dy))
+                x = int(subwin.x + (x_root - bnch.x_root))
+                y = int(subwin.y + (y_root - bnch.y_root))
                 self.move_page(subwin, x, y)
 
             elif bnch.action == 'resize':
-                wd = int(subwin.width + (x - bnch.dx))
-                ht = int(subwin.height + (y - bnch.dy))
-                self.resize_page(subwin, wd, ht)
+                self._resize(bnch, x_root, y_root)
+
+                self.update_subwin_position(subwin)
+                # NOTE: necessary for wrapped widget to remember position
+                self.move_page(subwin, subwin.x, subwin.y)
+
+                self.update_subwin_size(subwin)
+                # NOTE: necessary for wrapped widget to remember size
+                self.resize_page(subwin, subwin.width, subwin.height)
 
             self.selected_child = None
         return True
 
     def motion_notify_event(self, widget, event):
         button = self.kbdmouse_mask
-        x, y, state = event.x_root, event.y_root, event.state
+        x_root, y_root, state = event.x_root, event.y_root, event.state
 
         if state & gtk.gdk.BUTTON1_MASK:
             button |= 0x1
@@ -493,16 +600,13 @@ class MDIWidget(gtk.Layout):
             bnch = self.selected_child
             subwin = bnch.subwin
             if bnch.action == 'move':
-                x = int(subwin.x + (x - bnch.dx))
-                y = int(subwin.y + (y - bnch.dy))
+                x = int(subwin.x + (x_root - bnch.x_root))
+                y = int(subwin.y + (y_root - bnch.y_root))
                 # this works better if it is not self.move_page()
                 self.move(subwin.frame, x, y)
 
             elif bnch.action == 'resize':
-                wd = int(subwin.width + (x - bnch.dx))
-                ht = int(subwin.height + (y - bnch.dy))
-                # this works better if it is not self.resize_page()
-                subwin.frame.set_size_request(wd, ht)
+                self._resize(bnch, x_root, y_root)
 
         return True
 
@@ -573,6 +677,8 @@ class MDIWidget(gtk.Layout):
         self.move_page(subwin, x, height-ht)
         #self.lower_widget(subwin)
 
+    def close_page(self, subwin):
+        pass
 
 class FileSelection(object):
 
@@ -633,36 +739,45 @@ class DirectorySelection(FileSelection):
 class Timer(object):
     """Abstraction of a GUI-toolkit implemented timer."""
 
-    def __init__(self, ival_sec, expire_cb, data=None):
-        """Create a timer set to expire after `ival_sec` and which will
-        call the callable `expire_cb` when it expires.
+class Timer(Callback.Callbacks):
+    """Abstraction of a GUI-toolkit implemented timer."""
+
+    def __init__(self, duration=0.0):
+        """Create a timer set to expire after `duration` sec.
         """
-        self.ival_sec = ival_sec
-        self.cb = expire_cb
-        self.data = data
+        super(Timer, self).__init__()
+
+        self.duration = duration
+        # For storing aritrary data with timers
+        self.data = Bunch.Bunch()
+
         self._timer = None
 
-    def start(self, ival_sec=None):
-        """Start the timer.  If `ival_sec` is not None, it should
+        for name in ('expired', 'canceled'):
+            self.enable_callback(name)
+
+    def start(self, duration=None):
+        """Start the timer.  If `duration` is not None, it should
         specify the time to expiration in seconds.
         """
-        if ival_sec is None:
-            ival_sec = self.ival_sec
+        if duration is None:
+            duration = self.duration
 
-        self.cancel()
+        self.set(duration)
+
+    def set(self, duration):
+
+        self.stop()
 
         # Gtk timer set in milliseconds
-        time_ms = int(ival_sec * 1000.0)
+        time_ms = int(duration * 1000.0)
         self._timer = gobject.timeout_add(time_ms, self._redirect_cb)
 
     def _redirect_cb(self):
         self._timer = None
-        self.cb(self)
+        self.make_callback('expired')
 
-    def cancel(self):
-        """Cancel this timer.  If the timer is not running, there
-        is no error.
-        """
+    def stop(self):
         try:
             if self._timer is not None:
                 gobject.source_remove(self._timer)
@@ -670,6 +785,15 @@ class Timer(object):
         except:
             pass
 
+    def cancel(self):
+        """Cancel this timer.  If the timer is not running, there
+        is no error.
+        """
+        self.stop()
+        self.make_callback('canceled')
+
+    clear = cancel
+
 
 def combo_box_new_text():
     liststore = gtk.ListStore(gobject.TYPE_STRING)
