@@ -12,15 +12,35 @@ import gobject
 from ginga.misc import Callback, Bunch, LineHistory
 from functools import reduce
 
+has_webkit = False
+try:
+    import webkit  # noqa
+    has_webkit = True
+except ImportError:
+    pass
+
+__all__ = ['WidgetError', 'WidgetBase', 'TextEntry', 'TextEntrySet',
+           'TextArea', 'Label', 'Button', 'ComboBox',
+           'SpinBox', 'Slider', 'ScrollBar', 'CheckBox', 'ToggleButton',
+           'RadioButton', 'Image', 'ProgressBar', 'StatusBar', 'TreeView',
+           'WebView', 'ContainerBase', 'Box', 'HBox', 'VBox', 'Frame',
+           'Expander', 'TabWidget', 'StackWidget', 'MDIWidget', 'ScrollArea',
+           'Splitter', 'GridBox', 'Toolbar', 'MenuAction',
+           'Menu', 'Menubar', 'TopLevelMixin', 'TopLevel', 'Application',
+           'Dialog', 'SaveDialog', 'DragPackage', 'WidgetMoveEvent',
+           'name_mangle', 'make_widget', 'hadjust', 'build_info', 'wrap',
+           'has_webkit']
+
+
 class WidgetError(Exception):
     """For errors thrown in this module."""
     pass
 
-
 # (see TabWidget)
 _widget_move_event = None
 _app = None
 
+
 # BASE
 
 class WidgetBase(Callback.Callbacks):
@@ -38,6 +58,9 @@ class WidgetBase(Callback.Callbacks):
     def set_tooltip(self, text):
         self.widget.set_tooltip_text(text)
 
+    def get_enabled(self):
+        self.widget.get_sensitive()
+
     def set_enabled(self, tf):
         self.widget.set_sensitive(tf)
 
@@ -47,7 +70,8 @@ class WidgetBase(Callback.Callbacks):
             # NOTE: we don't seem to get accurate information unless
             # the window is actually mapped, thus the test
             rect = self.widget.get_allocation()
-            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+            wd, ht = rect.width, rect.height
+            # x, y = rect.x, rect.y
 
         else:
             # window maybe isn't realized yet--try other ways
@@ -59,7 +83,8 @@ class WidgetBase(Callback.Callbacks):
 
     def get_pos(self):
         rect = self.widget.get_allocation()
-        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        x, y = rect.x, rect.y
+        # width, height = rect.width, rect.height
         return x, y
 
     def get_app(self):
@@ -69,7 +94,7 @@ class WidgetBase(Callback.Callbacks):
         self.widget.destroy()
 
     def show(self):
-        #self.widget.show()
+        # self.widget.show()
         self.widget.show_all()
 
     def hide(self):
@@ -82,7 +107,7 @@ class WidgetBase(Callback.Callbacks):
         self.widget.set_size_request(width, height)
         # hackish way to allow the widget to be resized down again later
         # NOTE: for gtk2 we can't do this because it affects get_size()
-        #gobject.idle_add(self.widget.set_size_request, -1, -1)
+        # gobject.idle_add(self.widget.set_size_request, -1, -1)
 
     def get_font(self, font_family, point_size):
         font = GtkHelp.get_font(font_family, point_size)
@@ -92,6 +117,7 @@ class WidgetBase(Callback.Callbacks):
         # this is for compatibility with Qt widgets
         pass
 
+
 # BASIC WIDGETS
 
 class TextEntry(WidgetBase):
@@ -150,6 +176,7 @@ class TextEntry(WidgetBase):
         self.widget.set_width_chars(numchars)
         pass
 
+
 class TextEntrySet(WidgetBase):
     def __init__(self, text='', editable=True):
         super(TextEntrySet, self).__init__()
@@ -186,13 +213,14 @@ class TextEntrySet(WidgetBase):
         self.widget.modify_font(font)
 
     def set_length(self, numchars):
-        #self.widget.set_width_chars(numchars)
+        # self.widget.set_width_chars(numchars)
         pass
 
     def set_enabled(self, tf):
         super(TextEntrySet, self).set_enabled(tf)
         self.entry.set_sensitive(tf)
 
+
 class TextArea(WidgetBase):
     def __init__(self, wrap=False, editable=False):
         super(TextArea, self).__init__()
@@ -226,12 +254,12 @@ class TextArea(WidgetBase):
 
         end = buf.get_end_iter()
         mark = buf.get_insert()
-        #self.tw.scroll_to_iter(end, 0.5)
+        # self.tw.scroll_to_iter(end, 0.5)
         # NOTE: this was causing a segfault if the text widget is
         # not mapped yet!  Seems to be fixed in recent versions of
         # gtk
         buf.move_mark(mark, end)
-        res = self.tw.scroll_to_mark(mark, 0.2, True)
+        res = self.tw.scroll_to_mark(mark, 0.2, True)  # noqa
 
     def get_text(self):
         buf = self.tw.get_buffer()
@@ -244,7 +272,7 @@ class TextArea(WidgetBase):
         if numlines > self.histlimit:
             rmcount = int(numlines - self.histlimit)
             start = buf.get_iter_at_line(0)
-            end   = buf.get_iter_at_line(rmcount)
+            end = buf.get_iter_at_line(rmcount)
             buf.delete(start, end)
 
     def clear(self):
@@ -273,6 +301,7 @@ class TextArea(WidgetBase):
         else:
             self.tw.set_wrap_mode(gtk.WRAP_NONE)
 
+
 class Label(WidgetBase):
     def __init__(self, text='', halign='left', style='normal', menu=None):
         super(Label, self).__init__()
@@ -293,6 +322,8 @@ class Label(WidgetBase):
 
         evbox.connect("button_press_event", self._cb_redirect)
         self.enable_callback('activated')
+        evbox.connect("button_release_event", self._cb_redirect2)
+        self.enable_callback('released')
 
         self.label = label
         self.menu = menu
@@ -307,15 +338,21 @@ class Label(WidgetBase):
             self.widget = fr
 
     def _cb_redirect(self, widget, event):
-        # event.button, event.x, event.y
         if event.button == 1:
             self.make_callback('activated')
-            return True
+            return False
 
         elif event.button == 3 and self.menu is not None:
             menu_w = self.menu.get_widget()
-            return menu_w.popup(None, None, None,
-                                event.button, event.time)
+            if menu_w.get_sensitive():
+                return menu_w.popup(None, None, None,
+                                    event.button, event.time)
+        return False
+
+    def _cb_redirect2(self, widget, event):
+        if event.button == 1:
+            self.make_callback('released')
+            return True
         return False
 
     def get_text(self):
@@ -333,6 +370,7 @@ class Label(WidgetBase):
         if fg is not None:
             self.label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(fg))
 
+
 class Button(WidgetBase):
     def __init__(self, text=''):
         super(Button, self).__init__()
@@ -426,7 +464,7 @@ class SpinBox(WidgetBase):
         super(SpinBox, self).__init__()
 
         self.widget = GtkHelp.SpinButton()
-        #self.widget.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
+        # self.widget.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
         self.widget.sconnect('value-changed', self._cb_redirect)
 
         self.enable_callback('value-changed')
@@ -598,8 +636,9 @@ class Image(WidgetBase):
 
             elif event.button == 3 and self.menu is not None:
                 menu_w = self.menu.get_widget()
-                return menu_w.popup(None, None, None,
-                                    event.button, event.time)
+                if menu_w.get_sensitive():
+                    return menu_w.popup(None, None, None,
+                                        event.button, event.time)
 
     def _cb_redirect2(self, widget, event):
         if event.type == gtk.gdk.BUTTON_RELEASE:
@@ -610,14 +649,15 @@ class Image(WidgetBase):
     def _set_image(self, native_image):
         self.image.set_from_pixbuf(native_image.get_pixbuf())
 
+
 class ProgressBar(WidgetBase):
     def __init__(self):
         super(ProgressBar, self).__init__()
 
         w = gtk.ProgressBar()
         # GTK3
-        #w.set_orientation(gtk.ORIENTATION_HORIZONTAL)
-        #w.set_inverted(False)
+        # w.set_orientation(gtk.ORIENTATION_HORIZONTAL)
+        # w.set_inverted(False)
         self.widget = w
 
     def set_value(self, pct):
@@ -625,6 +665,7 @@ class ProgressBar(WidgetBase):
         self.widget.set_fraction(pct)
         self.widget.set_text("%.2f %%" % (pct * 100.0))
 
+
 class StatusBar(WidgetBase):
     def __init__(self):
         super(StatusBar, self).__init__()
@@ -680,7 +721,7 @@ class TreeView(WidgetBase):
         if self.dragable:
             tv = GtkHelp.MultiDragDropTreeView()
             # enable drag from this widget
-            toImage = [ ( "text/plain", 0, 0 ) ]
+            toImage = [("text/plain", 0, 0)]
             tv.enable_model_drag_source(gtk.gdk.BUTTON1_MASK,
                                         toImage, gtk.gdk.ACTION_COPY)
             tv.connect("drag-data-get", self._start_drag)
@@ -712,8 +753,8 @@ class TreeView(WidgetBase):
         # create the column headers
         if not isinstance(columns[0], str):
             # columns specifies a mapping
-            headers = [ col[0] for col in columns ]
-            datakeys = [ col[1] for col in columns ]
+            headers = [col[0] for col in columns]
+            datakeys = [col[1] for col in columns]
         else:
             headers = datakeys = columns
 
@@ -744,7 +785,7 @@ class TreeView(WidgetBase):
                 tvc.set_clickable(True)
             if n == 0:
                 fn_data = self._mkcolfn0(kwd)
-                ## cell.set_property('xalign', 1.0)
+                # cell.set_property('xalign', 1.0)
             else:
                 fn_data = self._mkcolfnN(kwd)
             tvc.set_cell_data_func(cell, fn_data)
@@ -888,7 +929,7 @@ class TreeView(WidgetBase):
         treeselection.select_iter(item)
 
     def highlight_path(self, path, onoff, font_color='green'):
-        item = self._path_to_item(path)
+        item = self._path_to_item(path)  # noqa
         # TODO
 
     def _path_to_item(self, path):
@@ -935,13 +976,13 @@ class TreeView(WidgetBase):
             bnch2 = model.get_value(iter2, 0)
             if isinstance(bnch1, str):
                 if isinstance(bnch2, str):
-                    return cmp(bnch1.lower(), bnch2.lower())
+                    return bnch1.lower() == bnch2.lower()  # was using cmp
                 return 0
             val1, val2 = bnch1[idx], bnch2[idx]
             if isinstance(val1, str):
                 val1 = val1.lower()
                 val2 = val2.lower()
-            res = cmp(val1, val2)
+            res = val1 == val2  # was using nonexistent cmp
             return res
         return fn
 
@@ -974,12 +1015,40 @@ class TreeView(WidgetBase):
         return fn
 
     def _start_drag(self, treeview, context, selection,
-                         info, timestamp):
+                    info, timestamp):
         res_dict = self.get_selected()
         drag_pkg = DragPackage(self.tv, selection)
         self.make_callback('drag-start', drag_pkg, res_dict)
         drag_pkg.start_drag()
 
+
+class WebView(WidgetBase):
+    def __init__(self):
+        if not has_webkit:
+            raise NotImplementedError("Missing webkit")
+
+        super(WebView, self).__init__()
+        self.widget = webkit.WebView()
+
+    def load_url(self, url):
+        self.widget.open(url)
+
+    def load_html_string(self, html_string):
+        self.widget.load_string(html_string, 'text/html', 'utf-8', 'file://')
+
+    def go_back(self):
+        self.widget.go_back()
+
+    def go_forward(self):
+        self.widget.go_forward()
+
+    def reload_page(self):
+        self.widget.reload()
+
+    def stop_loading(self):
+        self.widget.stop_loading()
+
+
 # CONTAINERS
 
 class ContainerBase(WidgetBase):
@@ -997,7 +1066,7 @@ class ContainerBase(WidgetBase):
             childw.destroy()
 
     def remove(self, w, delete=False):
-        if not w in self.children:
+        if w not in self.children:
             raise KeyError("Widget is not a child of this container")
         self.children.remove(w)
 
@@ -1052,10 +1121,12 @@ class Box(ContainerBase):
         self.widget.pack_start(child_w, expand=expand, fill=True)
         self.widget.show_all()
 
+
 class VBox(Box):
     def __init__(self):
         super(VBox, self).__init__(orientation='vertical')
 
+
 class HBox(Box):
     def __init__(self):
         super(HBox, self).__init__(orientation='horizontal')
@@ -1100,7 +1171,7 @@ class TabWidget(ContainerBase):
         self.detachable = detachable
 
         nb = GtkHelp.Notebook()
-        #nb = gtk.Notebook()
+        # nb = gtk.Notebook()
         nb.set_show_border(False)
         nb.set_scrollable(True)
         # Allows drag-and-drop between notebooks
@@ -1110,7 +1181,7 @@ class TabWidget(ContainerBase):
         nb.connect("page-added", self._tab_insert_cb)
         nb.connect("page-removed", self._tab_remove_cb)
         nb.sconnect("switch-page", self._cb_redirect)
-        #nb.connect("switch-page", self._cb_redirect)
+        # nb.connect("switch-page", self._cb_redirect)
         self.widget = nb
         self.set_tab_position(tabpos)
 
@@ -1134,7 +1205,7 @@ class TabWidget(ContainerBase):
         # (native widget already has been removed by gtk)
         self.children.remove(child)
 
-        #nchild_w.unparent()
+        # nchild_w.unparent()
         self.make_callback('page-detach', child)
 
     def _tab_insert_cb(self, nbw, nchild_w, page_num):
@@ -1204,15 +1275,17 @@ class TabWidget(ContainerBase):
         else:
             evbox.modify_bg(gtk.STATE_NORMAL, None)
 
+
 class StackWidget(TabWidget):
     def __init__(self):
         super(StackWidget, self).__init__()
 
         nb = self.widget
-        #nb.set_scrollable(False)
+        # nb.set_scrollable(False)
         nb.set_show_tabs(False)
         nb.set_show_border(False)
 
+
 class MDIWidget(ContainerBase):
 
     def __init__(self, tabpos='top', mode='tabs'):
@@ -1237,6 +1310,9 @@ class MDIWidget(ContainerBase):
         sw.set_vadjustment(self.mdi_w.get_vadjustment())
         sw.add(self.mdi_w)
 
+        for name in ('page-switch', 'page-close'):
+            self.enable_callback(name)
+
     def get_mode(self):
         return self.mode
 
@@ -1248,7 +1324,7 @@ class MDIWidget(ContainerBase):
         child_w = child.get_widget()
         label = gtk.Label(title)
         subwin = self.mdi_w.append_page(child_w, label)
-        #self.mdi_w.show_all()
+        # self.mdi_w.show_all()
         # attach title to child
         child.extdata.tab_title = title
 
@@ -1264,6 +1340,8 @@ class MDIWidget(ContainerBase):
             x, y = pos
             self.mdi_w.move_page(subwin, x, y)
 
+        subwin.add_callback('close', self._window_close, child)
+
     def _remove(self, childw, delete=False):
         self.mdi_w.remove(childw)
         if delete:
@@ -1291,6 +1369,9 @@ class MDIWidget(ContainerBase):
         child.extdata.mdi_pos = (x, y)
         return True
 
+    def _window_close(self, subwin, child):
+        return self.make_callback('page-close', child)
+
     def get_index(self):
         return self.mdi_w.get_current_page()
 
@@ -1314,6 +1395,7 @@ class MDIWidget(ContainerBase):
     def use_tabs(self, tf):
         pass
 
+
 class ScrollArea(ContainerBase):
     def __init__(self):
         super(ScrollArea, self).__init__()
@@ -1328,7 +1410,8 @@ class ScrollArea(ContainerBase):
 
     def _resize_cb(self, widget, allocation):
         rect = widget.get_allocation()
-        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        # x, y = rect.x, rect.y
+        width, height = rect.width, rect.height
         self.make_callback('configure', width, height)
         return True
 
@@ -1447,8 +1530,8 @@ class GridBox(ContainerBase):
         self.add_ref(child)
         w = child.get_widget()
         if stretch > 0:
-            xoptions = gtk.EXPAND|gtk.FILL
-            yoptions = gtk.EXPAND|gtk.FILL
+            xoptions = gtk.EXPAND | gtk.FILL
+            yoptions = gtk.EXPAND | gtk.FILL
         else:
             xoptions = gtk.FILL
             yoptions = gtk.FILL
@@ -1493,10 +1576,16 @@ class Toolbar(ContainerBase):
         w = child.get_widget()
         self.widget.append_widget(w, None, None)
 
-    def add_menu(self, text, menu=None):
+    def add_menu(self, text, menu=None, mtype='tool'):
         if menu is None:
             menu = Menu()
-        child = self.add_action(text)
+        if mtype == 'tool':
+            child = self.add_action(text)
+        else:
+            child = Label(text, style='clickable', menu=menu)
+            self.add_widget(child)
+            child.add_callback('released', lambda w: menu.hide())
+
         child.add_callback('activated', lambda w: menu.popup())
         return menu
 
@@ -1543,18 +1632,32 @@ class Menu(ContainerBase):
 
         self.widget = gtk.Menu()
         self.widget.show()
+        self.menus = Bunch.Bunch(caseless=True)
 
     def add_widget(self, child):
         menuitem_w = child.get_widget()
         self.widget.append(menuitem_w)
         self.add_ref(child)
-        #self.widget.show_all()
+        # self.widget.show_all()
 
     def add_name(self, name, checkable=False):
         child = MenuAction(text=name, checkable=checkable)
         self.add_widget(child)
         return child
 
+    def add_menu(self, name):
+        item_w = gtk.MenuItem(label=name)
+        child = Menu()
+        self.add_ref(child)
+        self.menus[name] = child
+        item_w.set_submenu(child.get_widget())
+        self.widget.append(item_w)
+        item_w.show()
+        return child
+
+    def get_menu(self, name):
+        return self.menus[name]
+
     def add_separator(self):
         sep = gtk.SeparatorMenuItem()
         self.widget.append(sep)
@@ -1564,10 +1667,11 @@ class Menu(ContainerBase):
         menu = self.widget
         menu.show_all()
         if six.PY2:
-            now = long(0)
+            now = long(0)  # noqa
         else:
             now = int(0)
-        menu.popup(None, None, None, 0, now)
+        if menu.get_sensitive():
+            menu.popup(None, None, None, 0, now)
 
 
 class Menubar(ContainerBase):
@@ -1608,7 +1712,7 @@ class TopLevelMixin(object):
         self.widget.connect("window_state_event", self._window_event)
         self.widget.connect("configure-event", self._configure_event)
 
-        if not title is None:
+        if title is not None:
             self.widget.set_title(title)
 
         self.enable_callback('close')
@@ -1632,7 +1736,7 @@ class TopLevelMixin(object):
 
     def _window_event(self, widget, event):
         if ((event.changed_mask & gtk.gdk.WINDOW_STATE_FULLSCREEN) or
-            (event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED)):
+                (event.changed_mask & gtk.gdk.WINDOW_STATE_MAXIMIZED)):
             self._fullscreen = True
         else:
             self._fullscreen = False
@@ -1644,11 +1748,11 @@ class TopLevelMixin(object):
         return False
 
     def close(self):
-        ## try:
-        ##     self.widget.destroy()
-        ## except Exception as e:
-        ##     pass
-        #self.widget = None
+        # try:
+        #     self.widget.destroy()
+        # except Exception as e:
+        #     pass
+        # self.widget = None
 
         self.make_callback('close')
 
@@ -1656,7 +1760,8 @@ class TopLevelMixin(object):
         window = self.widget.get_window()
         if window is not None:
             rect = self.widget.get_allocation()
-            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+            # x, y = rect.x, rect.y
+            wd, ht = rect.width, rect.height
 
         else:
             # window maybe isn't realized yet--try other ways
@@ -1761,8 +1866,8 @@ class Application(Callback.Callbacks):
         except:
             self.screen_wd = 1600
             self.screen_ht = 1200
-        ## self.logger.debug("screen dimensions %dx%d" % (
-        ##     self.screen_wd, self.screen_ht))
+        # self.logger.debug("screen dimensions %dx%d" % (
+        #     self.screen_wd, self.screen_ht))
 
         _app = self
 
@@ -1777,7 +1882,7 @@ class Application(Callback.Callbacks):
 
     def process_events(self):
         while gtk.events_pending():
-            #gtk.main_iteration(False)
+            # gtk.main_iteration(False)
             gtk.main_iteration()
 
     def process_end(self):
@@ -1880,7 +1985,8 @@ class SaveDialog(object):
 
         if response == gtk.RESPONSE_OK:
             path = self.widget.get_filename()
-            if self.selectedfilter is not None and not path.endswith(self.selectedfilter):
+            if (self.selectedfilter is not None and
+                    not path.endswith(self.selectedfilter)):
                 path += self.selectedfilter
             self.widget.destroy()
             return path
@@ -1888,6 +1994,7 @@ class SaveDialog(object):
             self.widget.destroy()
             return None
 
+
 class DragPackage(object):
     def __init__(self, src_widget, selection):
         self.src_widget = src_widget
@@ -1900,6 +2007,7 @@ class DragPackage(object):
     def start_drag(self):
         pass
 
+
 class WidgetMoveEvent(object):
     def __init__(self, src_widget, child):
         self.src_widget = src_widget
@@ -1912,6 +2020,7 @@ class WidgetMoveEvent(object):
     def reject(self):
         self._result = False
 
+
 # MODULE FUNCTIONS
 
 def name_mangle(name, pfx=''):
@@ -1923,6 +2032,7 @@ def name_mangle(name, pfx=''):
             newname.append(c)
     return pfx + ''.join(newname)
 
+
 def make_widget(title, wtype):
     if wtype == 'label':
         w = Label(title)
@@ -1932,7 +2042,7 @@ def make_widget(title, wtype):
         w.label.set_alignment(0.05, 0.95)
     elif wtype == 'entry':
         w = TextEntry()
-        #w.get_widget().set_width_chars(12)
+        # w.get_widget().set_width_chars(12)
     elif wtype == 'entryset':
         w = TextEntrySet()
     elif wtype == 'combobox':
@@ -2028,36 +2138,4 @@ def wrap(native_widget):
     return wrapper
 
 
-def get_orientation(container):
-    if not hasattr(container, 'size'):
-        return 'vertical'
-    (wd, ht) = container.size
-    ## wd, ht = container.get_size()
-    #print('container size is %dx%d' % (wd, ht))
-    if wd < ht:
-        return 'vertical'
-    else:
-        return 'horizontal'
-
-def get_oriented_box(container, scrolled=True, fill=False):
-    orientation = get_orientation(container)
-
-    if orientation == 'vertical':
-        box1 = VBox()
-        box2 = VBox()
-    else:
-        box1 = HBox()
-        box2 = VBox()
-
-    box2.add_widget(box1, stretch=1)
-    if not fill:
-        box2.add_widget(Label(''), stretch=1)
-    if scrolled:
-        sw = ScrollArea()
-        sw.set_widget(box2)
-    else:
-        sw = box2
-
-    return box1, sw, orientation
-
-#END
+# END
