@@ -138,11 +138,7 @@ class caselessDict(object):
         return repr(self)
 
     def copy(self):
-        d={}
-        for k,v in self.dict.items():
-            d[k]=v[1]
-        return d
-
+        return caselessDict(self.dict)
 
 
 class Bunch(object):
@@ -310,7 +306,7 @@ class Bunch(object):
         return self.tbl.values()
 
     def copy(self):
-        return self.tbl.copy()
+        return Bunch(inDict=self.tbl)
 
 
 class threadSafeBunch(object):
@@ -517,6 +513,10 @@ class threadSafeBunch(object):
             return self.tbl.items()
 
 
+    def copy(self):
+        with self.lock:
+            return threadSafeBunch(inDict=self.tbl)
+
     def get(self, key, alt=None):
         """If dictionary contains _key_ return the associated value,
         otherwise return _alt_.
