@@ -2,9 +2,6 @@
 # transform.py -- a custom projection for supporting matplotlib plotting
 #                          on ginga
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c)  Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
@@ -211,6 +208,7 @@ class GingaAxes(Axes):
         input_dims = 2
         output_dims = 2
         is_separable = False
+        has_inverse = True
         viewer = None
         #pass_through = True
 
@@ -225,12 +223,11 @@ class GingaAxes(Axes):
 
             The input and output are Nx2 numpy arrays.
             """
-            #print(("transform in:", xy))
             if self.viewer is None:
                 return xy
 
-            res = np.dstack(self.viewer.get_canvas_xy(xy.T[0], xy.T[1]))[0]
-            #print(("transform out:", res))
+            tr = self.viewer.tform['data_to_native']
+            res = tr.to_(xy)
             return res
 
         # This is where things get interesting.  With this projection,
@@ -271,15 +268,15 @@ class GingaAxes(Axes):
         input_dims = 2
         output_dims = 2
         is_separable = False
+        has_inverse = True
         viewer = None
 
         def transform_non_affine(self, xy):
-            #print "transform in:", xy
             if self.viewer is None:
                 return xy
 
-            res = np.dstack(self.viewer.get_data_xy(xy.T[0], xy.T[1]))[0]
-            #print "transform out:", res
+            tr = self.viewer.tform['data_to_native']
+            res = tr.from_(xy)
             return res
 
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
