@@ -37,8 +37,8 @@ class RenderGraphicsView(QtGui.QGraphicsView):
         if not self.pixmap:
             return
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         # redraw the screen from backing pixmap
         rect = QtCore.QRect(x1, y1, width, height)
@@ -47,8 +47,8 @@ class RenderGraphicsView(QtGui.QGraphicsView):
     def resizeEvent(self, event):
         rect = self.geometry()
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         self.viewer.configure_window(width, height)
 
@@ -79,8 +79,8 @@ class RenderWidget(QtGui.QWidget):
             return
         rect = event.rect()
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         # redraw the screen from backing pixmap
         painter = QPainter(self)
@@ -90,8 +90,8 @@ class RenderWidget(QtGui.QWidget):
     def resizeEvent(self, event):
         rect = self.geometry()
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         self.viewer.configure_window(width, height)
         #self.update()
@@ -125,14 +125,18 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.imgwin.viewer = self
         self.pixmap = None
         # Qt expects 32bit BGRA data for color images
-        self._rgb_order = 'BGRA'
+        self.rgb_order = 'BGRA'
 
         self.renderer = CanvasRenderer(self)
 
-        self.msgtimer = Timer(0.0, lambda timer: self.onscreen_message_off())
+        self.msgtimer = Timer()
+        self.msgtimer.add_callback('expired',
+                                   lambda timer: self.onscreen_message_off())
 
         # For optomized redrawing
-        self._defer_task = Timer(0.0, lambda timer: self.delayed_redraw())
+        self._defer_task = Timer()
+        self._defer_task.add_callback('expired',
+                                      lambda timer: self.delayed_redraw())
 
     def get_widget(self):
         return self.imgwin
@@ -171,7 +175,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.logger.debug("drawing to pixmap")
 
         # Prepare array for rendering
-        arr = rgbobj.get_array(self._rgb_order)
+        arr = rgbobj.get_array(self.rgb_order)
         (height, width) = arr.shape[:2]
 
         return self._render_offscreen(self.pixmap, arr, dst_x, dst_y,
@@ -213,8 +217,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         qbuf.close()
         return ibuf
 
-    def get_rgb_image_as_widget(self, output=None, format='png',
-                                quality=90):
+    def get_rgb_image_as_widget(self):
         imgwin_wd, imgwin_ht = self.get_window_size()
         qpix = self.pixmap.copy(0, 0,
                                 imgwin_wd, imgwin_ht)
@@ -228,7 +231,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        arr = self.getwin_array(order=self._rgb_order)
+        arr = self.getwin_array(order=self.rgb_order)
         image = self._get_qimage(arr)
         return image
 
@@ -240,7 +243,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         res = qimg.save(filepath, format=format, quality=quality)
 
     def reschedule_redraw(self, time_sec):
-        self._defer_task.cancel()
+        self._defer_task.stop()
         self._defer_task.start(time_sec)
 
     def update_image(self):
@@ -254,7 +257,6 @@ class ImageViewQt(ImageView.ImageViewBase):
                                   QtGui.QGraphicsScene.BackgroundLayer)
         else:
             self.imgwin.update()
-            #self.imgwin.show()
 
     def set_cursor(self, cursor):
         if self.imgwin is not None:
@@ -288,8 +290,8 @@ class ImageViewQt(ImageView.ImageViewBase):
         cursor = self.imgwin.cursor()
         cursor.setPos(s_pt)
 
-    def get_rgb_order(self):
-        return self._rgb_order
+    def make_timer(self):
+        return Timer()
 
     def _get_qimage(self, bgra):
         h, w, channels = bgra.shape
@@ -306,7 +308,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         return clr
 
     def onscreen_message(self, text, delay=None, redraw=True):
-        self.msgtimer.cancel()
+        self.msgtimer.stop()
         self.set_onscreen_message(text, redraw=redraw)
         if delay is not None:
             self.msgtimer.start(delay)
@@ -384,21 +386,11 @@ class RenderWidgetZoom(RenderMixin, RenderWidget):
 class RenderGraphicsViewZoom(RenderMixin, RenderGraphicsView):
     pass
 
-class ImageViewEvent(ImageViewQt):
 
-    def __init__(self, logger=None, rgbmap=None, settings=None, render=None):
-        ImageViewQt.__init__(self, logger=logger, rgbmap=rgbmap,
-                             settings=settings, render=render)
+class QtEventMixin(object):
 
-        # replace the widget our parent provided
-        if self.wtype == 'scene':
-            imgwin = RenderGraphicsViewZoom()
-            imgwin.setScene(self.scene)
-        else:
-            imgwin = RenderWidgetZoom()
-
-        imgwin.viewer = self
-        self.imgwin = imgwin
+    def __init__(self):
+        imgwin = self.imgwin
         imgwin.setFocusPolicy(QtCore.Qt.FocusPolicy(
                               QtCore.Qt.TabFocus |
                               QtCore.Qt.ClickFocus |
@@ -421,7 +413,7 @@ class ImageViewEvent(ImageViewQt):
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
-                               ('pick', 'thinCrossCursor.png')):
+                                  ('pick', 'thinCrossCursor.png')):
             path = os.path.join(icondir, filename)
             cur = self.make_cursor(path, 8, 8)
             self.define_cursor(curname, cur)
@@ -445,9 +437,13 @@ class ImageViewEvent(ImageViewQt):
         for name in ('motion', 'button-press', 'button-release',
                      'key-press', 'key-release', 'drag-drop',
                      'scroll', 'map', 'focus', 'enter', 'leave',
-                     'pinch', 'pan', 'swipe', 'tap'):
+                     'pinch', 'pan', #'swipe', 'tap'
+                     ):
             self.enable_callback(name)
 
+    def set_enter_focus(self, tf):
+        self.enter_focus = tf
+
     def transkey(self, keycode, keyname):
         self.logger.debug("keycode=%d keyname='%s'" % (
             keycode, keyname))
@@ -499,14 +495,11 @@ class ImageViewEvent(ImageViewQt):
     def get_keyTable(self):
         return self._keytbl
 
-    def set_enter_focus(self, tf):
-        self.enter_focus = tf
-
     def map_event(self, widget, event):
         rect = widget.geometry()
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         self.configure_window(width, height)
         return self.make_callback('map')
@@ -551,8 +544,7 @@ class ImageViewEvent(ImageViewQt):
             button |= 0x4
         self.logger.debug("button down event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
@@ -570,8 +562,7 @@ class ImageViewEvent(ImageViewQt):
         if buttons & QtCore.Qt.RightButton:
             button |= 0x4
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
@@ -598,8 +589,7 @@ class ImageViewEvent(ImageViewQt):
         if buttons & QtCore.Qt.RightButton:
             button |= 0x4
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('motion', button, data_x, data_y)
 
@@ -607,13 +597,30 @@ class ImageViewEvent(ImageViewQt):
         x, y = event.x(), event.y()
         self.last_win_x, self.last_win_y = x, y
 
+        data_x, data_y = self.check_cursor_location()
+
+        # NOTE: for future use in distinguishing mouse wheel vs.
+        # trackpad events
+        src = 'wheel'
+        if hasattr(event, 'source'):
+            # Qt5 only, it seems
+            _src = event.source()
+            if _src == QtCore.Qt.MouseEventNotSynthesized:
+                src = 'wheel'
+            else:
+                src = 'trackpad'
+                point = event.pixelDelta()
+                dx, dy = point.x(), point.y()
+
+                # Synthesize this as a pan gesture event
+                self.make_ui_callback('pan', 'start', 0, 0)
+                self.make_ui_callback('pan', 'move', dx, dy)
+                return self.make_ui_callback('pan', 'stop', 0, 0)
+
         num_degrees, direction = get_scroll_info(event)
         self.logger.debug("scroll deg={} direction={}".format(
             num_degrees, direction))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
-
         return self.make_ui_callback('scroll', direction, num_degrees,
                                      data_x, data_y)
 
@@ -625,9 +632,9 @@ class ImageViewEvent(ImageViewQt):
         elif state == QtCore.Qt.GestureUpdated:
             gstate = 'move'
         elif state == QtCore.Qt.GestureFinished:
-            gstate = 'end'
+            gstate = 'stop'
         elif state == QtCore.Qt.GestureCancelled:
-            gstate = 'end'
+            gstate = 'stop'
 
         # dispatch on gesture type
         gtype = event.gesture(QtCore.Qt.SwipeGesture)
@@ -653,7 +660,7 @@ class ImageViewEvent(ImageViewQt):
         return True
 
     def gs_swiping(self, event, gesture, gstate):
-        if gstate == 'end':
+        if gstate == 'stop':
             _hd = gesture.horizontalDirection()
             hdir = None
             if _hd == QtGui.QSwipeGesture.Left:
@@ -674,7 +681,6 @@ class ImageViewEvent(ImageViewQt):
             return self.make_ui_callback('swipe', gstate, hdir, vdir)
 
     def gs_pinching(self, event, gesture, gstate):
-        #print("PINCHING")
         rot = gesture.rotationAngle()
         scale = gesture.scaleFactor()
         self.logger.debug("pinch gesture rot=%f scale=%f state=%s" % (
@@ -683,13 +689,6 @@ class ImageViewEvent(ImageViewQt):
         return self.make_ui_callback('pinch', gstate, rot, scale)
 
     def gs_panning(self, event, gesture, gstate):
-        #print("PANNING")
-        # x, y = event.x(), event.y()
-        # self.last_win_x, self.last_win_y = x, y
-
-        # data_x, data_y = self.get_data_xy(x, y)
-        # self.last_data_x, self.last_data_y = data_x, data_y
-
         d = gesture.delta()
         dx, dy = d.x(), d.y()
         self.logger.debug("pan gesture dx=%f dy=%f state=%s" % (
@@ -735,6 +734,26 @@ class ImageViewEvent(ImageViewQt):
         #event.acceptProposedAction()
         self.make_ui_callback('drag-drop', data)
 
+
+class ImageViewEvent(QtEventMixin, ImageViewQt):
+
+    def __init__(self, logger=None, rgbmap=None, settings=None, render=None):
+        ImageViewQt.__init__(self, logger=logger, rgbmap=rgbmap,
+                             settings=settings, render=render)
+
+        # replace the widget our parent provided
+        if self.wtype == 'scene':
+            imgwin = RenderGraphicsViewZoom()
+            imgwin.setScene(self.scene)
+        else:
+            imgwin = RenderWidgetZoom()
+
+        imgwin.viewer = self
+        self.imgwin = imgwin
+
+        QtEventMixin.__init__(self)
+
+
 class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
 
     # class variables for binding map and bindings can be set
@@ -756,7 +775,7 @@ class ImageViewZoom(Mixins.UIMixin, ImageViewEvent):
                                 rgbmap=rgbmap, render=render)
         Mixins.UIMixin.__init__(self)
 
-        self.ui_setActive(True)
+        self.ui_set_active(True)
 
         if bindmap is None:
             bindmap = ImageViewZoom.bindmapClass(self.logger)
@@ -815,8 +834,10 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         self._adjusting = False
         self._scrolling = False
         self.pad = 20
-        self.upper_h = 100
-        self.upper_v = 100
+        self.range_h = 10000
+        self.range_v = 10000
+        self.upper_h = self.range_h
+        self.upper_v = self.range_v
 
         # reparent the viewer widget
         vp = self.viewport()
@@ -825,10 +846,10 @@ class ScrolledView(QtGui.QAbstractScrollArea):
 
         hsb = self.horizontalScrollBar()
         hsb.setTracking(True)
-        hsb.setRange(0, 100)
+        hsb.setRange(0, self.upper_h)
         hsb.setSingleStep(1)
         vsb = self.verticalScrollBar()
-        vsb.setRange(0, 100)
+        vsb.setRange(0, self.upper_v)
         vsb.setSingleStep(1)
         vsb.setTracking(True)
 
@@ -845,8 +866,8 @@ class ScrolledView(QtGui.QAbstractScrollArea):
         vp = self.viewport()
         rect = vp.geometry()
         x1, y1, x2, y2 = rect.getCoords()
-        width = x2 - x1
-        height = y2 - y1
+        width = x2 - x1 + 1
+        height = y2 - y1 + 1
 
         self.v_w.resize(width, height)
 
@@ -868,12 +889,13 @@ class ScrolledView(QtGui.QAbstractScrollArea):
             hsb = self.horizontalScrollBar()
             vsb = self.verticalScrollBar()
 
-            page_h, page_v = (int(round(res.thm_pct_x * 100)),
-                              int(round(res.thm_pct_y * 100)))
+            page_h, page_v = (int(round(res.thm_pct_x * self.range_h)),
+                              int(round(res.thm_pct_y * self.range_v)))
             hsb.setPageStep(page_h)
             vsb.setPageStep(page_v)
 
-            upper_h, upper_v = max(1, 100 - page_h), max(1, 100 - page_v)
+            upper_h = max(1, self.range_h - page_h)
+            upper_v = max(1, self.range_v - page_v)
             hsb.setRange(0, upper_h)
             vsb.setRange(0, upper_v)
             self.upper_h, self.upper_v = upper_h, upper_v
@@ -895,12 +917,22 @@ class ScrolledView(QtGui.QAbstractScrollArea):
 
         self._scrolling = True
         try:
-            hsb = self.horizontalScrollBar()
-            vsb = self.verticalScrollBar()
+            bd = self.viewer.get_bindings()
+            res = bd.calc_pan_pct(self.viewer, pad=self.pad)
+            if res is None:
+                return
+            pct_x, pct_y = res.pan_pct_x, res.pan_pct_y
 
-            pct_x = hsb.value() / float(self.upper_h)
-            # invert Y pct because of orientation of scrollbar
-            pct_y = 1.0 - (vsb.value() / float(self.upper_v))
+            # Only adjust pan setting for axes that have changed
+            if dx != 0:
+                hsb = self.horizontalScrollBar()
+                pos_x = float(hsb.value())
+                pct_x = pos_x / float(self.upper_h)
+            if dy != 0:
+                vsb = self.verticalScrollBar()
+                pos_y = float(vsb.value())
+                # invert Y pct because of orientation of scrollbar
+                pct_y = 1.0 - (pos_y / float(self.upper_v))
 
             bd = self.viewer.get_bindings()
             bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
