@@ -8,6 +8,7 @@
 from __future__ import print_function
 
 # stdlib imports
+import glob
 import sys
 import os
 import logging
@@ -20,7 +21,7 @@ from ginga.misc.Bunch import Bunch
 from ginga.misc import Task, ModuleManager, Settings, log
 import ginga.version as version
 import ginga.toolkit as ginga_toolkit
-from ginga.util import paths
+from ginga.util import paths, rgb_cms
 
 # Catch warnings
 logging.captureWarnings(True)
@@ -28,7 +29,7 @@ logging.captureWarnings(True)
 __all__ = ['ReferenceViewer']
 
 default_layout = ['seq', {},
-                   ['vbox', dict(name='top', width=1520, height=900),
+                   ['vbox', dict(name='top', width=1400, height=700),
                     dict(row=['hbox', dict(name='menu')],
                          stretch=0),
                     dict(row=['hpanel', dict(name='hpnl'),
@@ -37,12 +38,12 @@ default_layout = ['seq', {},
                       # (tabname, layout), ...
                       [("Info", ['vpanel', {},
                                  ['ws', dict(name='uleft', wstype='stack',
-                                             height=300, group=3)],
+                                             height=250, group=3)],
                                  ['ws', dict(name='lleft', wstype='tabs',
-                                             height=430, group=3)],
+                                             height=330, group=3)],
                                  ]
                         )]],
-                     ['vbox', dict(name='main', width=700),
+                     ['vbox', dict(name='main', width=600),
                       dict(row=['ws', dict(name='channels', wstype='tabs',
                                            group=1, use_toolbar=True)],
                            stretch=1),
@@ -69,57 +70,78 @@ default_layout = ['seq', {},
                     ]]
 
 global_plugins = [
-    Bunch(module='Toolbar', tab='Toolbar', ws='toolbar', start=True),
-    Bunch(module='Pan', tab='_pan', ws='uleft', start=True, raisekey=None),
-    Bunch(module='Info', tab='Synopsis', ws='lleft', start=True,
-          raisekey=None),
-    Bunch(module='Header', tab='Header', ws='left', start=True, raisekey='H'),
-    Bunch(module='Zoom', tab='Zoom', ws='left', start=True, raisekey='Z'),
-    Bunch(module='Command', tab='Command', ws='lleft', start=True),
-    Bunch(module='Thumbs', tab='Thumbs', ws='right', start=True, raisekey='T'),
-    Bunch(module='Contents', tab='Contents', ws='right', start=True,
-          raisekey='c'),
-    Bunch(module='Colorbar', tab='_cbar', ws='cbar', start=True),
-    Bunch(module='Cursor', tab='_readout', ws='readout', start=True),
-    Bunch(module='Operations', tab='_opns', ws='operations', start=True),
-    Bunch(module='WBrowser', tab='Help', ws='channels', raisekey='?',
-          start=False),
-    Bunch(module='FBrowser', tab='Open File', ws='right', start=False),
-    Bunch(module='ColorMapPicker', tab='Color Map Picker', ws='right',
-          start=False),
+    Bunch(module='Operations', workspace='operations', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Toolbar', workspace='toolbar', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Pan', workspace='uleft', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Info', tab='Synopsis', workspace='lleft', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Header', tab='Header', workspace='left', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Zoom', tab='Zoom', workspace='left', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Thumbs', tab='Thumbs', workspace='right', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Contents', tab='Contents', workspace='right', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Colorbar', workspace='cbar', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Cursor', workspace='readout', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Errors', tab='Errors', workspace='right', start=True,
+          hidden=True, category='system'),
+    Bunch(module='Command', tab='Command', workspace='lleft', start=False,
+          category='Global'),
+    Bunch(module='WBrowser', tab='Help', workspace='channels', start=False,
+          category='Global'),
+    Bunch(module='FBrowser', tab='Open File', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='Blink', tab='Blink Channels', workspace='right', start=False,
+          menu="Blink Channels", category='Global'),
+    Bunch(module='ColorMapPicker', tab='Color Map Picker', workspace='right',
+          start=False, category='Global'),
     # TODO: Move SaveImage to File menu.
-    Bunch(module='SaveImage', tab='Save File', ws='right', start=False),
-    Bunch(module='Errors', tab='Errors', ws='right', start=True),
-    Bunch(module='RC', tab='RC', ws='right', start=False),
-    Bunch(module='WCSMatch', tab='WCSMatch', ws='right', start=False),
-    Bunch(module='ChangeHistory', tab='History', ws='right', start=False),
-    Bunch(module='SAMP', tab='SAMP', ws='right', start=False),
-    Bunch(module='IRAF', tab='IRAF', ws='right', start=False),
-    Bunch(module='Log', tab='Log', ws='right', start=False),
+    Bunch(module='SaveImage', tab='Save File', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='RC', tab='RC', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='WCSMatch', tab='WCSMatch', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='ChangeHistory', tab='History', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='SAMP', tab='SAMP', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='IRAF', tab='IRAF', workspace='right', start=False,
+          category='Global'),
+    Bunch(module='Log', tab='Log', workspace='right', start=False,
+          category='Global'),
     ]
 
 local_plugins = [
-    Bunch(module='Pick', ws='dialogs', shortkey='f1'),
-    Bunch(module='Ruler', ws='dialogs', shortkey='f2'),
-    Bunch(module='MultiDim', ws='lleft', shortkey='f4'),
-    Bunch(module='Cuts', ws='dialogs', shortkey='f5'),
-    Bunch(module='Histogram', ws='dialogs', shortkey='f6'),
-    Bunch(module='Crosshair', ws='dialogs'),
-    Bunch(module='Overlays', ws='dialogs'),
-    Bunch(module='TVMark', ws='dialogs'),
-    Bunch(module='TVMask', ws='dialogs'),
-    Bunch(module='Blink', ws='dialogs'),
-    Bunch(module='LineProfile', ws='dialogs'),
-    Bunch(module='PixTable', ws='dialogs', shortkey='f7'),
-    Bunch(module='Preferences', ws='dialogs', shortkey='f9'),
-    Bunch(module='Catalogs', ws='dialogs', shortkey='f10'),
-    Bunch(module='Mosaic', ws='dialogs'),
-    Bunch(module='Drawing', ws='dialogs', shortkey='f11'),
-    Bunch(module='FBrowser', ws='dialogs', shortkey='f12'),
-    Bunch(module='Compose', ws='dialogs'),
-    Bunch(module='PlotTable', ws='dialogs'),
-    # Not ready for prime time
-    #Bunch(module='Pipeline', ws='dialogs'),
+    Bunch(module='Pick', workspace='dialogs', category=None),
+    Bunch(module='Ruler', workspace='dialogs', category=None),
+    Bunch(module='MultiDim', workspace='lleft', category=None),
+    Bunch(module='Cuts', workspace='dialogs', category=None),
+    Bunch(module='Histogram', workspace='dialogs', category=None),
+    Bunch(module='Crosshair', workspace='dialogs', category=None),
+    Bunch(module='Overlays', workspace='dialogs', category=None),
+    Bunch(module='WCSAxes', workspace='dialogs', category=None),
+    Bunch(module='TVMark', workspace='dialogs', category=None),
+    Bunch(module='TVMask', workspace='dialogs', category=None),
+    Bunch(module='Blink', workspace='dialogs', menu='Blink Images',
+          category=None),
+    Bunch(module='LineProfile', workspace='dialogs', category=None),
+    Bunch(module='PixTable', workspace='dialogs', category=None),
+    Bunch(module='Preferences', workspace='dialogs', category=None),
+    Bunch(module='Catalogs', workspace='dialogs', category=None),
+    Bunch(module='Mosaic', workspace='dialogs', category=None),
+    Bunch(module='Drawing', workspace='dialogs', category=None),
+    Bunch(module='FBrowser', workspace='dialogs', category=None),
+    Bunch(module='Compose', workspace='dialogs', category=None),
+    Bunch(module='PlotTable', workspace='dialogs', category=None),
+    Bunch(module='ScreenShot', workspace='dialogs', category=None),
     ]
 
 
@@ -133,17 +155,30 @@ class ReferenceViewer(object):
         self.global_plugins = []
         self.layout = layout
 
-    def add_local_plugin(self, module_name, ws_name, pfx=None):
-        self.local_plugins.append(
-            Bunch(module=module_name, ws=ws_name, pfx=pfx))
+    def add_local_plugin_spec(self, spec):
+        self.local_plugins.append(spec)
+
+    def add_global_plugin_spec(self, spec):
+        self.global_plugins.append(spec)
+
+    def add_local_plugin(self, module_name, ws_name,
+                         path=None, klass=None, pfx=None, category=None):
+        """TO BE DEPRECATED--DO NOT USE.
+        Use add_local_plugin_spec() instead.
+        """
+        self.add_local_plugin_spec(
+            Bunch(module=module_name, workspace=ws_name, category=category,
+                  path=path, klass=klass, pfx=pfx))
 
     def add_global_plugin(self, module_name, ws_name,
+                          path=None, klass=None, category='Global',
                           tab_name=None, start_plugin=True, pfx=None):
-        if tab_name is None:
-            tab_name = module_name
-
-        self.global_plugins.append(
-            Bunch(module=module_name, ws=ws_name, tab=tab_name,
+        """TO BE DEPRECATED--DO NOT USE.
+        Use add_global_plugin_spec() instead.
+        """
+        self.add_global_plugin_spec(
+            Bunch(module=module_name, workspace=ws_name, tab=tab_name,
+                  path=path, klass=klass, category=category,
                   start=start_plugin, pfx=pfx))
 
     def clear_default_plugins(self):
@@ -156,19 +191,44 @@ class ReferenceViewer(object):
         reference viewer.
         """
         # add default global plugins
-        for bnch in global_plugins:
-            if bnch.module not in except_global:
-                start = bnch.get('start', True)
-                pfx = bnch.get('pfx', None)
-                self.add_global_plugin(bnch.module, bnch.ws,
-                                       tab_name=bnch.tab, start_plugin=start,
-                                       pfx=pfx)
+        for spec in global_plugins:
+            if spec.module not in except_global:
+                self.add_global_plugin_spec(spec)
 
         # add default local plugins
-        for bnch in local_plugins:
-            if bnch.module not in except_local:
-                pfx = bnch.get('pfx', None)
-                self.add_local_plugin(bnch.module, bnch.ws, pfx=pfx)
+        for spec in local_plugins:
+            if spec.module not in except_local:
+                self.add_local_plugin_spec(spec)
+
+    def add_separately_distributed_plugins(self):
+        from pkg_resources import iter_entry_points
+        groups = ['ginga.rv.plugins']
+        available_methods = []
+
+        for group in groups:
+            for entry_point in iter_entry_points(group=group, name=None):
+                try:
+                    method = entry_point.load()
+                    available_methods.append(method)
+
+                except Exception as e:
+                    print("Error trying to load entry point %s: %s" % (
+                        str(entry_point), str(e)))
+
+        for method in available_methods:
+            try:
+                spec = method()
+                if 'start' in spec:
+                    # global plugin
+                    # TODO: need a better differentiator
+                    self.add_global_plugin_spec(spec)
+                else:
+                    # local plugin
+                    self.add_local_plugin_spec(spec)
+
+            except Exception as e:
+                print("Error trying to instantiate external plugin using %s: %s" % (
+                    str(method), str(e)))
 
     def add_default_options(self, optprs):
         """
@@ -178,7 +238,7 @@ class ReferenceViewer(object):
         optprs.add_option("--bufsize", dest="bufsize", metavar="NUM",
                           type="int", default=10,
                           help="Buffer length to NUM")
-        optprs.add_option("--channels", dest="channels", default="Image",
+        optprs.add_option('-c', "--channels", dest="channels", default="Image",
                           help="Specify list of channels to create")
         optprs.add_option("--debug", dest="debug", default=False,
                           action="store_true",
@@ -213,6 +273,9 @@ class ReferenceViewer(object):
         optprs.add_option("--profile", dest="profile", action="store_true",
                           default=False,
                           help="Run the profiler on main()")
+        optprs.add_option("--sep", dest="separate_channels", default=False,
+                          action="store_true",
+                          help="Load files in separate channels")
         optprs.add_option("-t", "--toolkit", dest="toolkit", metavar="NAME",
                           default=None,
                           help="Prefer GUI toolkit (gtk|qt)")
@@ -242,19 +305,21 @@ class ReferenceViewer(object):
             try:
                 os.mkdir(basedir)
             except OSError as e:
-                logger.warning("Couldn't create ginga settings area (%s): %s" % (
-                    basedir, str(e)))
+                logger.warning(
+                    "Couldn't create ginga settings area (%s): %s" % (
+                        basedir, str(e)))
                 logger.warning("Preferences will not be able to be saved")
 
         # Set up preferences
         prefs = Settings.Preferences(basefolder=basedir, logger=logger)
-        settings = prefs.createCategory('general')
+        settings = prefs.create_category('general')
         settings.load(onError='silent')
-        settings.setDefaults(useMatplotlibColormaps=False,
-                             widgetSet='choose',
-                             WCSpkg='choose', FITSpkg='choose',
-                             recursion_limit=2000,
-                             save_layout=False)
+        settings.set_defaults(useMatplotlibColormaps=False,
+                              widgetSet='choose',
+                              WCSpkg='choose', FITSpkg='choose',
+                              recursion_limit=2000,
+                              icc_working_profile=None,
+                              save_layout=False)
 
         # default of 1000 is a little too small
         sys.setrecursionlimit(settings.get('recursion_limit'))
@@ -298,13 +363,10 @@ class ReferenceViewer(object):
 
         if toolkit == 'choose':
             try:
-                from ginga.qtw import QtHelp  # noqa
-            except ImportError:
-                try:
-                    from ginga.gtkw import GtkHelp  # noqa
-                except ImportError:
-                    print("You need python-gtk or python-qt to run Ginga!")
-                    sys.exit(1)
+                ginga_toolkit.choose()
+            except ImportError as e:
+                print("UI toolkit choose error: %s" % str(e))
+                sys.exit(1)
         else:
             ginga_toolkit.use(toolkit)
 
@@ -325,6 +387,10 @@ class ReferenceViewer(object):
                 logger.warning(
                     "failed to load matplotlib colormaps: %s" % (str(e)))
 
+        # Set a working RGB ICC profile if user has one
+        working_profile = settings.get('icc_working_profile', None)
+        rgb_cms.working_profile = working_profile
+
         # User wants to customize the WCS package?
         if options.wcspkg:
             wcspkg = options.wcspkg
@@ -436,16 +502,16 @@ class ReferenceViewer(object):
         # Load any custom modules
         if options.modules:
             modules = options.modules.split(',')
-            for longPluginName in modules:
-                if '.' in longPluginName:
-                    tmpstr = longPluginName.split('.')
-                    pluginName = tmpstr[-1]
+            for long_plugin_name in modules:
+                if '.' in long_plugin_name:
+                    tmpstr = long_plugin_name.split('.')
+                    plugin_name = tmpstr[-1]
                     pfx = '.'.join(tmpstr[:-1])
                 else:
-                    pluginName = longPluginName
+                    plugin_name = long_plugin_name
                     pfx = None
-                spec = Bunch(name=pluginName, module=pluginName,
-                             tab=pluginName, ws='right', pfx=pfx)
+                spec = Bunch(name=plugin_name, module=plugin_name,
+                             tab=plugin_name, workspace='right', pfx=pfx)
                 ginga_shell.add_global_plugin(spec)
 
         # Load modules for "local" (per-channel) plug ins
@@ -456,15 +522,15 @@ class ReferenceViewer(object):
         # Load any custom plugins
         if options.plugins:
             plugins = options.plugins.split(',')
-            for longPluginName in plugins:
-                if '.' in longPluginName:
-                    tmpstr = longPluginName.split('.')
-                    pluginName = tmpstr[-1]
+            for long_plugin_name in plugins:
+                if '.' in long_plugin_name:
+                    tmpstr = long_plugin_name.split('.')
+                    plugin_name = tmpstr[-1]
                     pfx = '.'.join(tmpstr[:-1])
                 else:
-                    pluginName = longPluginName
+                    plugin_name = long_plugin_name
                     pfx = None
-                spec = Bunch(module=pluginName, ws='dialogs',
+                spec = Bunch(module=plugin_name, workspace='dialogs',
                              hidden=False, pfx=pfx)
                 ginga_shell.add_local_plugin(spec)
 
@@ -521,9 +587,30 @@ class ReferenceViewer(object):
         if (not options.nosplash) and (len(args) == 0) and showBanner:
             ginga_shell.banner(raiseTab=True)
 
-        # Assume remaining arguments are fits files and load them.
+        # Handle inputs like "*.fits[ext]" that sys cmd cannot auto expand.
+        expanded_args = []
         for imgfile in args:
-            ginga_shell.nongui_do(ginga_shell.load_file, imgfile)
+            if '*' in imgfile:
+                if '[' in imgfile and imgfile.endswith(']'):
+                    s = imgfile.split('[')
+                    ext = '[' + s[1]
+                else:
+                    ext = ''
+                for fname in glob.iglob(s[0]):
+                    expanded_args.append(fname + ext)
+            else:
+                expanded_args.append(imgfile)
+
+        # Assume remaining arguments are fits files and load them.
+        chname = None
+        for imgfile in expanded_args:
+            if options.separate_channels and (chname is not None):
+                channel = ginga_shell.add_channel_auto()
+            else:
+                channel = ginga_shell.get_channel_info()
+            chname = channel.name
+            ginga_shell.nongui_do(ginga_shell.load_file, imgfile,
+                                  chname=chname)
 
         try:
             try:
@@ -550,6 +637,7 @@ def reference_viewer(sys_argv):
     """Create reference viewer from command line."""
     viewer = ReferenceViewer(layout=default_layout)
     viewer.add_default_plugins()
+    viewer.add_separately_distributed_plugins()
 
     # Parse command line options with optparse module
     from optparse import OptionParser
