@@ -6,22 +6,43 @@
 #
 import os
 import math
-import numpy
+import numpy as np
 from collections import OrderedDict
 
 from ginga.misc import Bunch
 from ginga import GingaPlugin
 from ginga import cmap, imap
 from ginga.util import wcs
-from ginga.util.six.moves import map, zip
+from ginga.util.six.moves import map
 from ginga.gw import ColorBar, Widgets
 
 
 class Catalogs(GingaPlugin.LocalPlugin):
+    """
+    Catalogs
+    ========
+    A plugin for plotting object locations from a catalog on an image.
 
+    Plugin Type: Local
+    ------------------
+    Catalogs is a local plugin, which means it is associated with a
+    channel.  An instance can be opened for each channel.
+
+    Usage
+    -----
+    TBD
+    """
     def __init__(self, fv, fitsimage):
         super(Catalogs, self).__init__(fv, fitsimage)
 
+        prefs = self.fv.get_preferences()
+        self.settings = prefs.create_category('plugin_Catalogs')
+        self.settings.add_defaults(draw_type='circle',
+                                   select_color='skyblue',
+                                   color_outline='aquamarine',
+                                   click_radius=10)
+        self.settings.load(onError='silent')
+
         self.limit_stars_to_area = False
         self.pan_to_selected = False
         self.use_dss_channel = False
@@ -29,21 +50,17 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.plot_max = 500
         self.plot_limit = 100
         self.plot_start = 0
-        self.drawtype = 'circle'
+        self.drawtype = self.settings.get('draw_type', 'circle')
 
         # star list
         self.starlist = []
         # catalog listing
         self.table = None
 
-        self.color_outline = 'aquamarine'
+        self.color_outline = self.settings.get('color_outline', 'aquamarine')
         self.layertag = 'catalog-canvas'
         self.areatag = None
 
-        prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_Catalogs')
-        self.settings.load(onError='silent')
-
         self.image_server_options = []
         self.image_server_params = None
 
@@ -53,32 +70,22 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.dc = fv.get_draw_classes()
         canvas = self.dc.DrawingCanvas()
         canvas.enable_draw(True)
-        canvas.set_drawtype(self.drawtype, color='cyan', linestyle='dash',
-                            #drawdims=True
-                            )
-        canvas.set_callback('cursor-down', self.btndown)
-        canvas.set_callback('cursor-up', self.btnup)
+        canvas.enable_edit(True)
+        canvas.set_drawtype(self.drawtype, color='cyan', linestyle='dash')
         canvas.set_callback('draw-event', self.draw_cb)
+        canvas.set_callback('edit-event', self.edit_cb)
+        canvas.register_for_cursor_drawing(self.fitsimage)
         canvas.set_surface(self.fitsimage)
+        canvas.set_draw_mode('draw')
         self.canvas = canvas
 
-        self.color_selected = 'skyblue'
+        self.color_selected = self.settings.get('select_color', 'skyblue')
         self.hilite = None
         self.gui_up = False
 
     def build_gui(self, container, future=None):
         vbox1 = Widgets.VBox()
 
-        msg_font = self.fv.get_font("sansFont", 14)
-        tw = Widgets.TextArea()
-        tw.set_font(msg_font)
-        tw.set_wrap(True)
-        self.tw = tw
-
-        fr = Widgets.Expander("Instructions")
-        fr.set_widget(tw)
-        vbox1.add_widget(fr, stretch=0)
-
         nb = Widgets.TabWidget(tabpos='bottom')
         self.w.nb = nb
         vbox1.add_widget(nb, stretch=1)
@@ -103,7 +110,8 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.w.get_image.add_callback('activated',
                                       lambda w: self.getimage_cb())
         self.w.use_dss_channel.set_state(self.use_dss_channel)
-        self.w.use_dss_channel.add_callback('activated', self.use_dss_channel_cb)
+        self.w.use_dss_channel.add_callback(
+            'activated', self.use_dss_channel_cb)
 
         vbox.add_widget(w, stretch=0)
 
@@ -146,7 +154,8 @@ class Catalogs(GingaPlugin.LocalPlugin):
 
         combobox = self.w2.server
         index = 0
-        self.catalog_server_options = self.fv.imgsrv.getServerNames(kind='catalog')
+        self.catalog_server_options = self.fv.imgsrv.getServerNames(
+            kind='catalog')
         for name in self.catalog_server_options:
             combobox.append_text(name)
             index += 1
@@ -166,7 +175,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
         btn1 = Widgets.RadioButton("Rectangle")
         btn1.set_state(self.drawtype == 'rectangle')
         btn1.add_callback('activated',
-                         lambda w, tf: self.set_drawtype_cb(tf, 'rectangle'))
+                          lambda w, tf: self.set_drawtype_cb(tf, 'rectangle'))
         btns.add_widget(btn1, stretch=0)
         btn2 = Widgets.RadioButton("Circle", group=btn1)
         btn2.set_state(self.drawtype == 'circle')
@@ -178,6 +187,36 @@ class Catalogs(GingaPlugin.LocalPlugin):
         btns.add_widget(btn, stretch=0)
         vbox0.add_widget(btns, stretch=0)
 
+        mode = self.canvas.get_draw_mode()
+        btns = Widgets.HBox()
+        btns.set_spacing(5)
+
+        btn1 = Widgets.RadioButton("Select")
+        btn1.set_state(mode == 'select')
+        btn1.add_callback('activated',
+                          lambda w, val: self.set_mode_cb('select', val))
+        btn1.set_tooltip("Choose this to highlight selection on table")
+        self.w.btn_select = btn1
+        btns.add_widget(btn1, stretch=0)
+
+        btn2 = Widgets.RadioButton("Draw", group=btn1)
+        btn2.set_state(mode == 'draw')
+        btn2.add_callback('activated',
+                          lambda w, val: self.set_mode_cb('draw', val))
+        btn2.set_tooltip("Choose this to define search region")
+        self.w.btn_draw = btn2
+        btns.add_widget(btn2, stretch=0)
+
+        btn3 = Widgets.RadioButton("Edit", group=btn1)
+        btn3.set_state(mode == 'edit')
+        btn3.add_callback('activated', lambda w, val: self.set_mode_cb('edit', val))
+        btn3.set_tooltip("Choose this to edit or move a region")
+        self.w.btn_edit = btn3
+        btns.add_widget(btn3)
+
+        btns.add_widget(Widgets.Label(''), stretch=1)
+        vbox0.add_widget(btns, stretch=0)
+
         self.w.params = vbox0
 
         sw = Widgets.ScrollArea()
@@ -228,6 +267,11 @@ class Catalogs(GingaPlugin.LocalPlugin):
             btn.add_callback('activated', lambda w: self.cancel())
             btns.add_widget(btn, stretch=0)
 
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
+        btns.add_widget(Widgets.Label(''), stretch=1)
+
         vbox1.add_widget(btns, stretch=0)
 
         container.add_widget(vbox1, stretch=1)
@@ -247,12 +291,10 @@ class Catalogs(GingaPlugin.LocalPlugin):
         return True
 
     def start(self, future=None):
-        self.instructions()
         # start catalog operation
         p_canvas = self.fitsimage.get_canvas()
         try:
-            obj = p_canvas.get_object_by_tag(self.layertag)
-
+            p_canvas.get_object_by_tag(self.layertag)
         except KeyError:
             # Add canvas layer
             p_canvas.add(self.canvas, tag=self.layertag)
@@ -264,20 +306,23 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.resume()
 
     def pause(self):
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
 
     def resume(self):
         # turn off any mode user may be in
         self.modes_off()
 
-        self.canvas.ui_setActive(True)
+        self.canvas.ui_set_active(True)
         #self.fv.show_status("Draw a rectangle with the right mouse button")
 
     def stop(self):
         # stop catalog operation
-        self.clearAll()
+        try:
+            self.clear_all()
+        except:
+            pass
         # remove the canvas from the image
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
         p_canvas = self.fitsimage.get_canvas()
         try:
             p_canvas.delete_object_by_tag(self.layertag)
@@ -291,15 +336,18 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.fv.show_status("")
 
     def redo(self):
-        obj = self.canvas.get_object_by_tag(self.areatag)
-        if not obj.kind in ('rectangle', 'circle'):
+        try:
+            obj = self.canvas.get_object_by_tag(self.areatag)
+        except Exception:
+            return True
+        if obj.kind not in ('rectangle', 'circle'):
             return True
 
         try:
             image = self.fitsimage.get_image()
 
             if obj.kind == 'rectangle':
-                # if  the object drawn is a rectangle, calculate the radius
+                # if the object drawn is a rectangle, calculate the radius
                 # of a circle necessary to cover the area
                 # calculate center of bbox
                 x1, y1, x2, y2 = obj.get_llur()
@@ -329,9 +377,9 @@ class Catalogs(GingaPlugin.LocalPlugin):
             else:
                 # if the object drawn is a circle, calculate the box
                 # enclosed by the circle
-                ctr_x, ctr_y = obj.crdmap.to_data(obj.x, obj.y)
+                ctr_x, ctr_y = obj.crdmap.to_data((obj.x, obj.y))
                 ra_ctr, dec_ctr = image.pixtoradec(ctr_x, ctr_y)
-                dst_x, dst_y = obj.crdmap.to_data(obj.x + obj.radius, obj.y)
+                dst_x, dst_y = obj.crdmap.to_data((obj.x + obj.radius, obj.y))
                 ra_dst, dec_dst = image.pixtoradec(dst_x, dst_y)
                 radius_deg = wcs.deltaStarsRaDecDeg(ra_ctr, dec_ctr,
                                                     ra_dst, dec_dst)
@@ -367,25 +415,37 @@ class Catalogs(GingaPlugin.LocalPlugin):
             return True
 
         # Copy the image parameters out to the widget
-        d = { 'ra': ra_ctr, 'dec': dec_ctr, 'width': str(wd),
-              'height': ht, 'r': radius, 'r2': radius,
-              'r1': 0.0,
-              }
+        d = {'ra': ra_ctr, 'dec': dec_ctr, 'width': str(wd),
+             'height': ht, 'r': radius, 'r2': radius,
+             'r1': 0.0}
         self._update_widgets(d)
         return True
 
-    def btndown(self, canvas, event, data_x, data_y):
+    def edit_select_region(self):
+        if self.areatag is not None:
+            obj = self.canvas.get_object_by_tag(self.areatag)
+            self.canvas.edit_select(obj)
+        else:
+            self.canvas.clear_selected()
+        self.canvas.update_canvas()
+
+    def set_mode_cb(self, mode, tf):
+        """Called when one of the Select/Draw/Edit radio buttons is selected."""
+        if tf:
+            if mode == 'select':
+                mode = 'pick'
+            self.canvas.set_draw_mode(mode)
+            if mode == 'edit':
+                self.edit_select_region()
         return True
 
-    def btnup(self, canvas, event, data_x, data_y):
-
-        objs = self.canvas.get_items_at(data_x, data_y)
-        for obj in objs:
-            if (obj.tag is not None) and obj.tag.startswith('star'):
-                info = obj.get_data()
-                self.table.show_selection(info.star)
-                return True
-        return True
+    def set_mode(self, mode):
+        if mode == 'select':
+            mode = 'pick'
+        self.canvas.set_draw_mode(mode)
+        self.w.btn_select.set_state(mode == 'pick')
+        self.w.btn_draw.set_state(mode == 'draw')
+        self.w.btn_edit.set_state(mode == 'edit')
 
     def highlight_object(self, obj, tag, color, redraw=True):
         x = obj.objects[0].x
@@ -394,6 +454,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
         radius = obj.objects[0].radius + delta
 
         hilite = self.dc.Circle(x, y, radius, linewidth=4, color=color)
+        hilite.editable = False
         self.hilite.add_object(hilite)
         if redraw:
             self.canvas.update_canvas()
@@ -420,23 +481,20 @@ class Catalogs(GingaPlugin.LocalPlugin):
         if redraw:
             self.canvas.update_canvas()
 
-
     def setfromimage(self):
         image = self.fitsimage.get_image()
         if image is None:
             return
         x1, y1 = 0, 0
         x2, y2 = self.fitsimage.get_data_size()
-        Rectangle = self.canvas.getDrawClass('Rectangle')
-        tag = self.canvas.add(Rectangle(x1, y1, x2, y2,
-                                        color=self.color_outline))
+        tag = self.canvas.add(self.dc.Rectangle(x1, y1, x2, y2,
+                                                color=self.color_outline))
 
         self.draw_cb(self.canvas, tag)
 
-
     def draw_cb(self, canvas, tag):
         obj = canvas.get_object_by_tag(tag)
-        if not obj.kind in ('rectangle', 'circle'):
+        if obj.kind not in ('rectangle', 'circle'):
             return True
 
         if self.areatag:
@@ -454,13 +512,24 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self._raise_tab(self.w.params)
         return self.redo()
 
+    def edit_cb(self, canvas, obj):
+        if self.areatag is not None:
+            obj2 = canvas.get_object_by_tag(self.areatag)
+            if obj != obj2:
+                # Not editing the area
+                return
+
+        # Raise the params tab
+        self._raise_tab(self.w.params)
+        return self.redo()
+
     def getimage_cb(self):
         params = self.get_params(self.image_server_params)
 
         index = self._get_cbidx(self.w.server)
         server = self.image_server_options[index]
 
-        self.clearAll()
+        self.clear_all()
 
         if self.use_dss_channel:
             chname = 'DSS'
@@ -571,6 +640,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self._raise_tab(self.w.listing)
 
         self._update_plotscroll()
+        self.set_mode('select')
 
     def filter_results(self, starlist, filter_obj):
         image = self.fitsimage.get_image()
@@ -579,19 +649,19 @@ class Catalogs(GingaPlugin.LocalPlugin):
         if filter_obj:
             num_cat = len(starlist)
             self.logger.debug("number of incoming stars=%d" % (num_cat))
-            # TODO: vectorize wcs lookup
-            coords = [ image.radectopix(star['ra_deg'], star['dec_deg'])
-                       for star in starlist ]
-            arr = numpy.array(coords)
-            self.logger.debug("arr.shape = %s" % str(arr.shape))
+            coords = np.asarray([(star['ra_deg'], star['dec_deg'])
+                                    for star in starlist])
+
+            # vectorized wcs transform to data coords
+            coords = image.wcs.wcspt_to_datapt(coords)
 
             # vectorized test for inclusion in shape
-            res = filter_obj.contains_arr(arr.T[0], arr.T[1])
+            res = filter_obj.contains_pts(coords)
             self.logger.debug("res.shape = %s" % str(res.shape))
 
-            stars = [ starlist[i] for i in range(num_cat) if res[i] ]
+            stars = np.array(starlist)[np.flatnonzero(res)]
             self.logger.debug("number of filtered stars=%d" % (len(stars)))
-            starlist = stars
+            starlist = list(stars)
 
         return starlist
 
@@ -600,25 +670,35 @@ class Catalogs(GingaPlugin.LocalPlugin):
         self.canvas.delete_objects(objects)
         self.canvas.delete_objects_by_tag(['selected'])
 
-    def clearAll(self):
+    def clear_all(self):
         self.canvas.delete_all_objects()
+        self.set_mode('draw')
 
     def reset(self):
         self.clear()
-        #self.clearAll()
+        #self.clear_all()
         self.table.clear()
 
+    def select_cb(self, obj, canvas, event, pt):
+        if (obj.tag is not None) and obj.tag.startswith('star'):
+            info = obj.get_data()
+            self.table.show_selection(info.star)
+            return True
+        return True
+
     def plot_star(self, obj, image=None):
 
         if not image:
             image = self.fitsimage.get_image()
         x, y = image.radectopix(obj['ra_deg'], obj['dec_deg'])
         # TODO: auto-pick a decent radius
-        radius = 10
+        radius = self.settings.get('click_radius', 10)
         color = self.table.get_color(obj)
 
         circle = self.dc.Circle(x, y, radius, color=color)
+        circle.editable = False
         point = self.dc.Point(x, y, radius, color=color)
+        point.editable = False
 
         ## What is this from?
         if 'pick' in obj:
@@ -633,6 +713,10 @@ class Catalogs(GingaPlugin.LocalPlugin):
         else:
             star = self.dc.Canvas(circle, point)
 
+        star.pickable = True
+        star.add_callback('pick-up', self.select_cb)
+        star.opaque = True
+        # see select_cb() above
         star.set_data(star=obj)
         obj.canvobj = star
 
@@ -681,12 +765,12 @@ class Catalogs(GingaPlugin.LocalPlugin):
 
             canvas.update_canvas()
 
-    def limit_area_cb(self, tf):
-        self.limit_stars_to_area = (tf != 0)
+    def limit_area_cb(self, w, tf):
+        self.limit_stars_to_area = tf
         return True
 
-    def use_dss_channel_cb(self, tf):
-        self.use_dss_channel = (tf != 0)
+    def use_dss_channel_cb(self, w, tf):
+        self.use_dss_channel = tf
         return True
 
     def plot_pct_cb(self, widget, val):
@@ -699,7 +783,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
         num_stars = len(self.starlist)
         if num_stars > 0:
             adj = self.w.plotgrp
-            page_size = self.plot_limit
+            #page_size = self.plot_limit
             self.plot_start = min(self.plot_start, num_stars-1)
             adj.set_limits(0, num_stars, incr_value=1)
 
@@ -769,19 +853,16 @@ class Catalogs(GingaPlugin.LocalPlugin):
         for bnch in (self.image_server_params,
                      self.catalog_server_params):
             if bnch is not None:
-                for key in bnch.keys():
+                for key in list(bnch.keys()):
                     if key in d:
                         bnch[key].set_text(str(d[key]))
 
     def get_params(self, bnch):
         params = {}
-        for key in bnch.keys():
+        for key in list(bnch.keys()):
             params[key] = str(bnch[key].get_text())
         return params
 
-    def instructions(self):
-        self.set_message("""TBD.""")
-
     def set_drawtype_cb(self, tf, drawtype):
         if tf:
             self.drawtype = drawtype
@@ -791,6 +872,7 @@ class Catalogs(GingaPlugin.LocalPlugin):
     def __str__(self):
         return 'catalogs'
 
+
 class CatalogListing(object):
 
     def __init__(self, logger, container):
@@ -831,13 +913,13 @@ class CatalogListing(object):
 
         self.cmap = cmap.get_cmap(self.magcmap)
         self.imap = imap.get_imap('ramp')
+        self.cbar_ht = 32
 
         self.operation_table = []
         self._select_flag = False
 
         self._build_gui(container)
 
-
     def get_color(self, obj):
         try:
             mag = obj[self.mag_field]
@@ -891,13 +973,14 @@ class CatalogListing(object):
             return False
         else:
             if self.selection_mode == 'single':
-                # if selection mode is 'single' unselect any existing selections
+                # if selection mode is 'single' unselect any existing selection
                 for star2 in self.selected:
                     self.selected.remove(star2)
                     try:
                         self._unselect_tv(star2, fromtable=fromtable)
                     except Exception as e:
-                        self.logger.warning("Error unhilighting star: %s" % (str(e)))
+                        self.logger.warning(
+                            "Error unhilighting star: %s" % (str(e)))
             self.selected.append(star)
             try:
                 # highlight line in table
@@ -910,7 +993,6 @@ class CatalogListing(object):
             self.catalog.update_selected()
             return True
 
-
     def show_selection(self, star):
         """This method is called when the user clicks on a plotted star in the
         fitsviewer.
@@ -974,7 +1056,6 @@ class CatalogListing(object):
     def add_operation(self, name, fn):
         self.operation_table.append((name, fn))
 
-
     def _build_gui(self, container):
         self.mframe = container
 
@@ -988,12 +1069,15 @@ class CatalogListing(object):
         vbox.add_widget(table, stretch=1)
 
         self.cbar = ColorBar.ColorBar(self.logger)
+        # hack to set font size of this color bar
+        self.cbar.cbar.fontsize = 8
         self.cbar.set_cmap(self.cmap)
         self.cbar.set_imap(self.imap)
         rgbmap = self.cbar.get_rgbmap()
         rgbmap.add_callback('changed', lambda *args: self.replot_stars())
 
         cbar_w = self.cbar.get_widget()
+        cbar_w.resize(-1, self.cbar_ht)
         vbox.add_widget(cbar_w, stretch=0)
 
         btns = Widgets.HBox()
@@ -1012,7 +1096,8 @@ class CatalogListing(object):
         except Exception:
             index = self.cmap_names.index('gray')
         combobox.set_index(index)
-        combobox.add_callback('activated', lambda w, idx: self.set_cmap_cb(idx))
+        combobox.add_callback(
+            'activated', lambda w, idx: self.set_cmap_cb(idx))
         self.btn['cmap'] = combobox
         btns.add_widget(combobox, stretch=0)
 
@@ -1029,7 +1114,8 @@ class CatalogListing(object):
         except Exception:
             index = self.imap_names.index('ramp')
         combobox.set_index(index)
-        combobox.add_callback('activated', lambda w, idx: self.set_imap_cb(idx))
+        combobox.add_callback(
+            'activated', lambda w, idx: self.set_imap_cb(idx))
         self.btn['imap'] = combobox
         btns.add_widget(combobox, stretch=0)
 
@@ -1054,8 +1140,7 @@ class CatalogListing(object):
         btns = Widgets.HBox()
         btns.set_spacing(5)
 
-        for name in ('Plot', 'Clear', #'Close'
-                     ):
+        for name in ('Plot', 'Clear'):  # 'Close'
             btn = Widgets.Button(name)
             btns.add_widget(btn, stretch=0)
             self.btn[name.lower()] = btn
@@ -1065,7 +1150,7 @@ class CatalogListing(object):
         self.btn.clear.add_callback('activated',
                                     lambda w: self.clear())
         #self.btn.close.add_callback('activated',
-        #                              lambda w: self.close())
+        #                            lambda w: self.close())
 
         combobox = Widgets.ComboBox()
         options = []
@@ -1192,11 +1277,15 @@ class CatalogListing(object):
     def select_star_cb(self, widget, res_dict):
         """This method is called when the user selects a star from the table.
         """
-        key = list(res_dict.keys())[0]
-        idx = int(key)
-        star = self.starlist[idx]
-        if not self._select_flag:
-            self.mark_selection(star, fromtable=True)
+        keys = list(res_dict.keys())
+        if len(keys) == 0:
+            self.selected = []
+            self.replot_stars()
+        else:
+            idx = int(keys[0])
+            star = self.starlist[idx]
+            if not self._select_flag:
+                self.mark_selection(star, fromtable=True)
         return True
 
     def set_cmap_cb(self, index):
