@@ -19,7 +19,20 @@ fillkinds = ('circle', 'rectangle', 'polygon', 'triangle', 'righttriangle',
 
 
 class Drawing(GingaPlugin.LocalPlugin):
-    """Local plugin to draw shapes on canvas."""
+    """
+    Drawing
+    =======
+    A plugin for drawing canvas forms (overlaid graphics).
+
+    Plugin Type: Local
+    ------------------
+    Drawing is a local plugin, which means it is associated with a
+    channel.  An instance can be opened for each channel.
+
+    Usage
+    -----
+    TBD
+    """
 
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
@@ -43,7 +56,7 @@ class Drawing(GingaPlugin.LocalPlugin):
         self.drawtypes = list(canvas.get_drawtypes())
         self.drawcolors = draw_colors
         self.linestyles = ['solid', 'dash']
-        self.coordtypes = ['data', 'wcs', 'cartesian', 'canvas']
+        self.coordtypes = ['data', 'wcs', 'cartesian', 'window']
         # contains all parameters to be passed to the constructor
         self.draw_args = []
         self.draw_kwdargs = {}
@@ -65,19 +78,9 @@ class Drawing(GingaPlugin.LocalPlugin):
         vbox.set_border_width(4)
         vbox.set_spacing(2)
 
-        msg_font = self.fv.get_font("sansFont", 12)
-        tw = Widgets.TextArea(wrap=True, editable=False)
-        tw.set_font(msg_font)
-        self.tw = tw
-
-        fr = Widgets.Expander("Instructions")
-        fr.set_widget(tw)
-        vbox.add_widget(fr, stretch=0)
-
         fr = Widgets.Frame("Drawing")
 
         captions = (("Draw type:", 'label', "Draw type", 'combobox'),
-                    ("Coord type:", 'label', "Coord type", 'combobox'),
                     )
         w, b = Widgets.build_info(captions)
         self.w.update(b)
@@ -87,14 +90,7 @@ class Drawing(GingaPlugin.LocalPlugin):
             combobox.append_text(name)
         index = self.drawtypes.index(default_drawtype)
         combobox.set_index(index)
-        combobox.add_callback('activated', lambda w, idx: self.set_drawparams_cb())
-
-        combobox = b.coord_type
-        for name in self.coordtypes:
-            combobox.append_text(name)
-        index = 0
-        combobox.set_index(index)
-        combobox.add_callback('activated', lambda w, idx: self.set_drawparams_cb())
+        combobox.add_callback('activated', lambda w, idx: self.set_drawtype_cb())
 
         fr.set_widget(w)
         vbox.add_widget(fr, stretch=0)
@@ -164,6 +160,9 @@ class Drawing(GingaPlugin.LocalPlugin):
         btn = Widgets.Button("Close")
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn, stretch=0)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btns.add_widget(Widgets.Label(''), stretch=1)
         top.add_widget(btns, stretch=0)
 
@@ -174,14 +173,7 @@ class Drawing(GingaPlugin.LocalPlugin):
     def close(self):
         self.fv.stop_local_plugin(self.chname, str(self))
 
-    def instructions(self):
-        self.tw.set_text(
-            """Draw a figure with the cursor.
-
-For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
-
     def start(self):
-        self.instructions()
         self.set_drawparams_cb()
 
         # insert layer if it is not already
@@ -196,10 +188,10 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
         self.resume()
 
     def pause(self):
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
 
     def resume(self):
-        self.canvas.ui_setActive(True)
+        self.canvas.ui_set_active(True)
         self.fv.show_status("Draw a figure with the right mouse button")
 
     def stop(self):
@@ -211,7 +203,7 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
             pass
         # don't leave us stuck in edit mode
         self.canvas.set_draw_mode('draw')
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
         self.fv.show_status("")
 
     def redo(self):
@@ -224,14 +216,15 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
         self.logger.info("drew a %s" % (obj.kind))
         return True
 
+    def set_drawtype_cb(self):
+        if self.canvas.get_draw_mode() != 'draw':
+            self.canvas.set_draw_mode('draw')
+        self.w.btn_draw.set_state(True)
+        self.set_mode_cb('draw', True)
+
     def set_drawparams_cb(self):
-        ## if self.canvas.get_draw_mode() != 'draw':
-        ##     # if we are in edit mode then don't initialize draw gui
-        ##     return
         index = self.w.draw_type.get_index()
         kind = self.drawtypes[index]
-        index = self.w.coord_type.get_index()
-        coord = self.coordtypes[index]
 
         # remove old params
         self.w.drawvbox.remove_all()
@@ -259,9 +252,7 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
         self.w.rotate_by.set_enabled(False)
 
         args, kwdargs = self.draw_params.get_params()
-        #self.logger.debug("changing params to: %s" % str(kwdargs))
-        if kind != 'compass':
-            kwdargs['coord'] = coord
+        self.logger.debug("changing params to: %s" % (str(kwdargs)))
         self.canvas.set_drawtype(kind, **kwdargs)
 
     def draw_params_changed_cb(self, paramObj, params):
@@ -269,7 +260,7 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
         kind = self.drawtypes[index]
 
         args, kwdargs = self.draw_params.get_params()
-        #self.logger.debug("changing params to: %s" % str(kwdargs))
+        self.logger.debug("changing params to: %s" % (str(kwdargs)))
         self.canvas.set_drawtype(kind, **kwdargs)
 
     def edit_cb(self, fitsimage, obj):
@@ -287,8 +278,8 @@ For polygons/paths press 'v' to create a vertex, 'z' to remove last vertex.""")
         if hasattr(obj, 'coord'):
             tomap = self.fitsimage.get_coordmap(obj.coord)
             if obj.crdmap != tomap:
-                #self.logger.debug("coordmap has changed to '%s'--converting mapper" % (
-                #    str(tomap)))
+                self.logger.debug("coordmap has changed to '%s'--converting mapper" % (
+                    str(tomap)))
                 # user changed type of mapper; convert coordinates to
                 # new mapper and update widgets
                 obj.convert_mapper(tomap)
