@@ -12,7 +12,16 @@ from ginga.gw import Widgets
 
 
 class Header(GingaPlugin.GlobalPlugin):
+    """
+    Header
+    ======
+    The Header plugin provides a listing of the metadata associated with the
+    image.
 
+    Plugin Type: Global
+    -------------------
+    Header is a global plugin.  Only one instance can be opened.
+    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(Header, self).__init__(fv)
@@ -26,10 +35,10 @@ class Header(GingaPlugin.GlobalPlugin):
                         ]
 
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_Header')
-        self.settings.addDefaults(sortable=False,
-                                  color_alternate_rows=True,
-                                  max_rows_for_col_resize=5000)
+        self.settings = prefs.create_category('plugin_Header')
+        self.settings.add_defaults(sortable=False,
+                                   color_alternate_rows=True,
+                                   max_rows_for_col_resize=5000)
         self.settings.load(onError='silent')
 
         fv.add_callback('add-channel', self.add_channel)
@@ -145,13 +154,18 @@ class Header(GingaPlugin.GlobalPlugin):
             self.add_channel(self.fv, channel)
 
     def redo(self, channel, image):
-        """This is called when buffer is modified."""
+        """This is called when image changes."""
         self._image = None  # Skip cache checking in set_header()
-        chname = channel.name
         info = channel.extdata._header_info
 
         self.set_header(info, image)
 
+    def blank(self, channel):
+        """This is called when image is cleared."""
+        self._image = None
+        info = channel.extdata._header_info
+        info.table.clear()
+
     def set_sortable_cb(self, info):
         self._image = None
         channel = self.fv.get_channel(info.chname)
