@@ -6,14 +6,64 @@
 #
 import numpy
 
-from ginga.gw import Widgets, Plot
+from ginga.gw import Widgets
 from ginga import GingaPlugin
 from ginga import AutoCuts
-from ginga.util import plots
+
+try:
+    from ginga.gw import Plot
+    from ginga.util import plots
+    have_mpl = True
+except ImportError:
+    have_mpl = False
 
 
 class Histogram(GingaPlugin.LocalPlugin):
+    """
+    Histogram
+    =========
+    Histogram plots a histogram for a region drawn in the image, or for the
+    entire image.
 
+    Plugin Type: Local
+    ------------------
+    Histogram is a local plugin, which means it is associated with a channel.
+    An instance can be opened for each channel.
+
+    If a new image is selected for the channel the histogram plot will be
+    recalculated based on the current parameters with the new data.
+
+    Usage
+    -----
+    Click and drag to define a region within the image that will be used to
+    calculate the histogram.  To take the histogram of the full image, click
+    the button in the UI labeled "Full Image" (NOTE: depending on the size
+    of the image calculating the full histogram can take time).
+
+    UI Controls
+    -----------
+    Three radio buttons at the bottom of the UI are used to control the
+    effects of the click/drag action:
+
+    * select "Move" to drag the region to a different location
+    * select "Draw" to draw a new region
+    * select "Edit" to edit the region
+
+    To make a log plot of the histogram, check the "Log Histogram" checkbox.
+    To plot by the full range of values in the image instead of by the range
+    within the cut values, uncheck the "Plot By Cuts" checkbox.
+
+    The "NumBins" parameter determines how many bins are used in calculating
+    the histogram.  Type a number in the box and press Enter to change the
+    default value.
+
+    Cut Levels Convenience Controls
+    -------------------------------
+    Because a histogram is useful feedback for setting the cut levels,
+    controls are provided in the UI for setting the low and high cut levels
+    in the image, as well as for performing an auto cut levels, according to
+    the auto cut levels settings in the channel preferences.
+    """
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
         super(Histogram, self).__init__(fv, fitsimage)
@@ -25,9 +75,9 @@ class Histogram(GingaPlugin.LocalPlugin):
 
         # get Histogram preferences
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_Histogram')
-        self.settings.addDefaults(draw_then_move=True, num_bins=2048,
-                                  hist_color='aquamarine')
+        self.settings = prefs.create_category('plugin_Histogram')
+        self.settings.add_defaults(draw_then_move=True, num_bins=2048,
+                                   hist_color='aquamarine')
         self.settings.load(onError='silent')
 
         # Set up histogram control parameters
@@ -53,27 +103,23 @@ class Histogram(GingaPlugin.LocalPlugin):
 
         fitssettings = fitsimage.get_settings()
         for name in ['cuts']:
-            fitssettings.getSetting(name).add_callback(
+            fitssettings.get_setting(name).add_callback(
                 'set', self.cutset_ext_cb, fitsimage)
         self.gui_up = False
 
     def build_gui(self, container):
+        if not have_mpl:
+            raise ImportError('Install matplotlib to use this plugin')
+
         top = Widgets.VBox()
         top.set_border_width(4)
 
         # Make the cuts plot
-        vbox, sw, orientation = Widgets.get_oriented_box(container)
-        vbox.set_border_width(4)
-        vbox.set_spacing(2)
+        box, sw, orientation = Widgets.get_oriented_box(container)
+        box.set_border_width(4)
+        box.set_spacing(2)
 
-        msg_font = self.fv.get_font("sansFont", 12)
-        tw = Widgets.TextArea(wrap=True, editable=False)
-        tw.set_font(msg_font)
-        self.tw = tw
-
-        fr = Widgets.Expander("Instructions")
-        fr.set_widget(tw)
-        vbox.add_widget(fr, stretch=0)
+        paned = Widgets.Splitter(orientation=orientation)
 
         self.plot = plots.Plot(logger=self.logger,
                                width=400, height=400)
@@ -81,7 +127,7 @@ class Histogram(GingaPlugin.LocalPlugin):
         ax.grid(True)
         w = Plot.PlotWidget(self.plot)
         w.resize(400, 400)
-        vbox.add_widget(w, stretch=1)
+        paned.add_widget(Widgets.hadjust(w, orientation))
 
         captions = (('Cut Low:', 'label', 'Cut Low', 'entry'),
                     ('Cut High:', 'label', 'Cut High', 'entry',
@@ -119,7 +165,12 @@ class Histogram(GingaPlugin.LocalPlugin):
         b.numbins.add_callback('activated', lambda w: self.set_numbins_cb())
         b.full_image.add_callback('activated', lambda w: self.full_image_cb())
 
-        vbox.add_widget(w, stretch=0)
+        fr = Widgets.Frame("Histogram")
+        fr.set_widget(w)
+        box.add_widget(fr, stretch=0)
+        paned.add_widget(sw)
+        # hack to set a reasonable starting position for the splitter
+        paned.set_sizes([400, 500])
 
         mode = self.canvas.get_draw_mode()
         hbox = Widgets.HBox()
@@ -152,12 +203,9 @@ class Histogram(GingaPlugin.LocalPlugin):
             self.w.btn_edit.set_enabled(False)
 
         hbox.add_widget(Widgets.Label(''), stretch=1)
-        vbox.add_widget(hbox, stretch=0)
 
-        ## spacer = Widgets.Label('')
-        ## vbox.add_widget(spacer, stretch=1)
-
-        top.add_widget(sw, stretch=1)
+        top.add_widget(paned, stretch=5)
+        top.add_widget(hbox, stretch=0)
 
         btns = Widgets.HBox()
         btns.set_border_width(4)
@@ -166,6 +214,9 @@ class Histogram(GingaPlugin.LocalPlugin):
         btn = Widgets.Button("Close")
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn, stretch=0)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btns.add_widget(Widgets.Label(''), stretch=1)
 
         top.add_widget(btns, stretch=0)
@@ -173,15 +224,11 @@ class Histogram(GingaPlugin.LocalPlugin):
         container.add_widget(top, stretch=1)
         self.gui_up = True
 
-    def instructions(self):
-        self.tw.set_text("""Draw (or redraw) a region with the right mouse button.  Click or drag left mouse button to reposition region.""")
-
     def close(self):
         self.fv.stop_local_plugin(self.chname, str(self))
         return True
 
     def start(self):
-        self.instructions()
         self.plot.set_titles(rtitle="Histogram")
 
         # insert canvas, if not already
@@ -197,13 +244,13 @@ class Histogram(GingaPlugin.LocalPlugin):
         self.resume()
 
     def pause(self):
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
 
     def resume(self):
         # turn off any mode user may be in
         self.modes_off()
 
-        self.canvas.ui_setActive(True)
+        self.canvas.ui_set_active(True)
         self.fv.show_status("Draw a rectangle with the right mouse button")
 
     def stop(self):
