@@ -12,7 +12,37 @@ from ginga import GingaPlugin
 
 
 class Info(GingaPlugin.GlobalPlugin):
+    """
+    Info
+    ====
+    The Info plugin provides a pane of commonly useful metadata about the
+    associated channel image.  Common information includes some
+    FITS header values, the equinox, dimensions of the image, minimum and
+    maximum values and the zoom level.  As the cursor is moved around the
+    image, the X, Y, Value, RA and DEC values are updated to reflect the
+    value under the cursor.
 
+    Plugin Type: Global
+    -------------------
+    Info is a global plugin.  Only one instance can be opened.
+
+    Usage
+    -----
+    At the bottom of the Info interface are the cut levels controls. Here
+    the low and high cut levels are shown and can be adjusted.  Pressing the
+    "Auto Levels" button will recalculate cut levels based on the current
+    auto cut levels algorithm and parameters defined in the channel
+    preferences.
+
+    Below the "Auto Levels" button, the status of the settings for
+    "Cut New", "Zoom New" and "Center New" are shown for the currently active
+    channel.  These indicate how new images that are added to the channel
+    will be affected by auto cut levels, fitting to the window and panning
+    to the center of the image.
+
+    The Info plugin typically appears under the "Synopsis" tab in the user
+    interface.
+    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(Info, self).__init__(fv)
@@ -22,7 +52,10 @@ class Info(GingaPlugin.GlobalPlugin):
         # truncate names after this size
         self.maxstr = 60
 
-        #self.w = Bunch.Bunch()
+        self.autozoom_options = ['on', 'override', 'once', 'off']
+        self.autocut_options = self.autozoom_options
+        self.autocenter_options = self.autozoom_options
+
         fv.add_callback('add-channel', self.add_channel)
         fv.add_callback('delete-channel', self.delete_channel)
         fv.add_callback('field-info', self.field_info)
@@ -60,15 +93,10 @@ class Info(GingaPlugin.GlobalPlugin):
         row.add_widget(w, stretch=0)
         row.add_widget(Widgets.Label(''), stretch=1)
         col.add_widget(row, stretch=0)
-        #col.add_widget(Widgets.Label(''), stretch=1)
+        col.add_widget(Widgets.Label(''), stretch=1)
         sw2 = Widgets.ScrollArea()
-        # hack for Qt to expand this widget properly
-        sw2.cfg_expand(0x7, 0x4)
         sw2.set_widget(col)
-        vbox.add_widget(sw2, stretch=2)
-
-        # stretcher
-        vbox.add_widget(Widgets.Label(''), stretch=1)
+        vbox.add_widget(sw2, stretch=1)
 
         captions = (('Zoom:', 'label', 'Zoom', 'llabel'),
                     ('Cut Low:', 'label', 'Cut Low Value', 'llabel',
@@ -77,31 +105,46 @@ class Info(GingaPlugin.GlobalPlugin):
                      'Cut High', 'entry'),
                     ('Auto Levels', 'button', 'spacer1', 'spacer',
                      'Cut Levels', 'button'),
-                    ('Cut New:', 'label', 'Cut New', 'llabel'),
-                    ('Zoom New:', 'label', 'Zoom New', 'llabel'),
-                    ('Center New:', 'label', 'Center New', 'llabel'),
+                    ('Cut New:', 'label', 'Cut New', 'combobox'),
+                    ('Zoom New:', 'label', 'Zoom New', 'combobox'),
+                    ('Center New:', 'label', 'Center New', 'combobox'),
                     )
 
         w, b2 = Widgets.build_info(captions)
         b.update(b2)
-        # TODO: need a more general solution to gtk labels resizing their
-        # parent window
-        #b.object.set_length(12)
         b.cut_levels.set_tooltip("Set cut levels manually")
         b.auto_levels.set_tooltip("Set cut levels by algorithm")
         b.cut_low.set_tooltip("Set low cut level (press Enter)")
         b.cut_low_value.set_text('')
         b.cut_high.set_tooltip("Set high cut level (press Enter)")
         b.cut_high_value.set_text('')
-        b.cut_new.set_text('')
-        b.zoom_new.set_text('')
-        b.center_new.set_text('')
+
+        combobox = b.cut_new
+        index = 0
+        for name in self.autocut_options:
+            combobox.append_text(name)
+            index += 1
+        b.cut_new.set_tooltip("Automatically set cut levels for new images")
+
+        combobox = b.zoom_new
+        index = 0
+        for name in self.autozoom_options:
+            combobox.append_text(name)
+            index += 1
+        b.zoom_new.set_tooltip("Automatically fit new images to window")
+
+        combobox = b.center_new
+        index = 0
+        for name in self.autocenter_options:
+            combobox.append_text(name)
+            index += 1
+        b.center_new.set_tooltip("Automatically center new images in window")
 
         row = Widgets.HBox()
         row.set_spacing(0)
         row.set_border_width(0)
         row.add_widget(w, stretch=0)
-        ## row.add_widget(Widgets.Label(''), stretch=1)
+        row.add_widget(Widgets.Label(''), stretch=1)
         vbox.add_widget(row, stretch=0)
 
         return sw, b
@@ -125,20 +168,26 @@ class Info(GingaPlugin.GlobalPlugin):
                                       channel.fitsimage, info)
         winfo.auto_levels.add_callback('activated', self.auto_levels,
                                        channel.fitsimage, info)
+        winfo.cut_new.add_callback('activated', self.set_autocuts_cb,
+                                   channel.fitsimage, info)
+        winfo.zoom_new.add_callback('activated', self.set_autozoom_cb,
+                                    channel.fitsimage, info)
+        winfo.center_new.add_callback('activated', self.set_autocenter_cb,
+                                      channel.fitsimage, info)
 
         fitsimage = channel.fitsimage
         fitssettings = fitsimage.get_settings()
         for name in ['cuts']:
-            fitssettings.getSetting(name).add_callback('set',
+            fitssettings.get_setting(name).add_callback('set',
                                self.cutset_cb, fitsimage, info)
         for name in ['scale']:
-            fitssettings.getSetting(name).add_callback('set',
+            fitssettings.get_setting(name).add_callback('set',
                                self.zoomset_cb, fitsimage, info)
-        fitssettings.getSetting('autocuts').add_callback('set',
+        fitssettings.get_setting('autocuts').add_callback('set',
                                self.autocuts_cb, fitsimage, info)
-        fitssettings.getSetting('autozoom').add_callback('set',
+        fitssettings.get_setting('autozoom').add_callback('set',
                                self.autozoom_cb, fitsimage, info)
-        fitssettings.getSetting('autocenter').add_callback('set',
+        fitssettings.get_setting('autocenter').add_callback('set',
                                self.autocenter_cb, fitsimage, info)
 
     def delete_channel(self, viewer, channel):
@@ -156,18 +205,9 @@ class Info(GingaPlugin.GlobalPlugin):
         fitsimage = channel.fitsimage
         info = channel.extdata._info_info
 
-        # add cb to image so that if it is modified we can update info
-        image.add_callback('modified', self.image_update_cb, fitsimage, info)
-
         self.set_info(info, fitsimage)
         return True
 
-    def image_update_cb(self, image, fitsimage, info):
-        cur_img = fitsimage.get_image()
-        if cur_img == image:
-            self.fv.gui_do(self.set_info, info, fitsimage)
-        return True
-
     def focus_cb(self, viewer, channel):
         chname = channel.name
 
@@ -206,17 +246,33 @@ class Info(GingaPlugin.GlobalPlugin):
         info.winfo.cut_high_value.set_text('%.4g' % (hival))
 
     def autocuts_cb(self, setting, option, fitsimage, info):
-        info.winfo.cut_new.set_text(option)
+        self.logger.debug("autocuts changed to %s" % option)
+        index = self.autocut_options.index(option)
+        info.winfo.cut_new.set_index(index)
 
     def autozoom_cb(self, setting, option, fitsimage, info):
-        info.winfo.zoom_new.set_text(option)
+        index = self.autozoom_options.index(option)
+        info.winfo.zoom_new.set_index(index)
 
     def autocenter_cb(self, setting, option, fitsimage, info):
         # Hack to convert old values that used to be T/F
         if isinstance(option, bool):
             choice = { True: 'on', False: 'off' }
             option = choice[option]
-        info.winfo.center_new.set_text(option)
+        index = self.autocenter_options.index(option)
+        info.winfo.center_new.set_index(index)
+
+    def set_autocuts_cb(self, w, index, fitsimage, info):
+        option = self.autocut_options[index]
+        fitsimage.enable_autocuts(option)
+
+    def set_autozoom_cb(self, w, index, fitsimage, info):
+        option = self.autozoom_options[index]
+        fitsimage.enable_autozoom(option)
+
+    def set_autocenter_cb(self, w, index, fitsimage, info):
+        option = self.autocenter_options[index]
+        fitsimage.enable_autocenter(option)
 
     # LOGIC
 
@@ -264,14 +320,17 @@ class Info(GingaPlugin.GlobalPlugin):
 
         # update cut new/zoom new indicators
         t_ = fitsimage.get_settings()
-        info.winfo.cut_new.set_text(t_['autocuts'])
-        info.winfo.zoom_new.set_text(t_['autozoom'])
+        index = self.autocut_options.index(t_['autocuts'])
+        info.winfo.cut_new.set_index(index)
+        index = self.autozoom_options.index(t_['autozoom'])
+        info.winfo.zoom_new.set_index(index)
         option = t_['autocenter']
         # Hack to convert old values that used to be T/F
         if isinstance(option, bool):
             choice = { True: 'on', False: 'off' }
             option = choice[option]
-        info.winfo.center_new.set_text(option)
+        index = self.autocenter_options.index(option)
+        info.winfo.center_new.set_index(index)
 
 
     def field_info(self, viewer, channel, info):
