@@ -10,6 +10,34 @@ from ginga import GingaPlugin, RGBImage, colors
 from ginga.gw import Widgets
 
 class Overlays(GingaPlugin.LocalPlugin):
+    """
+    Overlays
+    ========
+    A plugin for generating color overlays representing under- and
+    overexposure in the loaded image.
+
+    Plugin Type: Local
+    ------------------
+    Overlays is a local plugin, which means it is associated with a channel.
+    An instance can be opened for each channel.
+
+    Usage
+    -----
+    Choose colors from the drop-down menus for the low limit and/or
+    high-limit ("Lo color" and "Hi color").  Specify the limits for low and
+    high values in the limit boxes ("Lo limit" and "Hi limit").  Set the
+    opacity of the overlays with a value between 0 and 1 in the "Opacity"
+    box.  Finally, press the "Redo" button.
+
+    The color overlay should show areas below the low limit with a low color
+    and the areas above the high limit in the high color.
+
+    If you omit a limit (leave the box blank), that color won't be shown in
+    the overlay.
+
+    If a new image is selected for the channel the overlays image will be
+    recalculated based on the current parameters with the new data.
+    """
 
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
@@ -44,15 +72,6 @@ class Overlays(GingaPlugin.LocalPlugin):
         vbox.set_border_width(4)
         vbox.set_spacing(2)
 
-        self.msg_font = self.fv.get_font("sansFont", 12)
-        tw = Widgets.TextArea(wrap=True, editable=False)
-        tw.set_font(self.msg_font)
-        self.tw = tw
-
-        fr = Widgets.Expander("Instructions")
-        fr.set_widget(tw)
-        vbox.add_widget(fr, stretch=0)
-
         fr = Widgets.Frame("Limits")
 
         captions = (('Opacity:', 'label', 'Opacity', 'spinfloat'),
@@ -109,6 +128,9 @@ class Overlays(GingaPlugin.LocalPlugin):
         btn = Widgets.Button("Close")
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn, stretch=0)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btns.add_widget(Widgets.Label(''), stretch=1)
         top.add_widget(btns, stretch=0)
 
@@ -118,11 +140,7 @@ class Overlays(GingaPlugin.LocalPlugin):
         self.fv.stop_local_plugin(self.chname, str(self))
         return True
 
-    def instructions(self):
-        self.tw.set_text("""Enter a limit for saturation.""")
-
     def start(self):
-        self.instructions()
         # start ruler drawing operation
         p_canvas = self.fitsimage.get_canvas()
         try:
@@ -137,10 +155,10 @@ class Overlays(GingaPlugin.LocalPlugin):
             self.redo()
 
     def pause(self):
-        self.canvas.ui_setActive(False)
+        self.canvas.ui_set_active(False)
 
     def resume(self):
-        self.canvas.ui_setActive(True)
+        self.canvas.ui_set_active(True)
         self.fv.show_status("Enter a value for saturation limit")
 
     def stop(self):
@@ -153,7 +171,7 @@ class Overlays(GingaPlugin.LocalPlugin):
             p_canvas.delete_object_by_tag(self.layertag)
         except:
             pass
-        #self.canvas.ui_setActive(False)
+        #self.canvas.ui_set_active(False)
         self.fv.show_status("")
 
     def redo(self):
