@@ -13,7 +13,36 @@ from ginga.misc import Bunch
 from ginga import GingaPlugin
 
 class Pan(GingaPlugin.GlobalPlugin):
+    """
+    Pan
+    ===
+    The Pan plugin provides a small panning image that gives an overall
+    "birds-eye" view of the channel image that last had the focus.  If the
+    channel image is zoomed in 2X or greater then the pan region is shown
+    graphically in the Pan image by a rectangle.
 
+    Plugin Type: Global
+    -------------------
+    Pan is a global plugin.  Only one instance can be opened.
+
+    Usage
+    -----
+    The channel image can be panned by clicking and/or dragging to place
+    the rectangle.  Using the right mouse button to drag a rectangle will
+    force the channel image viewer to try to match the region (taking into
+    account the differences in the aspect ratio between the drawn rectangle
+    and the window dimensions).  Scrolling in the Pan image will zoom the
+    channel image.
+
+    The color/intensity map and cut levels of the Pan image are updated
+    when they are changed in the corresponding channel image.
+    The Pan image also displays the World Coordinate System compass, if
+    valid WCS metadata is present in the FITS HDU being viewed in the
+    channel.
+
+    The Pan plugin usually appears as a sub-pane under the "Info" tab, next
+    to the Info plugin.
+    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(Pan, self).__init__(fv)
@@ -28,12 +57,12 @@ class Pan(GingaPlugin.GlobalPlugin):
         self.dc = fv.get_draw_classes()
 
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_Pan')
-        self.settings.addDefaults(use_shared_canvas=False,
-                                  pan_position_color='yellow',
-                                  pan_rectangle_color='red',
-                                  compass_color='skyblue',
-                                  rotate_pan_image=True)
+        self.settings = prefs.create_category('plugin_Pan')
+        self.settings.add_defaults(use_shared_canvas=False,
+                                   pan_position_color='yellow',
+                                   pan_rectangle_color='red',
+                                   compass_color='skyblue',
+                                   rotate_pan_image=True)
         self.settings.load(onError='silent')
         # share canvas with channel viewer?
         self.use_shared_canvas = self.settings.get('use_shared_canvas', False)
@@ -58,16 +87,17 @@ class Pan(GingaPlugin.GlobalPlugin):
         pi.set_callback('cursor-move', self.drag_cb)
         pi.set_callback('none-move', self.motion_cb)
         pi.set_callback('zoom-scroll', self.zoom_cb)
+        pi.set_callback('zoom-pinch', self.zoom_pinch_cb)
         pi.set_callback('configure', self.reconfigure)
         # for debugging
         pi.set_name('panimage')
-        #pi.ui_setActive(True)
+        #pi.ui_set_active(True)
 
         my_canvas = pi.get_canvas()
         my_canvas.enable_draw(True)
         my_canvas.set_drawtype('rectangle', linestyle='dash', color='green')
         my_canvas.set_callback('draw-event', self.draw_cb)
-        my_canvas.ui_setActive(True)
+        my_canvas.ui_set_active(True)
 
         if self.use_shared_canvas:
             canvas = fitsimage.get_canvas()
@@ -108,19 +138,19 @@ class Pan(GingaPlugin.GlobalPlugin):
             xfrmsettings.append('rot_deg')
         fitssettings.shareSettings(pansettings, xfrmsettings)
         for key in xfrmsettings:
-            pansettings.getSetting(key).add_callback('set', self.settings_cb,
+            pansettings.get_setting(key).add_callback('set', self.settings_cb,
                                                      fitsimage, channel, paninfo, 0)
 
 
         fitssettings.shareSettings(pansettings, ['cuts'])
-        pansettings.getSetting('cuts').add_callback('set', self.settings_cb,
+        pansettings.get_setting('cuts').add_callback('set', self.settings_cb,
                                                     fitsimage, channel, paninfo, 1)
 
         zoomsettings = ['zoom_algorithm', 'zoom_rate',
                         'scale_x_base', 'scale_y_base']
         fitssettings.shareSettings(pansettings, zoomsettings)
         for key in zoomsettings:
-            pansettings.getSetting(key).add_callback('set', self.zoom_ext_cb,
+            pansettings.get_setting(key).add_callback('set', self.zoom_ext_cb,
                                                      fitsimage, channel, paninfo)
 
         fitsimage.add_callback('redraw', self.redraw_cb, channel, paninfo)
@@ -151,24 +181,17 @@ class Pan(GingaPlugin.GlobalPlugin):
         paninfo = channel.extdata._pan_info
 
         if (image is None) or not isinstance(image, BaseImage):
-            # TODO: clear not yet working
-            #paninfo.panimage.clear()
+            paninfo.panimage.clear()
             return
 
         loval, hival = channel.fitsimage.get_cut_levels()
         paninfo.panimage.cut_levels(loval, hival)
 
-        # add cb to image so that if it is modified we can update info
-        ## image.add_callback('modified', self.image_update_cb, fitsimage,
-        ##                    channel, paninfo)
         self.set_image(channel, paninfo, image)
 
-    ## def image_update_cb(self, image, fitsimage, channel, paninfo):
-    ##     # image has changed (e.g. size, value range, etc)
-    ##     cur_img = fitsimage.get_image()
-    ##     if cur_img == image:
-    ##         self.fv.gui_do(self.set_image, channel, paninfo, image)
-    ##     return False
+    def blank(self, channel):
+        paninfo = channel.extdata._pan_info
+        paninfo.panimage.clear()
 
     def focus_cb(self, viewer, channel):
         chname = channel.name
@@ -217,8 +240,7 @@ class Pan(GingaPlugin.GlobalPlugin):
 
     def set_image(self, channel, paninfo, image):
         if (image is None) or not isinstance(image, BaseImage):
-            # TODO: clear not yet working
-            #paninfo.panimage.clear()
+            paninfo.panimage.clear()
             return
 
         if not self.use_shared_canvas:
@@ -265,8 +287,7 @@ class Pan(GingaPlugin.GlobalPlugin):
     def panset(self, fitsimage, channel, paninfo):
         image = fitsimage.get_image()
         if (image is None) or not isinstance(image, BaseImage):
-            # TODO: clear not yet working
-            #paninfo.panimage.clear()
+            paninfo.panimage.clear()
             return
 
         x, y = fitsimage.get_pan()
@@ -327,8 +348,7 @@ class Pan(GingaPlugin.GlobalPlugin):
         return True
 
     def zoom_cb(self, fitsimage, event):
-        """Zoom event in the small fits window.  Just zoom the large fits
-        window.
+        """Zoom event in the pan window.  Just zoom the channel viewer.
         """
         chviewer = self.fv.getfocus_viewer()
         bd = chviewer.get_bindings()
@@ -338,6 +358,17 @@ class Pan(GingaPlugin.GlobalPlugin):
 
         return False
 
+    def zoom_pinch_cb(self, fitsimage, event):
+        """Pinch event in the pan window.  Just zoom the channel viewer.
+        """
+        chviewer = self.fv.getfocus_viewer()
+        bd = chviewer.get_bindings()
+
+        if hasattr(bd, 'pi_zoom'):
+            return bd.pi_zoom(chviewer, event)
+
+        return False
+
     def draw_cb(self, canvas, tag):
         # Get and delete the drawn object
         obj = canvas.get_object_by_tag(tag)
