@@ -11,12 +11,29 @@ from ginga.misc import ParamSet, Bunch
 from ginga import cmap, imap, trcalc
 from ginga import GingaPlugin
 from ginga import AutoCuts, ColorDist
-from ginga.util import wcs, wcsmod, io_rgb
+from ginga.util import wcs, wcsmod, rgb_cms
 
 from ginga.misc import Bunch
 
 class Preferences(GingaPlugin.LocalPlugin):
+    """
+    Preferences
+    ===========
+    A plugin to manage the settings for a channel.
 
+    Plugin Type: Local
+    ------------------
+    Preferences is a local plugin, which means it is associated with a
+    channel.  An instance can be opened for each channel.
+
+    Usage
+    -----
+    Make changes to settings graphically in the UI.
+
+    If "Save Settings" is pressed it will save the settings to the user's
+    home Ginga folder so that when a channel with the same name is created
+    in future Ginga sessions it will obtain the same settings.
+    """
     def __init__(self, fv, fitsimage):
         # superclass defines some variables for us, like logger
         super(Preferences, self).__init__(fv, fitsimage)
@@ -37,47 +54,47 @@ class Preferences(GingaPlugin.LocalPlugin):
         self.sort_options = ('loadtime', 'alpha')
 
         self.t_ = self.fitsimage.get_settings()
-        self.t_.getSetting('autocuts').add_callback('set',
+        self.t_.get_setting('autocuts').add_callback('set',
                                                self.autocuts_changed_ext_cb)
-        self.t_.getSetting('autozoom').add_callback('set',
+        self.t_.get_setting('autozoom').add_callback('set',
                                                self.autozoom_changed_ext_cb)
-        self.t_.getSetting('autocenter').add_callback('set',
+        self.t_.get_setting('autocenter').add_callback('set',
                                                       self.autocenter_changed_ext_cb)
         for key in ['pan']:
-            self.t_.getSetting(key).add_callback('set',
+            self.t_.get_setting(key).add_callback('set',
                                           self.pan_changed_ext_cb)
         for key in ['scale']:
-            self.t_.getSetting(key).add_callback('set',
+            self.t_.get_setting(key).add_callback('set',
                                           self.scale_changed_ext_cb)
 
-        self.t_.getSetting('zoom_algorithm').add_callback('set', self.set_zoomalg_ext_cb)
-        self.t_.getSetting('zoom_rate').add_callback('set', self.set_zoomrate_ext_cb)
+        self.t_.get_setting('zoom_algorithm').add_callback('set', self.set_zoomalg_ext_cb)
+        self.t_.get_setting('zoom_rate').add_callback('set', self.set_zoomrate_ext_cb)
         for key in ['scale_x_base', 'scale_y_base']:
-            self.t_.getSetting(key).add_callback('set', self.scalebase_changed_ext_cb)
-        self.t_.getSetting('rot_deg').add_callback('set', self.set_rotate_ext_cb)
+            self.t_.get_setting(key).add_callback('set', self.scalebase_changed_ext_cb)
+        self.t_.get_setting('rot_deg').add_callback('set', self.set_rotate_ext_cb)
         for name in ('flip_x', 'flip_y', 'swap_xy'):
-            self.t_.getSetting(name).add_callback('set', self.set_transform_ext_cb)
+            self.t_.get_setting(name).add_callback('set', self.set_transform_ext_cb)
 
         ## for name in ('autocut_method', 'autocut_params'):
-        ##     self.t_.getSetting(name).add_callback('set', self.set_autocuts_ext_cb)
+        ##     self.t_.get_setting(name).add_callback('set', self.set_autocuts_ext_cb)
 
         ## for key in ['color_algorithm', 'color_hashsize', 'color_map',
         ##             'intensity_map']:
-        ##     self.t_.getSetting(key).add_callback('set', self.cmap_changed_ext_cb)
+        ##     self.t_.get_setting(key).add_callback('set', self.cmap_changed_ext_cb)
 
         self.t_.setdefault('wcs_coords', 'icrs')
         self.t_.setdefault('wcs_display', 'sexagesimal')
 
         # buffer len (number of images in memory)
-        self.t_.addDefaults(numImages=4)
-        self.t_.getSetting('numImages').add_callback('set', self.set_buflen_ext_cb)
+        self.t_.add_defaults(numImages=4)
+        self.t_.get_setting('numImages').add_callback('set', self.set_buflen_ext_cb)
 
         # preload images
-        self.t_.addDefaults(preload_images=False)
+        self.t_.add_defaults(preload_images=False)
 
-        self.icc_profiles = list(io_rgb.get_profiles())
+        self.icc_profiles = list(rgb_cms.get_profiles())
         self.icc_profiles.insert(0, None)
-        self.icc_intents = io_rgb.get_intents()
+        self.icc_intents = rgb_cms.get_intents()
 
     def build_gui(self, container):
         top = Widgets.VBox()
@@ -377,21 +394,25 @@ class Preferences(GingaPlugin.LocalPlugin):
 
         pan_x, pan_y = self.fitsimage.get_pan()
         coord_offset = self.fv.settings.get('pixel_coords_offset', 0.0)
+        pan_coord = self.t_.get('pan_coord', "data")
+        if pan_coord == 'data':
+            pan_x, pan_y = pan_x + coord_offset, pan_y + coord_offset
         b.pan_x.set_tooltip("Coordinate for the pan position in X axis")
-        b.pan_x.set_text(str(pan_x + coord_offset))
+        b.pan_x.set_text(str(pan_x))
         #b.pan_x.add_callback('activated', self.set_pan_cb)
         b.pan_y.set_tooltip("Coordinate for the pan position in Y axis")
-        b.pan_y.set_text(str(pan_y + coord_offset))
+        b.pan_y.set_text(str(pan_y))
         #b.pan_y.add_callback('activated', self.set_pan_cb)
         b.apply_pan.add_callback('activated', self.set_pan_cb)
         b.apply_pan.set_tooltip("Set the pan position")
         b.wcs_sexagesimal.set_tooltip("Display pan position in sexagesimal")
+        b.wcs_sexagesimal.add_callback('activated',
+                                       lambda w, tf: self._update_pan_coords())
 
         index = 0
         for name in self.pancoord_options:
             b.pan_coord.append_text(name)
             index += 1
-        pan_coord = self.t_.get('pan_coord', "data")
         index = self.pancoord_options.index(pan_coord)
         b.pan_coord.set_index(index)
         b.pan_coord.set_tooltip("Pan coordinates type")
@@ -536,9 +557,12 @@ class Preferences(GingaPlugin.LocalPlugin):
 
         exp = Widgets.Expander("ICC Profiles")
 
-        captions = (('Output ICC profile:', 'label', 'Output ICC profile', 'combobox'),
-                    ('Rendering intent:', 'label', 'Rendering intent', 'combobox'),
-                    ('Proof ICC profile:', 'label', 'Proof ICC profile', 'combobox'),
+        captions = (('Output ICC profile:', 'label', 'Output ICC profile',
+                     'combobox'),
+                    ('Rendering intent:', 'label', 'Rendering intent',
+                     'combobox'),
+                    ('Proof ICC profile:', 'label', 'Proof ICC profile',
+                     'combobox'),
                     ('Proof intent:', 'label', 'Proof intent', 'combobox'),
                     ('__x', 'spacer', 'Black point compensation', 'checkbutton'),
                     )
@@ -608,6 +632,9 @@ class Preferences(GingaPlugin.LocalPlugin):
         btn = Widgets.Button("Close")
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btn = Widgets.Button("Save Settings")
         btn.add_callback('activated', lambda w: self.save_preferences())
         btns.add_widget(btn)
@@ -881,8 +908,8 @@ class Preferences(GingaPlugin.LocalPlugin):
     def set_transform_ext_cb(self, setting, value):
         if not self.gui_up:
             return
-        flip_x, flip_y, swap_xy = \
-                self.t_['flip_x'], self.t_['flip_y'], self.t_['swap_xy']
+        flip_x, flip_y, swap_xy = (
+            self.t_['flip_x'], self.t_['flip_y'], self.t_['swap_xy'])
         self.w.flip_x.set_state(flip_x)
         self.w.flip_y.set_state(flip_y)
         self.w.swap_xy.set_state(swap_xy)
