@@ -14,7 +14,9 @@ import os
 import shutil
 
 # THIRD-PARTY
+import astropy
 from astropy.io import fits
+from astropy.utils.introspection import minversion
 
 # GINGA
 from ginga.GingaPlugin import GlobalPlugin
@@ -31,7 +33,37 @@ __all__ = []
 
 
 class SaveImage(GlobalPlugin):
-    """Save images to output files.
+    """
+    SaveImage
+    =========
+    Save images to output files.
+
+    Plugin Type: Global
+    -------------------
+    SaveImage is a global plugin.  Only one instance can be opened.
+
+    This global plugin is used to save any changes made in Ginga back to output
+    images. For example, a mosaic image that was created by the `Mosaic`
+    plugin. Currently, only FITS images (single or multiple extensions) are
+    supported.
+
+    Usage
+    -----
+    Given the output directory (e.g., ``/mypath/outputs/``), a suffix
+    (e.g., ``ginga``), an image channel (``Image``), and a selected image
+    (e.g., ``image1.fits``), the output file will be
+    ``/mypath/outputs/image1_ginga_Image.fits``. Inclusion of the channel name is
+    optional and can be omitted using plugin configuration file (see below).
+    The modified extension(s) will have new header or data extracted from
+    Ginga, while those not modified will remain untouched. Relevant change
+    log entries from the `ChangeHistory` global plugin will be inserted into
+    the history of its ``PRIMARY`` header.
+
+    .. note::
+
+      This plugin uses the module `astropy.io.fits` to write the output images,
+      regardless of what is chosen for ``FITSpkg`` in your
+      ``~/.ginga/general.cfg`` configuration file.
 
     """
     def __init__(self, fv):
@@ -44,14 +76,14 @@ class SaveImage(GlobalPlugin):
         # User preferences. Some are just default values and can also be
         # changed by GUI.
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_SaveImage')
-        self.settings.addDefaults(output_directory = '.',
-                                  output_suffix = 'ginga',
-                                  include_chname = True,
-                                  clobber = False,
-                                  modified_only = True,
-                                  max_mosaic_size = 1e8,
-                                  max_rows_for_col_resize = 5000)
+        self.settings = prefs.create_category('plugin_SaveImage')
+        self.settings.add_defaults(output_directory = '.',
+                                   output_suffix = 'ginga',
+                                   include_chname = True,
+                                   clobber = False,
+                                   modified_only = True,
+                                   max_mosaic_size = 1e8,
+                                   max_rows_for_col_resize = 5000)
         self.settings.load(onError='silent')
 
         self.outdir = os.path.abspath(
@@ -75,15 +107,6 @@ class SaveImage(GlobalPlugin):
 
         vbox, sw, orientation = Widgets.get_oriented_box(container)
 
-        msg_font = self.fv.get_font('sansFont', 12)
-        tw = Widgets.TextArea(wrap=True, editable=False)
-        tw.set_font(msg_font)
-        self.tw = tw
-
-        fr = Widgets.Expander('Instructions')
-        fr.set_widget(tw)
-        container.add_widget(fr, stretch=0)
-
         captions = (('Channel:', 'label', 'Channel Name', 'combobox',
                      'Modified only', 'checkbutton'), )
         w, b = Widgets.build_info(captions, orientation=orientation)
@@ -146,6 +169,9 @@ class SaveImage(GlobalPlugin):
         btn = Widgets.Button('Close')
         btn.add_callback('activated', lambda w: self.close())
         btns.add_widget(btn, stretch=0)
+        btn = Widgets.Button("Help")
+        btn.add_callback('activated', lambda w: self.help())
+        btns.add_widget(btn, stretch=0)
         btns.add_widget(Widgets.Label(''), stretch=1)
         container.add_widget(btns, stretch=0)
 
@@ -400,7 +426,10 @@ Output image will have the filename of <inputname>_<suffix>.fits.""")
         self._write_history(key, hdu)
 
         # Write to file
-        hdu.writeto(outfile, clobber=True)
+        if minversion(astropy, '1.3'):
+            hdu.writeto(outfile, overwrite=True)
+        else:
+            hdu.writeto(outfile, clobber=True)
 
     def _write_mef(self, key, extlist, outfile):
         """Write out regular multi-extension FITS data."""
@@ -480,7 +509,6 @@ Output image will have the filename of <inputname>_<suffix>.fits.""")
         self.fv.stop_global_plugin(str(self))
 
     def start(self):
-        self.instructions()
         self.resume()
 
     def resume(self):
@@ -490,7 +518,7 @@ Output image will have the filename of <inputname>_<suffix>.fits.""")
         except AttributeError:
             pass
 
-        self.fv.show_status('See instructions')
+        self.fv.show_status('Press "Help" for instructions')
 
     def stop(self):
         self.gui_up = False
