@@ -12,7 +12,46 @@ from ginga import GingaPlugin
 
 
 class Zoom(GingaPlugin.GlobalPlugin):
+    """
+    The Zoom plugin shows an enlarged image of a cutout region centered
+    under the cursor position in the associated channel image.  As the
+    cursor is moved around the image the zoom image updates to allow close
+    inspection of the pixels or precise control in conjunction with other
+    plugin operations.
 
+    Usage
+    -----
+    The size of the cutout radius can be adjusted by the slider below the
+    zoom image labeled "Zoom Radius". The default radius is 30 pixels,
+    making a 61x61 zoom image.  The magnification can be changed by
+    adjusting the "Zoom Amount" slider.
+
+    Two modes of operation are possible: absolute and relative zoom:
+
+    * In absolute mode, the zoom amount controls exactly the zoom level
+      shown in the cutout: for example, the channel image may be zoomed into
+      10X, but the zoom image will only show a 3X image if the zoom amount
+      is set to 3X.
+
+    * In relative mode, the zoom amount setting is interpreted as relative
+      to the zoom setting of the channel image.  If the zoom amount is set
+      to 3X and the channel image is zoomed to 10X then the zoom image shown
+      will be 10+3=13X.  Note that the zoom amount setting can be < 1,
+      so a setting of 1/3X with a 3X zoom in the channel image will produce
+      a 1X zoom image.
+
+    The `Refresh Interval` setting controls how quickly the Zoom plugin
+    responds to the movement of the cursor in updating the zoom image.  The
+    value is specified in milliseconds.
+
+    TIP: Usually setting a small refresh interval *improves* the overall
+	 responsiveness of the Zoom image, and the default value of 20 is
+	 a reasonable one.  You can experiment with the value if the zoom
+	 image seems too jerky or out of sync with the mouse movement in
+         the channel image window.
+
+    The "Defaults" button restores the default settings of the controls.
+    """
     def __init__(self, fv):
         # superclass defines some variables for us, like logger
         super(Zoom, self).__init__(fv)
@@ -30,8 +69,8 @@ class Zoom(GingaPlugin.GlobalPlugin):
 
         # read preferences for this plugin
         prefs = self.fv.get_preferences()
-        self.settings = prefs.createCategory('plugin_Zoom')
-        self.settings.addDefaults(zoom_radius=self.default_radius,
+        self.settings = prefs.create_category('plugin_Zoom')
+        self.settings.add_defaults(zoom_radius=self.default_radius,
                                   zoom_amount=self.default_zoom,
                                   zoom_cmap_name=None,
                                   zoom_imap_name=None,
@@ -72,8 +111,8 @@ class Zoom(GingaPlugin.GlobalPlugin):
         #zi.set_scale_limits(0.001, 1000.0)
         zi.zoom_to(self.default_zoom)
         settings = zi.get_settings()
-        settings.getSetting('zoomlevel').add_callback('set',
-                               self.zoomset, zi)
+        settings.get_setting('zoomlevel').add_callback('set',
+                                 self.zoomset, zi)
         zi.set_bg(0.4, 0.4, 0.4)
         zi.show_pan_mark(True)
         cmname = self.settings.get('zoom_cmap_name', None)
@@ -141,12 +180,9 @@ class Zoom(GingaPlugin.GlobalPlugin):
         vbox2.add_widget(spacer, stretch=1)
 
         vpaned.add_widget(vbox2)
-        #vpaned.add_widget(Widgets.Label(''))
 
         vbox.add_widget(vpaned, stretch=1)
 
-        #vbox.add_widget(Widgets.Label(''), stretch=1)
-
         container.add_widget(sw, stretch=1)
 
     def prepare(self, fitsimage):
@@ -155,12 +191,12 @@ class Zoom(GingaPlugin.GlobalPlugin):
         # TODO: should we add our own canvas instead?
         fitsimage.add_callback('motion', self.motion_cb)
         for name in ['cuts']:
-            fitssettings.getSetting(name).add_callback('set',
+            fitssettings.get_setting(name).add_callback('set',
                                self.cutset_cb, fitsimage)
         fitsimage.add_callback('transform', self.transform_cb)
-        fitssettings.copySettings(zoomsettings, ['rot_deg'])
-        fitssettings.getSetting('rot_deg').add_callback('set', self.rotate_cb, fitsimage)
-        fitssettings.getSetting('zoomlevel').add_callback('set',
+        fitssettings.copy_settings(zoomsettings, ['rot_deg'])
+        fitssettings.get_setting('rot_deg').add_callback('set', self.rotate_cb, fitsimage)
+        fitssettings.get_setting('zoomlevel').add_callback('set',
                                self.zoomset_cb, fitsimage)
 
     def add_channel(self, viewer, chinfo):
