@@ -5,9 +5,20 @@
 # Please see the file LICENSE.txt for details.
 
 import numpy
+from ginga.util import six
 
 import PIL.Image as PILimage
-import PIL.ImageTk as PILimageTk
+
+have_pil_imagetk = False
+try:
+    # sometimes this is not installed even though PIL is
+    from PIL.ImageTk import PhotoImage
+    have_pil_imagetk = True
+except ImportError:
+    if six.PY2:
+        from Tkinter import PhotoImage
+    else:
+        from tkinter import PhotoImage
 
 from ginga import Mixins, Bindings, colors
 from ginga.canvas.mixins import DrawingMixin, CanvasMixin, CompoundMixin
@@ -21,15 +32,15 @@ try:
 
 except ImportError:
     try:
-        # No, hmm..ok, see if we have opencv module...
-        from ginga.cvw.ImageViewCv import ImageViewCv as ImageView, \
-             ImageViewCvError as ImageViewError
+        # No dice. How about the PIL module?
+        from ginga.pilw.ImageViewPil import ImageViewPil as ImageView, \
+             ImageViewPilError as ImageViewError
 
     except ImportError:
         try:
-            # No dice. How about the PIL module?
-            from ginga.pilw.ImageViewPil import ImageViewPil as ImageView, \
-                 ImageViewPilError as ImageViewError
+            # No, hmm..ok, see if we have opencv module...
+            from ginga.cvw.ImageViewCv import ImageViewCv as ImageView, \
+                 ImageViewCvError as ImageViewError
 
         except ImportError:
             # Fall back to mock--there will be no graphic overlays
@@ -65,12 +76,12 @@ class ImageViewTk(ImageView):
         height = canvas.winfo_height()
 
         # see reschedule_redraw() method
-        self._defer_task = TkHelp.Timer(0.0,
-                                        lambda timer: self.delayed_redraw(),
-                                        tkcanvas=canvas)
-        self.msgtask = TkHelp.Timer(0.0,
-                                    lambda timer: self.onscreen_message(None),
-                                    tkcanvas=canvas)
+        self._defer_task = TkHelp.Timer(tkcanvas=canvas)
+        self._defer_task.add_callback('expired',
+                                      lambda timer: self.delayed_redraw())
+        self.msgtask = TkHelp.Timer(tkcanvas=canvas)
+        self.msgtask.add_callback('expired',
+                                  lambda timer: self.onscreen_message(None))
 
         self.configure_window(width, height)
 
@@ -90,12 +101,20 @@ class ImageViewTk(ImageView):
 
         wd, ht = self.get_window_size()
 
-        # Get surface as a numpy array
-        arr8 = self.get_image_as_array()
-
         # make a Tk photo image and stick it to the canvas
-        image = PILimage.fromarray(arr8)
-        photo = PILimageTk.PhotoImage(image)
+
+        if have_pil_imagetk:
+            # Get surface as a numpy array
+            arr8 = self.get_image_as_array()
+            image = PILimage.fromarray(arr8)
+            photo = PhotoImage(image)
+
+        else:
+            # fallback to a little slower method--make a PNG image
+            buf = self.get_rgb_image_as_buffer(format='png')
+            image = buf.getvalue()
+            photo = PhotoImage(data=image)
+
         # hang on to a reference otherwise it gets gc'd
         self.tkphoto = photo
 
@@ -105,7 +124,7 @@ class ImageViewTk(ImageView):
         cr.config(scrollregion=cr.bbox('all'))
 
     def reschedule_redraw(self, time_sec):
-        self._defer_task.cancel()
+        self._defer_task.stop()
         self._defer_task.start(time_sec)
 
     def configure_window(self, width, height):
@@ -122,7 +141,7 @@ class ImageViewTk(ImageView):
     def onscreen_message(self, text, delay=None, redraw=True):
         if self.tkcanvas is None:
             return
-        self.msgtask.cancel()
+        self.msgtask.stop()
         self.set_onscreen_message(text, redraw=redraw)
         if delay is not None:
             self.msgtask.start(delay)
@@ -198,6 +217,16 @@ class ImageViewEvent(ImageViewTk):
             'f10': 'f10',
             'f11': 'f11',
             'f12': 'f12',
+            'right': 'right',
+            'left': 'left',
+            'up': 'up',
+            'down': 'down',
+            'insert': 'insert',
+            'delete': 'delete',
+            'home': 'home',
+            'end': 'end',
+            'prior': 'page_up',
+            'next': 'page_down',
             }
 
         # Define cursors for pick and pan
@@ -294,8 +323,7 @@ class ImageViewEvent(ImageViewTk):
                 self.logger.debug("scroll deg=%f direction=%f" % (
                     numDegrees, direction))
 
-                data_x, data_y = self.get_data_xy(x, y)
-                self.last_data_x, self.last_data_y = data_x, data_y
+                data_x, data_y = self.check_cursor_location()
 
                 return self.make_ui_callback('scroll', direction, numDegrees,
                                           data_x, data_y)
@@ -304,8 +332,7 @@ class ImageViewEvent(ImageViewTk):
         self._button = button
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
@@ -323,8 +350,7 @@ class ImageViewEvent(ImageViewTk):
         self._button = 0
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
@@ -343,8 +369,7 @@ class ImageViewEvent(ImageViewTk):
         #     button |= 0x4
         self.logger.debug("motion event at %dx%d, button=%x" % (x, y, button))
 
-        data_x, data_y = self.get_data_xy(x, y)
-        self.last_data_x, self.last_data_y = data_x, data_y
+        data_x, data_y = self.check_cursor_location()
 
         return self.make_ui_callback('motion', button, data_x, data_y)
 
