@@ -21,6 +21,7 @@ any images.  Otherwise Ginga will try to pick one for you.
 """
 import numpy
 
+from ginga.misc import Bunch
 from ginga.util import iohelper
 
 fits_configured = False
@@ -72,11 +73,21 @@ class BaseFitsFileHandler(object):
         super(BaseFitsFileHandler, self).__init__()
 
         self.logger = logger
+        self.fileinfo = None
+        self.fits_f = None
+        self.hdu_info = []
+        self.hdu_db = {}
+        self.extver_db = {}
         self.factory_dict = {}
 
     def register_type(self, name, klass):
         self.factory_dict[name.lower()] = klass
 
+    def get_factory(self):
+        hdlr = self.__class__(self.logger)
+        hdlr.factory_dict.update(self.factory_dict)
+        return hdlr
+
 
 class PyFitsFileHandler(BaseFitsFileHandler):
 
@@ -104,8 +115,10 @@ class PyFitsFileHandler(BaseFitsFileHandler):
 
     def load_hdu(self, hdu, dstobj=None, **kwargs):
 
-        if (isinstance(hdu, pyfits.ImageHDU) or
-                isinstance(hdu, pyfits.PrimaryHDU)):
+        if isinstance(hdu, (pyfits.ImageHDU,
+                            pyfits.CompImageHDU,
+                            pyfits.PrimaryHDU,
+                            )):
             # <-- data is an image
 
             if dstobj is None:
@@ -145,113 +158,168 @@ class PyFitsFileHandler(BaseFitsFileHandler):
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
                   **kwargs):
+        inherit_primary_header = kwargs.pop('inherit_primary_header', False)
+        opener = self.get_factory()
+        opener.open_file(filespec, memmap=memmap, **kwargs)
+        try:
+            return opener.get_hdu(
+                numhdu, dstobj=dstobj,
+                inherit_primary_header=inherit_primary_header)
+        finally:
+            opener.close()
+
+    def open_file(self, filespec, memmap=None, **kwargs):
 
         info = iohelper.get_fileinfo(filespec)
         if not info.ondisk:
             raise FITSError("File does not appear to be on disk: %s" % (
                 info.url))
 
+        self.fileinfo = info
         filepath = info.filepath
-        if numhdu is None:
-            numhdu = info.numhdu
 
         self.logger.debug("Loading file '%s' ..." % (filepath))
-        with pyfits.open(filepath, 'readonly', memmap=memmap) as fits_f:
+        fits_f = pyfits.open(filepath, 'readonly', memmap=memmap)
+        self.fits_f = fits_f
 
-            # this seems to be necessary now for some fits files...
-            try:
-                fits_f.verify('fix')
+        # this seems to be necessary now for some fits files...
+        try:
+            fits_f.verify('fix')
 
-            except Exception as e:
-                # Let's hope for the best!
-                self.logger.warn("Problem verifying fits file '%s': %s" % (
-                    filepath, str(e)))
+        except Exception as e:
+            # Let's hope for the best!
+            self.logger.warn("Problem verifying fits file '%s': %s" % (
+                filepath, str(e)))
 
-            if numhdu is None:
-                try:
-                    # this can fail for certain FITS files, with a "name undefined"
-                    # error bubbling up from astropy
-                    hduinfo = fits_f.info(output=False)
+        try:
+            # this can fail for certain FITS files, with a "name undefined"
+            # error bubbling up from astropy
+            _hduinfo = fits_f.info(output=False)
 
-                except Exception as e:
-                    # if so, this insures that name will be translated into the
-                    # HDU index below
-                    hduinfo = tuple((None, '') for i in range(len(fits_f)))
+        except Exception as e:
+            # if so, this insures that name will be translated into the
+            # HDU index below
+            _hduinfo = tuple((None, '') for i in range(len(fits_f)))
 
-                extver_db = {}
+        idx = 0
+        extver_db = {}
+        self.hdu_info = []
+        self.hdu_db = {}
 
-                found_valid_hdu = False
-                for i in range(len(fits_f)):
-                    hdu = fits_f[i]
-                    tup = hduinfo[i]
-                    name = tup[1]
-                    # figure out the EXTVER for this HDU
-                    extver = extver_db.setdefault(name, 0)
-                    extver += 1
-                    extver_db[name] = extver
+        for tup in _hduinfo:
+            name = tup[1]
+            # figure out the EXTVER for this HDU
+            extver = extver_db.setdefault(name, 0)
+            extver += 1
+            extver_db[name] = extver
 
-                    # rule out HDUs we can't deal with
-                    if not isinstance(hdu, (pyfits.ImageHDU,
-                                            pyfits.PrimaryHDU,
-                                            pyfits.TableHDU,
-                                            pyfits.BinTableHDU,
-                                            )):
-                        continue
-                    if not isinstance(hdu.data, numpy.ndarray):
-                        # We need to open a numpy array
-                        continue
-                    if 0 in hdu.data.shape:
-                        # non-pixel or zero-length data hdu?
-                        continue
+            # prepare a record of pertinent info about the HDU for
+            # lookups by numerical index or (NAME, EXTVER)
+            d = Bunch.Bunch(index=idx, name=name, extver=extver)
+            if len(tup) > 5:
+                import astropy
+                from astropy.utils.introspection import minversion
+                if minversion(astropy, '2.0'):
+                    d.setvals(htype=tup[3], dtype=tup[6])
+                else:
+                    d.setvals(htype=tup[2], dtype=tup[5])
+            self.hdu_info.append(d)
+            # different ways of accessing this HDU:
+            # by numerical index
+            self.hdu_db[idx] = d
+            # by (hduname, extver)
+            self.hdu_db[(name, extver)] = d
+            idx += 1
 
-                    # Looks good, let's try it
-                    found_valid_hdu = True
-                    _numhdu = (name, extver)
-                    if (len(name) == 0) or (_numhdu not in fits_f):
-                        numhdu = i
-                    else:
-                        numhdu = _numhdu
-                    break
+        self.extver_db = extver_db
 
-                if not found_valid_hdu:
-                    # Load just the header
-                    hdu = fits_f[0]
-                    name = hdu.name
-                    extver = hdu.ver
-                    _numhdu = (name, extver)
-                    if (len(name) == 0) or (_numhdu not in fits_f):
-                        numhdu = 0
-                    else:
-                        numhdu = _numhdu
+    def close(self):
+        self.hdu_info = None
+        self.hdu_db = {}
+        self.extver_db = {}
+        self.info = None
+        self.fits_f = None
 
-            elif isinstance(numhdu, (int, str)):
-                hdu = fits_f[numhdu]
-                name = hdu.name
-                extver = hdu.ver
+    def __len__(self):
+        return len(self.hdu_info)
+
+    def get_hdu(self, numhdu, dstobj=None, **kwargs):
+
+        if numhdu is None:
+            found_valid_hdu = False
+            for i, d in enumerate(self.hdu_info):
+                name = d.name
+                hdu = self.fits_f[i]
+
+                # rule out HDUs we can't deal with
+                if not isinstance(hdu, (pyfits.ImageHDU,
+                                        pyfits.CompImageHDU,
+                                        pyfits.PrimaryHDU,
+                                        pyfits.TableHDU,
+                                        pyfits.BinTableHDU,
+                                        )):
+                    continue
+
+                if not isinstance(hdu.data, numpy.ndarray):
+                    # We need to open a numpy array
+                    continue
+
+                if 0 in hdu.data.shape:
+                    # non-pixel or zero-length data hdu?
+                    continue
+
+                # Looks good, let's try it
+                found_valid_hdu = True
+                extver = d.extver
                 _numhdu = (name, extver)
-                if (len(name) > 0) and (_numhdu in fits_f):
+                if (len(name) == 0) or (_numhdu not in self.fits_f):
+                    numhdu = i
+                else:
+                    numhdu = _numhdu
+                break
+
+            if not found_valid_hdu:
+                # Load just the header
+                hdu = self.fits_f[0]
+                d = self.hdu_info[0]
+                name = d.name
+                extver = d.extver
+                _numhdu = (name, extver)
+                if (len(name) == 0) or (_numhdu not in self.fits_f):
+                    numhdu = 0
+                else:
                     numhdu = _numhdu
 
-            self.logger.debug("HDU index looks like: %s" % str(numhdu))
-            hdu = fits_f[numhdu]
+        elif isinstance(numhdu, (int, str)):
+            hdu = self.fits_f[numhdu]
+            name = hdu.name
+            extver = hdu.ver
+            _numhdu = (name, extver)
+            if (len(name) > 0) and (_numhdu in self.fits_f):
+                numhdu = _numhdu
 
-            dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=fits_f,
-                                   **kwargs)
+        self.logger.debug("HDU index looks like: %s" % str(numhdu))
+        if numhdu not in self.fits_f:
+            info = self.hdu_db[numhdu]
+            hdu = self.fits_f[info.index]
+        else:
+            hdu = self.fits_f[numhdu]
 
-            # Set the name if no name currently exists for this object
-            # TODO: should this *change* the existing name, if any?
-            name = dstobj.get('name', None)
-            if name is None:
-                name = info.name
-                if ('[' not in name):
-                    ## if (numhdu is None) or allow_numhdu_override:
-                    ##     numhdu = numhdu_
-                    name += iohelper.get_hdu_suffix(numhdu)
-                dstobj.set(name=name)
+        dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=self.fits_f,
+                               **kwargs)
 
-            dstobj.set(path=filepath, idx=numhdu)
+        # Set the name if no name currently exists for this object
+        # TODO: should this *change* the existing name, if any?
+        name = dstobj.get('name', None)
+        if name is None:
+            name = self.fileinfo.name
+            if '[' not in name:
+                name += iohelper.get_hdu_suffix(numhdu)
+            dstobj.set(name=name)
 
-            return dstobj
+        dstobj.set(path=self.fileinfo.filepath, idx=numhdu)
+
+        return dstobj
 
     def create_fits(self, data, header):
         fits_f = pyfits.HDUList()
@@ -283,6 +351,10 @@ class FitsioFileHandler(BaseFitsFileHandler):
 
         super(FitsioFileHandler, self).__init__(logger)
         self.kind = 'fitsio'
+        self.hdutypes = {fitsio.IMAGE_HDU: 'ImageHDU',
+                         fitsio.BINARY_TBL: 'BinaryTBL',
+                         fitsio.ASCII_TBL: 'AsciiTBL',
+                         }
 
     def fromHDU(self, hdu, ahdr):
         header = hdu.read_header()
@@ -294,104 +366,166 @@ class FitsioFileHandler(BaseFitsFileHandler):
     def load_hdu(self, hdu, dstobj=None, **kwargs):
         from ginga import AstroImage  # Put here to avoid circular import
 
-        if dstobj is None:
-            # get model class for this type of object
-            obj_class = self.factory_dict.get('image', None)
-            if obj_class is None:
-                raise FITSError(
-                    "I don't know how to load objects of kind 'image'")
-
-            dstobj = obj_class(logger=self.logger)
-
-        data = hdu.read()
+        hduinfo = hdu.get_info()
+        hdutype = hduinfo.get('hdutype', None)
 
         ahdr = AstroImage.AstroHeader()
         self.fromHDU(hdu, ahdr)
 
         metadata = dict(header=ahdr)
-        dstobj.load_data(data, metadata=metadata)
+        data = hdu.read()
+
+        if hdutype == fitsio.IMAGE_HDU:
+            # <-- data is an image
+
+            if dstobj is None:
+                # get model class for this type of object
+                obj_class = self.factory_dict.get('image', None)
+                if obj_class is None:
+                    raise FITSError(
+                        "I don't know how to load objects of kind 'image'")
+
+                dstobj = obj_class(logger=self.logger)
+
+            dstobj.load_data(data, metadata=metadata)
+
+        elif hdutype in (fitsio.ASCII_TBL, fitsio.BINARY_TBL):
+            # <-- data is a table
+            raise FITSError(
+                "FITS tables are not yet readable using ginga/fitsio")
 
         return dstobj
 
     def load_file(self, filespec, numhdu=None, dstobj=None, memmap=None,
                   **kwargs):
+        inherit_primary_header = kwargs.pop('inherit_primary_header', False)
+        opener = self.get_factory()
+        opener.open_file(filespec, memmap=memmap, **kwargs)
+        try:
+            return opener.get_hdu(
+                numhdu, dstobj=dstobj,
+                inherit_primary_header=inherit_primary_header)
+        finally:
+            opener.close()
+
+    def open_file(self, filespec, memmap=None, **kwargs):
 
         info = iohelper.get_fileinfo(filespec)
         if not info.ondisk:
             raise FITSError("File does not appear to be on disk: %s" % (
                 info.url))
 
+        self.fileinfo = info
         filepath = info.filepath
-        if numhdu is None:
-            numhdu = info.numhdu
 
         self.logger.debug("Loading file '%s' ..." % (filepath))
         fits_f = fitsio.FITS(filepath, memmap=memmap)
-        try:
+        self.fits_f = fits_f
 
-            if numhdu is None:
+        extver_db = {}
+        self.extver_db = extver_db
+        self.hdu_info = []
+        self.hdu_db = {}
 
-                found_valid_hdu = False
-                for i in range(len(fits_f)):
-                    hdu = fits_f[i]
-                    hduinfo = hdu.get_info()
-                    #print(hduinfo)
-                    name = hduinfo['extname']
-                    extver = hduinfo['extver']
+        for idx in range(len(fits_f)):
+            hdu = fits_f[idx]
+            hduinfo = hdu.get_info()
+            name = hduinfo['extname']
+            extver = hduinfo['extver']
+            extver_db[name] = extver
 
-                    if not ('ndims' in hduinfo) or (hduinfo['ndims'] == 0):
-                        # compressed FITS file or non-pixel data hdu?
-                        continue
-                    #print(dir(hdu))
-                    # Looks good, let's try it
-                    found_valid_hdu = True
-                    if len(name) == 0:
-                        numhdu = i
-                    else:
-                        numhdu = (name, extver)
-                    break
+            # prepare a record of pertinent info about the HDU for
+            # lookups by numerical index or (NAME, EXTVER)
+            d = Bunch.Bunch(index=idx, name=name, extver=extver)
+            hdutype = self.hdutypes.get(hduinfo['hdutype'], 'UNKNOWN')
+            d.setvals(htype=hdutype,
+                      dtype=str(hduinfo.get('img_type', None)))
+            self.hdu_info.append(d)
+            # different ways of accessing this HDU:
+            # by numerical index
+            self.hdu_db[idx] = d
+            # by (hduname, extver)
+            self.hdu_db[(name, extver)] = d
 
-                if not found_valid_hdu:
-                    # Just load the header
-                    hdu = fits_f[0]
-                    hduinfo = hdu.get_info()
-                    name = hduinfo['extname']
-                    extver = hduinfo['extver']
-                    if len(name) == 0:
-                        numhdu = 0
-                    else:
-                        numhdu = (name, extver)
+    def close(self):
+        self.hdu_info = None
+        self.hdu_db = {}
+        self.extver_db = {}
+        self.info = None
+        self.fits_f = None
 
-            elif isinstance(numhdu, (int, str)):
-                hdu = fits_f[numhdu]
+    def __len__(self):
+        return len(self.hdu_info)
+
+    def get_hdu(self, numhdu, dstobj=None, **kwargs):
+
+        if numhdu is None:
+            found_valid_hdu = False
+            for i, d in enumerate(self.hdu_info):
+                name = d.name
+                extver = d.extver
+                hdu = self.fits_f[i]
                 hduinfo = hdu.get_info()
-                name = hduinfo['extname']
-                extver = hduinfo['extver']
-                if len(name) > 0:
+
+                if not ('ndims' in hduinfo) or (hduinfo['ndims'] == 0):
+                    # compressed FITS file or non-pixel data hdu?
+                    continue
+
+                if not hasattr(hdu, 'read'):
+                    continue
+                data = hdu.read()
+
+                if not isinstance(data, numpy.ndarray):
+                    # We need to open a numpy array
+                    continue
+
+                if 0 in data.shape:
+                    # non-pixel or zero-length data hdu?
+                    continue
+
+                # Looks good, let's try it
+                found_valid_hdu = True
+                if len(name) == 0:
+                    numhdu = i
+                else:
+                    numhdu = (name, extver)
+                break
+
+            if not found_valid_hdu:
+                # Just load the header
+                hdu = self.fits_f[0]
+                d = self.hdu_info[0]
+                name = d.name
+                extver = d.extver
+                if len(name) == 0:
+                    numhdu = 0
+                else:
                     numhdu = (name, extver)
 
-            self.logger.debug("HDU index looks like: %s" % str(numhdu))
-            hdu = fits_f[numhdu]
+        elif isinstance(numhdu, (int, str)):
+            hdu = self.fits_f[numhdu]
+            hduinfo = hdu.get_info()
+            name = hduinfo['extname']
+            extver = hduinfo['extver']
+            if len(name) > 0:
+                numhdu = (name, extver)
 
-            dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=fits_f,
-                                   **kwargs)
+        self.logger.debug("HDU index looks like: %s" % str(numhdu))
+        hdu = self.fits_f[numhdu]
 
-            # Read PRIMARY header
-            #self.fromHDU(fits_f[0], phdr)
-
-        finally:
-            fits_f.close()
+        dstobj = self.load_hdu(hdu, dstobj=dstobj, fobj=self.fits_f,
+                               **kwargs)
 
         # Set the name if no name currently exists for this object
         # TODO: should this *change* the existing name, if any?
         name = dstobj.get('name', None)
         if name is None:
-            name = info.name
-            if ('[' not in name):
+            name = self.fileinfo.name
+            if '[' not in name:
                 name += iohelper.get_hdu_suffix(numhdu)
             dstobj.set(name=name)
 
-        dstobj.set(path=filepath, idx=numhdu)
+        dstobj.set(path=self.fileinfo.filepath, idx=numhdu)
 
         return dstobj
 
