@@ -17,7 +17,7 @@ import binascii
 from collections import namedtuple
 from io import BytesIO
 
-from ginga.misc import Bunch
+from ginga.misc import Bunch, Callback
 from ginga.util import io_rgb
 
 default_interval = 10
@@ -25,7 +25,8 @@ default_interval = 10
 ConfigEvent = namedtuple("ConfigEvent", ["type", "id", "width", "height"])
 InputEvent = namedtuple("InputEvent", ["type", "id", "x", "y", "button",
                                        "delta", "alt_key", "ctrl_key",
-                                       "meta_key", "shift_key", "key_code"])
+                                       "meta_key", "shift_key", "key_code",
+                                       "key_name"])
 GestureEvent = namedtuple("GestureEvent", ["type", "id", "x", "y", "dx", "dy",
                                            "distance",
                                            "theta", "direction", "vx", "vy",
@@ -170,47 +171,60 @@ class WindowHandler(tornado.web.RequestHandler):
         self.write(output)
 
 
-class Timer(object):
+class Timer(Callback.Callbacks):
     """Abstraction of a GUI-toolkit implemented timer."""
 
-    def __init__(self, ival_sec, expire_cb, data=None, mplcanvas=None):
-        """Create a timer set to expire after `ival_sec` and which will
-        call the callable `expire_cb` when it expires.
+    def __init__(self, duration=0.0, mplcanvas=None):
+        """Create a timer set to expire after `duration` sec.
         """
-        self.ival_sec = ival_sec
-        self.cb = expire_cb
-        self.data = data
+        super(Timer, self).__init__()
+
+        self.duration = duration
+        # For storing aritrary data with timers
+        self.data = Bunch.Bunch()
 
         self._timer = mplcanvas.new_timer()
         self._timer.single_shot = True
         self._timer.add_callback(self._redirect_cb)
 
-    def start(self, ival_sec=None):
-        """Start the timer.  If `ival_sec` is not None, it should
+        for name in ('expired', 'canceled'):
+            self.enable_callback(name)
+
+    def start(self, duration=None):
+        """Start the timer.  If `duration` is not None, it should
         specify the time to expiration in seconds.
         """
-        if ival_sec is None:
-            ival_sec = self.ival_sec
+        if duration is None:
+            duration = self.duration
 
-        self.cancel()
+        self.set(duration)
 
-        # Matplotlib timer set in milliseconds
-        time_ms = int(ival_sec * 1000.0)
+    def set(self, duration):
+
+        self.stop()
+
+        # pg timer set in milliseconds
+        time_ms = int(duration * 1000.0)
         self._timer.interval = time_ms
         self._timer.start()
 
     def _redirect_cb(self):
-        self.cb(self)
+        self.make_callback('expired')
 
-    def cancel(self):
-        """Cancel this timer.  If the timer is not running, there
-        is no error.
-        """
+    def stop(self):
         try:
             self._timer.stop()
         except:
             pass
 
+    def cancel(self):
+        """Cancel this timer.  If the timer is not running, there
+        is no error.
+        """
+        self.stop()
+        self.make_callback('canceled')
+
+    clear = cancel
 
 def get_image_src_from_buffer(img_buf, imgtype='png'):
     img_string = binascii.b2a_base64(img_buf)
