@@ -20,17 +20,15 @@ Tested with Chromium 41.0.2272.76, Firefox 37.0.2, Safari 7.1.6
 """
 from __future__ import print_function
 import sys, os
-import math
 import logging
 import threading
 
 import tornado.web
 import tornado.ioloop
 
-from ginga import AstroImage, colors
+from ginga import AstroImage
 from ginga.canvas.CanvasObject import get_canvas_types
-from ginga.misc import log, Task
-from ginga.util import catalog, iohelper
+from ginga.misc import log, Task, Bunch
 from ginga.Bindings import ImageViewBindings
 from ginga.misc.Settings import SettingGroup
 from ginga.util.paths import ginga_home
@@ -38,7 +36,27 @@ from ginga.util.paths import ginga_home
 from ginga.web.pgw import templates, js, PgHelp, Widgets, Viewers
 
 
-class EnhancedCanvasView(Viewers.CanvasView):
+class BasicCanvasView(Viewers.CanvasView):
+
+    def build_gui(self, container):
+        """
+        This is responsible for building the viewer's UI.  It should
+        place the UI in `container`.  Override this to make a custom
+        UI.
+        """
+        vbox = Widgets.VBox()
+        vbox.set_border_width(0)
+
+        w = Viewers.GingaViewerWidget(viewer=self)
+        vbox.add_widget(w, stretch=1)
+
+        # need to put this in an hbox with an expanding label or the
+        # browser wants to resize the canvas, distorting it
+        hbox = Widgets.HBox()
+        hbox.add_widget(vbox, stretch=0)
+        hbox.add_widget(Widgets.Label(''), stretch=1)
+
+        container.set_widget(hbox)
 
     def embed(self, width=600, height=650):
         """
@@ -55,7 +73,7 @@ class EnhancedCanvasView(Viewers.CanvasView):
         import webbrowser
         webbrowser.open(self.url, new=new)
 
-    def show(self):
+    def show(self, fmt=None):
         """
         Capture the window of a viewer.
         """
@@ -65,8 +83,14 @@ class EnhancedCanvasView(Viewers.CanvasView):
         self.redraw_now()
 
         from IPython.display import Image
-        return Image(data=bytes(self.get_rgb_image_as_bytes(format='png')),
-                     format='png', embed=True)
+
+        if fmt is None:
+            # what format are we using for the HTML5 canvas--use that
+            settings = self.get_settings()
+            fmt = settings.get('html5_canvas_format', 'png')
+
+        return Image(data=bytes(self.get_rgb_image_as_bytes(format=fmt)),
+                     format=fmt, embed=True)
 
     def load_fits(self, filepath):
         """
@@ -106,7 +130,7 @@ class EnhancedCanvasView(Viewers.CanvasView):
         canvas.enable_draw(True)
         canvas.enable_edit(True)
         canvas.set_drawtype(None)
-        canvas.ui_setActive(True)
+        canvas.ui_set_active(True)
         canvas.set_surface(self)
         canvas.register_for_cursor_drawing(self)
         # add the canvas to the view.
@@ -114,85 +138,53 @@ class EnhancedCanvasView(Viewers.CanvasView):
 
         return canvas
 
-class ImageViewer(object):
+    def set_html5_canvas_format(self, fmt):
+        """
+        Sets the format used for rendering to the HTML5 canvas.
+        'png' offers greater clarity, especially for small text, but
+        does not have as good of performance as 'jpeg'.
+        """
+        fmt = fmt.lower()
+        if not fmt in ('jpeg', 'png'):
+            raise ValueError("Format must be one of {jpeg|png} not '%s'" % (
+                fmt))
 
-    def __init__(self, logger, window, viewer_class=None, width=512, height=512):
-        if viewer_class is None:
-            viewer_class = EnhancedCanvasView
-        self.logger = logger
-        self.url = window.url
-        self.dc = get_canvas_types()
-        self.pixel_base = 1.0
+        settings = self.get_settings()
+        settings.set(html5_canvas_format=fmt)
 
-        self.top = window
-        self.top.add_callback('close', self.closed)
+    def get_html5_canvas_format(self):
+        settings = self.get_settings()
+        return settings.get('html5_canvas_format')
 
+
+class EnhancedCanvasView(BasicCanvasView):
+    """
+    Like BasicCanvasView, but includes a readout widget for when the
+    cursor is moved over the canvas to display the coordinates.
+    """
+
+    def build_gui(self, container):
+        """
+        This is responsible for building the viewer's UI.  It should
+        place the UI in `container`.
+        """
         vbox = Widgets.VBox()
         vbox.set_border_width(2)
         vbox.set_spacing(1)
 
-        # load binding preferences if available
-        cfgfile = os.path.join(ginga_home, "ipg_bindings.cfg")
-        bindprefs = SettingGroup(name='bindings', logger=logger,
-                                 preffile=cfgfile)
-        bindprefs.load(onError='silent')
-
-        bd = ImageViewBindings(logger, settings=bindprefs)
-
-        fi = viewer_class(logger, bindings=bd)
-        fi.url = self.url
-        fi.enable_autocuts('on')
-        fi.set_autocut_params('zscale')
-        fi.set_zoom_algorithm('rate')
-        fi.set_zoomrate(1.1)
-        fi.enable_autozoom('on')
-        #fi.set_callback('drag-drop', self.drop_file)
-        fi.set_callback('none-move', self.motion)
-        fi.set_bg(0.2, 0.2, 0.2)
-        fi.ui_setActive(True)
-        self.fitsimage = fi
-        fi.ipg_parent = self
-
-        bd = fi.get_bindings()
-        bd.enable_all(True)
-
-        # canvas that we will draw on
-        canvas = self.dc.DrawingCanvas()
-        canvas.set_surface(fi)
-        self.canvas = canvas
-        # add canvas to view
-        private_canvas = fi.get_canvas()
-        private_canvas.add(canvas)
-        canvas.ui_setActive(True)
-        fi.set_canvas(canvas)
-
-        fi.set_desired_size(width, height)
-        # force allocation of a surface--may be resized later
-        fi.configure_surface(width, height)
-
-        # add little mode indicator that shows modal states in
-        # the corner
-        fi.show_mode_indicator(True)
-
-        w = Viewers.GingaViewerWidget(viewer=fi)
+        w = Viewers.GingaViewerWidget(viewer=self)
         vbox.add_widget(w, stretch=1)
 
+        # set up to capture cursor movement for reading out coordinates
+
+        # coordinates reported in base 1 or 0?
+        self.pixel_base = 1.0
+
         self.readout = Widgets.Label("")
         vbox.add_widget(self.readout, stretch=0)
 
-        hbox = Widgets.HBox()
-        hbox.add_widget(Widgets.Label('Zoom sensitivity: '))
-        slider = Widgets.Slider(orientation='horizontal', dtype=float)
-        slider.add_callback('value-changed',
-                            lambda w, val: self.adjust_scrolling_accel_cb(val))
-        slider.set_limits(1.0, 9.5, 0.1)
-        val = 4.0
-        slider.set_value(val)
-        self.adjust_scrolling_accel_cb(val)
-        hbox.add_widget(slider, stretch=1)
-
-        hbox.add_widget(Widgets.Label(''), stretch=1)
-        vbox.add_widget(hbox, stretch=0)
+        #self.set_callback('none-move', self.motion_cb)
+        self.set_callback('cursor-changed', self.motion_cb)
 
         # need to put this in an hbox with an expanding label or the
         # browser wants to resize the canvas, distorting it
@@ -200,24 +192,12 @@ class ImageViewer(object):
         hbox.add_widget(vbox, stretch=0)
         hbox.add_widget(Widgets.Label(''), stretch=1)
 
-        self.top.set_widget(hbox)
+        container.set_widget(hbox)
 
-    def load_file(self, filepath):
-        image = AstroImage.AstroImage(logger=self.logger)
-        image.load_file(filepath)
-
-        self.fitsimage.set_image(image)
-        self.top.set_title(filepath)
-
-    def drop_file(self, fitsimage, paths):
-        fileName = paths[0]
-        self.load_file(fileName)
-
-    def motion(self, viewer, button, data_x, data_y):
+    def motion_cb(self, viewer, button, data_x, data_y):
 
         # Get the value under the data coordinates
         try:
-            #value = viewer.get_data(data_x, data_y)
             # We report the value across the pixel, even though the coords
             # change halfway across the pixel
             value = viewer.get_data(int(data_x+0.5), int(data_y+0.5))
@@ -250,65 +230,86 @@ class ImageViewer(object):
     def set_readout_text(self, text):
         self.readout.set_text(text)
 
-    def adjust_scrolling_accel_cb(self, val):
-        def f(x):
-            return (math.log10(x) / (10 - x) * x/12) + 1.0001
-        val2 = f(val)
-        self.logger.debug("slider value is %f, setting will be %f" % (val, val2))
-        # save scale
-        scale_x, scale_y = self.fitsimage.get_scale_xy()
-        self.fitsimage.set_zoomrate(val2)
-        # restore scale
-        self.fitsimage.scale_to(scale_x, scale_y)
-        return True
-
-    def closed(self, w):
-        self.logger.info("Top window closed.")
-        self.top = None
-        sys.exit()
-
-
-class FileHandler(tornado.web.RequestHandler):
-
-    def initialize(self, name, factory):
-        self.name = name
-        self.viewer_factory = factory
-        self.logger = factory.logger
-        self.logger.info("filehandler initialize")
-
-    def get(self):
-        self.logger.info("filehandler get")
-        # Collect arguments
-        wid = self.get_argument('id', None)
-
-        # Get window with this id
-        window = self.app.get_window(wid)
-
-        output = window.render()
-        self.write(output)
 
 class ViewerFactory(object):
+    """
+    This is a factory class that churns out web viewers for a web
+    application.
 
-    def __init__(self, logger, basedir, app, thread_pool):
+    The most important method of interest is get_viewer().
+    """
+
+    def __init__(self, logger, app, thread_pool):
         """
-        Constructor parameters:
-          `logger` : a logging-module compatible logger object
-          `basedir`: directory to which paths requested on the viewer
-                        are considered relative to.
+        Parameters
+        ----------
+        logger : python compatible logger
+            a logging-module compatible logger object
+        app : ginga pgw web application object
+        thread_pool : a ginga thread pool
         """
         self.logger = logger
-        self.basedir = basedir
         self.app = app
         self.thread_pool = thread_pool
+        self.dc = get_canvas_types()
         # dict of viewers
         self.viewers = {}
 
-    def get_basedir(self):
-        return self.basedir
-
     def get_threadpool(self):
         return self.thread_pool
 
+    def make_viewer(self, window, viewer_class=None,
+                    width=512, height=512):
+        if viewer_class is None:
+            viewer_class = EnhancedCanvasView
+
+        # load binding preferences if available
+        cfgfile = os.path.join(ginga_home, "ipg_bindings.cfg")
+        bindprefs = SettingGroup(name='bindings', logger=self.logger,
+                                 preffile=cfgfile)
+        bindprefs.load(onError='silent')
+
+        bd = ImageViewBindings(self.logger, settings=bindprefs)
+
+        fi = viewer_class(self.logger, bindings=bd)
+        fi.url = window.url
+        # set up some reasonable defaults--user can change these later
+        # if desired
+        fi.set_autocut_params('zscale')
+        fi.enable_autocuts('on')
+        fi.enable_autozoom('on')
+        fi.set_bg(0.2, 0.2, 0.2)
+        fi.ui_set_active(True)
+        fi.ipg_parent = self
+
+        # enable most key/mouse operations
+        bd = fi.get_bindings()
+        bd.enable_all(True)
+
+        # set up a non-private canvas for drawing
+        canvas = self.dc.DrawingCanvas()
+        canvas.set_surface(fi)
+        # add canvas to view
+        private_canvas = fi.get_canvas()
+        private_canvas.add(canvas)
+        canvas.ui_set_active(True)
+        fi.set_canvas(canvas)
+
+        fi.set_desired_size(width, height)
+        # force allocation of a surface--may be resized later
+        fi.configure_surface(width, height)
+
+        # add little mode indicator that shows modal states in
+        # the corner
+        fi.show_mode_indicator(True)
+
+        # Have the viewer build it's UI into the container
+        fi.build_gui(window)
+
+        v_info = Bunch.Bunch(url=window.url, viewer=fi,
+                             top=window)
+        return v_info
+
     def get_viewer(self, v_id, viewer_class=None, width=512, height=512,
                    force_new=False):
         """
@@ -324,13 +325,13 @@ class ViewerFactory(object):
         #  create top level window
         window = self.app.make_window("Viewer %s" % v_id, wid=v_id)
 
-        # our own viewer object, customized with methods (see above)
-        viewer = ImageViewer(self.logger, window,
-                             viewer_class=viewer_class, width=width, height=height)
-        #viewer.url = window.url
+        # We get back a record with information about the viewer
+        v_info = self.make_viewer(window, viewer_class=viewer_class,
+                                  width=width, height=height)
 
-        self.viewers[v_id] = viewer
-        return viewer
+        # Save it under this viewer id
+        self.viewers[v_id] = v_info
+        return v_info
 
     def delete_viewer(self, v_id):
         del self.viewers[v_id]
@@ -338,10 +339,57 @@ class ViewerFactory(object):
     def delete_all_viewers(self):
         self.viewers = {}
 
+
+class FileHandler(tornado.web.RequestHandler):
+    """
+    This is a handler that is started to allow a REST-type web API to
+    create and manipulate viewers.
+
+    Currently it only allows the following commands:
+        .../viewer?id=v1&cmd=get           Create/access a viewer
+        .../viewer?id=v1&cmd=load&path=... Load the viewer
+    """
+
+    def initialize(self, name, factory):
+        self.name = name
+        self.factory = factory
+        self.logger = factory.logger
+        self.logger.debug("filehandler initialize")
+
+    def get(self):
+        self.logger.debug("filehandler get")
+        # Collect arguments
+        # TODO: width, height?
+        cmd = self.get_argument('cmd', 'get')
+        v_id = self.get_argument('id', 'v1')
+
+        v_info = self.factory.get_viewer(v_id)
+
+        if cmd == 'get':
+            self._do_get(v_info)
+
+        elif cmd == 'load':
+            self._do_load(v_info)
+
+    def _do_get(self, v_info):
+        # Get window
+        window = v_info.top
+
+        # render back to caller
+        output = window.render()
+        self.write(output)
+
+    def _do_load(self, v_info):
+        path = self.get_argument('path', None)
+        if path is not None:
+            v_info.viewer.load_fits(path)
+
+
 class WebServer(object):
 
     def __init__(self, app, thread_pool, factory,
-                 host='localhost', port=9909, ev_quit=None):
+                 host='localhost', port=9909, ev_quit=None,
+                 viewer_class=None):
 
         self.host = host
         self.port = port
@@ -352,6 +400,7 @@ class WebServer(object):
         if ev_quit is None:
             ev_quit = threading.Event()
         self.ev_quit = ev_quit
+        self.default_viewer_class = viewer_class
 
         self.server = None
         self.http_server = None
@@ -370,8 +419,6 @@ class WebServer(object):
               dict(name='Application', url='/app', app=self.app)),
             (r"/app/socket", PgHelp.ApplicationHandler,
               dict(name='Ginga', app=self.app)),
-            ## ("/viewer/socket", ViewerWidget,
-            ##  dict(name='Ginga', factory=self.factory)),
             ],
                factory=self.factory, logger=self.logger)
 
@@ -402,19 +449,19 @@ class WebServer(object):
 
     def get_viewer(self, v_id, viewer_class=None, width=512, height=512,
                    force_new=False):
-        from IPython.display import display, HTML
-        v = self.factory.get_viewer(v_id, viewer_class=viewer_class,
-                                    width=width, height=height,
-                                    force_new=force_new)
-        url = v.top.url
-        viewer = v.fitsimage
-        viewer.url = url
-        #display(HTML('<a href="%s">link to viewer</a>' % url))
-        return viewer
+
+        if viewer_class is None:
+            viewer_class = self.default_viewer_class
+
+        v_info = self.factory.get_viewer(v_id, viewer_class=viewer_class,
+                                         width=width, height=height,
+                                         force_new=force_new)
+        return v_info.viewer
 
 
 def make_server(logger=None, basedir='.', numthreads=5,
-                host='localhost', port=9909, use_opencv=False):
+                host='localhost', port=9909, viewer_class=None,
+                use_opencv=False):
 
     if logger is None:
         logger = log.get_logger("ipg", null=True)
@@ -434,10 +481,10 @@ def make_server(logger=None, basedir='.', numthreads=5,
     app = Widgets.Application(logger=logger, base_url=base_url,
                               host=host, port=port)
 
-    factory = ViewerFactory(logger, basedir, app, thread_pool)
+    factory = ViewerFactory(logger, app, thread_pool)
 
     server = WebServer(app, thread_pool, factory,
-                       host=host, port=port)
+                       host=host, port=port, viewer_class=viewer_class)
 
     return server
 
@@ -449,7 +496,9 @@ def main(options, args):
     server = make_server(logger=logger, basedir=options.basedir,
                          numthreads=options.numthreads, host=options.host,
                          port=options.port, use_opencv=options.use_opencv)
+    viewer = server.get_viewer('v1')
 
+    logger.info("Starting server with one viewer, connect at %s" % viewer.url)
     try:
         server.start(use_thread=False)
 
@@ -457,7 +506,7 @@ def main(options, args):
         logger.info("Interrupted!")
         server.stop()
 
-    logger.info("Server terminating...")
+    logger.info("Server terminating ...")
 
 
 if __name__ == "__main__":
